/* brcgi.c
 *
 *  CGI routines (with a few SSIs)
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 *
 */

#include "busterport.h"   /* Praemio */
#include "brnet.h"
#include "ip.h"
#include "brdb.h"
#include "brdns.h"

#include "websys.h"
#include "webio.h"
#include "webfs.h"
#include "wsfdata.h"

#include "busthtml.h"


//ip_addr  br_bugserver = 0x863d6443;  /* 67.100.61.134 for now -JB- */
ip_addr  br_bugserver = 0xD1361145;    /* 69.17.54.209 (net endian) */

/* Flag to permit connections by the localhost only (security). */
extern   int   wi_localhost;

unshort  br_bugport = 626;
static char     bugbuf[1500];

char  br_urlbuf[1500];           /* GUI url handling */


/* Send a bug. name, email and description are in bugfbuf[] */
extern   char *   br_sendbug(void);


int   ht_ssi_cvar(wi_sess * sess, int token);
struct connpush *
new_pushconn(SOCKTYPE socket, struct conn_entry * conn, int index);


extern char * iconcell;


/* scales of background colors from green to red */
char * bgstyles[] =
{
   "green",   /* green */
   "greeny1",
   "greeny2",
   "yellowg2",
   "yellowg1",
   "yellow",   /* YELLOW */
   "yellowo1",
   "yellowo2",
   "orangey2",
   "orangey1",
   "orange",   /* ORANGE */
   "oranger1",
   "oranger2",
   "redo2",
   "redo1",
   "red00",   /* Red */
};

int   load_level[16] = 
{
   0,    /* lowest load - 0 pkts/sec (or bytes * BPP) */
   1,    /* level 1 - 1 pkt/sec */
   2,
   3,
   4,
   5,      /* level 5 */
   8,
   11,
   16,      /* level 8 */
   32,
   64,
   128,
   256,
   512,     /* level 13 */
   1024,
   20000,   /* highest level - 15 - over 1024 pps */
};

int   load_scale = 16;   /* multiplier for adjusting load display colors */

char * icmp_types[] =
{
   "Ping reply",           /* 0 */
   "Bad format",           /* 1 */
   "Bad format",           /* 2 */
   "Destination Unreachable", /* 3 */
   "Source quench ",	      /* 4 */
   "Redirect ",		      /* 5 */
   "Bad format",           /* 6 */
   "Bad format",           /* 7 */
   "Ping request ",	      /* 8 */
   "Bad format",           /* 8 */
   "Bad format",           /* 10 */
   "Time exceeded ",	      /* 11 */
   "Parameter problem ",	/* 12 */
   "Timestamp request ",	/* 13 */
   "Timestamp request ",   /* 14 */
   "Information request ",	/* 15 */
};


/* return a level in the range of 0 - 15 for the passed pps figure. 
 * The returned level is used to index things like the bgstyles 
 * table above.
 */

int
pps_level(int pps)
{
   int i;

   /* Low values of pps does not get scaled. Any more, and we make the 
    * scaling adjustment. By using the default multiplier of 16
    * and hardcoding a division of 16, we get to make fine 
    * adjustments to the load scaling without using floating point math.
    */
   if((pps > 32) && (load_scale != 16))
      pps = (pps * load_scale) >> 4;
   
   for(i = 0; i < 14; i++)
      if(load_level[i] >= pps)
         return (i & 0x0F);

   return 15;
}


/* load level scaler for bytes/sec. Same idea as above, except bytes 
 * instead of pkts. We use a bytes/packet ratio of 128 - more than 
 * that number of bytes/packet on average, and the byte traffic will
 * turn red first; less and the packet number will turn red.
 */

int
bps_level(int bps)
{
   return pps_level((bps >> 7) | (bps & 1));
}


int
br_ipstats(wi_sess * sess, char * args)
{
   char * cp;
   char *   msg;
   u_long   ipaddr;
   u_long   sticks;     /* SNMP-ish ticks. Barf. */
   struct ip_hostentry * iphost;

   msg = wi_formipaddr(sess, "host", &ipaddr);
   if(msg)
   {
      dtrap();
      return -1;
   }
   iphost = br_iplookup(br_ifview, ipaddr);
   if(!iphost)
   {
      dtrap();
      return -1;
   }

   cp = nextarg(args);
   
   switch(*cp)
   {
   case 'p':      /* packets from the host */
      wi_printf(sess, "%lu", iphost->tot_pkts);
      break;
   case 'c':      /* connections with the host */
      wi_printf(sess, "%lu", iphost->conns);
      break;
   case 'd':      /* time we discovered the host */
      sticks = (cticks - iphost->tm_detected)/TPS;
      wi_printf(sess, "%s", print_seconds(sticks) );
      break;
   case 'r':      /* most recent time we saw the host */
      sticks = (cticks - iphost->tm_lastpkt)/TPS;
      wi_printf(sess, "%s", print_seconds(sticks) );
      break;
   default:
      dtrap();
      return -1;
   }
   return 0;
}


/* html_exec() - exec a command from an html file on the local system.
 * 
 * Returns 0 if command was understood by the command parser, else
 * non-zero error. 0 does not mean command succeeded, just that is
 * was parsed OK. non-zero means a definite problem.
 */

int
html_exec(wi_sess * sess, char * args)
{
   int      err;
   char     date[DB_MAXDTL];

   switch(*args)
   {
   case 'h':      /* "host" */
      return(br_hostparm(sess, args + 5));
   case 'v':      /* "version" */
      wi_printf(sess, "%s\n", br_version);
      return 0;
   case 'm':      /* "mode" */
      wi_printf(sess, "%s\n", br_modenames[br_mode] );
      return 0;
   case 'u':      /* "update" */
      err = br_writedatetime(br_dbdate, date, sizeof(date));
      if(err)
         wi_printf(sess, "No date\n");
      else
         wi_printf(sess, "%s\n", date);
      return 0;
   case 'i':      /* ipstat for a remote IP host */
      err = br_ipstats(sess, args);
      return err;
   default:
      dtrap();
   }

   return -1;      /* indicate if command was not found */
}


/* Helper routine for displaying a single connection */

static   int      cnline = 0;    /* count of lines in output */

char * cnkillcell = 
"</td>\n<td><a href=\"%s\" ><img height=20 width=20 border=0 src=\"%s\" ></a>";

void
cn_display1(wi_sess * sess, struct conn_entry * conn)
{
   u_long   traffic;
   u_long   pkts;       /* total in & out packets */
   u_long   bytes;      /* total in & out byte count */
   char *   state;
   char     urltext[64];
   char *   htclass;

   if(cnline++ & 1)    /* select row color, alternating white and tinted */
      htclass = "networktint";
   else
      htclass = "blanktint";

   wi_printf(sess, "<tr class=\"network\" ><td height=1 colspan=9 ></td></tr>");

   state = cn_state(conn);
   wi_printf(sess, "<tr class=\"%s\" ><td>", htclass );  /* start the row */

   wi_printf(sess, "%u.%u.%u.%u", PASS_IPADDR(conn->loc.ce_addr) );
   wi_printf(sess, "<->%u.%u.%u.%u", PASS_IPADDR(conn->rem.ce_addr) );
   wi_printf(sess, "</td>\n<td>%u<->%u", 
       conn->loc.ce_port, conn->rem.ce_port);

   /* print total pkts. Set bgcolor based on packets/second */
   pkts = conn->loc.ce_outpkts + conn->rem.ce_outpkts;
   traffic = (pkts)/(((cticks - conn->ce_start)/TPS) + 1);
   traffic = pps_level(traffic);
   wi_printf(sess, "</td>\n<td class=\"%s\" align=right >%d", bgstyles[traffic], pkts);

   /* print total bytes. Set bgcolor based on packets/second */
   bytes = conn->loc.ce_outbytes + conn->rem.ce_outbytes;
   traffic = bps_level(bytes/(((cticks - conn->ce_start)/TPS) + 1));
   wi_printf(sess, "</td>\n<td class=\"%s\" align=right >%d", bgstyles[traffic], bytes);

   traffic = cticks - conn->ce_start;
   if(traffic <= MINUTE)
      wi_printf(sess, "</td>\n<td>%lu sec.", traffic/TPS);
   else if(traffic <= HOUR)
      wi_printf(sess, "</td>\n<td>%lu min.", (traffic/MINUTE) );
   else
      wi_printf(sess, "</td>\n<td>%lu hours", (traffic/HOUR) );

   wi_printf(sess, "</td>\n<td>%s", discovery_type[conn->ce_type] ); 
   wi_printf(sess, "</td>\n<td>%s", state);

   sprintf(urltext, "connview.htm?index=%d&ptr=%lu", conn->hashx, conn);
   wi_printf(sess, iconcell, urltext, "btnview.gif");

   if(conn->ce_type == H_TCP)
   {
      sprintf(urltext, "javascript:kill_conn( %d, %lu)", conn->hashx, conn );
      wi_printf(sess, cnkillcell, urltext, "btnkill.gif");
   }
   else
      wi_printf(sess, "</td>\n<td bgcolor=#888888 >n/a");

   wi_printf(sess, "</td></tr>\n");
   return;
}

/* cn_compare()
 *
 * Helper routine for conenciton sorting.
 *
 * Returns TRUE if first connection is lower in list than second.
 */

int
cn_compare(struct conn_entry * c1, struct conn_entry * c2, char * sort)
{
   u_long   pkts1, pkts2;     /* local + remote packet totals */

   switch(*sort)
   {
   case 'i':      /* ipaddr */
      if(c1->rem.ce_addr < c2->rem.ce_addr)
         return TRUE;
      else
         return FALSE;
   case 'v':      /* volume of traffic */
      /* test total of packet counts in both directions */
      pkts1 = c1->loc.ce_outpkts + c1->rem.ce_outpkts;
      pkts2 = c2->loc.ce_outpkts + c2->rem.ce_outpkts;

      if(pkts1 < pkts2)
         return TRUE;
      else
         return FALSE;
   case 'p':      /* port */
      if(c1->loc.ce_port < c2->loc.ce_port)
         return TRUE;
      else
         return FALSE;
   case 'a':      /* age */
      if(c1->ce_start < c2->ce_start)
         return TRUE;
      else
         return FALSE;
   case 't':      /* idle time */
      if(c1->ce_time < c2->ce_time)
         return TRUE;
      else
         return FALSE;
   }
   return TRUE;
}


/* ht_cnlist()
 *
 * SSI routine.
 *
 * Returns 0 if OK.
 *
 * Generates table entrys for the headings:
 * IP addresses  Ports  Packets  Bandwidth  State  kill  detail  
*/

int
ht_cnlist(wi_sess * sess, EOFILE * eofile)
{
   int      i;
   int      prottype;
   ip_addr  ipaddr;
   char *   prot;    /* from form */
   char *   sort;    /* from form */
   char *   app;     /* from form */
   char *   host;    /* from form */
   unshort  port;    /* converted form information */
   struct conn_entry *  conn;
   struct conn_entry *  sort_tmp;            /* scratch, for sorting */
   struct conn_entry *  sort_list = NULL;    /* endpoint for sorted list */

   /* See if name/value info came in with the request */
   if(sess->ws_formlist)
   {
      prot = wi_formvalue(sess, "prot");
      if(!prot) prot = "all";
      switch(*prot)     /* Set protocol to filter for */
      {
      case 't':
         prottype = H_TCP;
         break;
      case 'u':
         prottype = H_UDP;
         break;
      case 'i':
         prottype = H_ICMP;
         break;
      case 'a':      /* "all" */
      default:
         prottype = 0;
         break;
      }
      host= wi_formvalue(sess, "host");
      if(!host) 
         ipaddr = 0L;
      else
         ipaddr = inet_addr(host);     /* filter for passed host IP */

      /* get sort type, NULL if unsorted */
      sort = wi_formvalue(sess, "sort");

      /* get port number for which to filter */
      app = wi_formvalue(sess, "application");
      if(app && (*app != '0'))
      {
         port = atoi(app);
      }
      else     /* "application" was not set, check raw port number */
      {
         app = wi_formvalue(sess, "port");
         if(app)
            port = atoi(app);
         else
            port = 0;      /* no port specified, set wildcard */
      }
      /* port number now set; 0 is wildcard */
   }
   else  /* Set variables to clear/wildcard values. */
   {
      prottype = 0;     /* display all types */
      ipaddr = 0L;      /* all hosts */
      sort = NULL;
      port = 0;
   }

   /* loop through connection database */
   for(i = 0; i < HASH_CONN_ENTRIES; i++)
   {
      /* loop through hash bucket */
      for(conn = ce_table[i]; conn; conn = conn->next)
      {
         /* See if we should filter out out own connections */
         if((br_cfg.hide_parm & HP_BUSTER) && 
            (conn->loc.ce_addr == nets[br_ifview]->n_ipaddr) &&
            (conn->loc.ce_port == http_port))
         {
            continue;
         }

         /* filter protocols, 0 is wildcard */
         if(prottype && (conn->ce_type != prottype))
            continue;

         /* filter port number */
         if(port && 
            (conn->loc.ce_port != port) &&
            (conn->rem.ce_port != port))
         {
            continue;
         }

         /* filter IP addresses, 0L is wildcard */
         if(ipaddr && 
            (conn->loc.ce_addr != ipaddr) &&
            (conn->rem.ce_addr != ipaddr))
         {
            continue;
         }

         if(sort == NULL)
         {
            cn_display1(sess, conn);  /* output html to display the connection */
         }
         else
         {
            /* put connection in sort list */
            if((sort_list == NULL) || (cn_compare(conn, sort_list, sort)))
            {
               /* conn goes at head of sort list */
               conn->sort_next = sort_list;
               sort_list = conn;
            }
            else  /* sort list is not null, and conn does not go at head */
            {
               for(sort_tmp = sort_list; 
                  sort_tmp->sort_next; 
                  sort_tmp = sort_tmp->sort_next)
               {
                  if(cn_compare(conn, sort_tmp->sort_next, sort))
                  {
                     /* conn goes between sort_tmp and sort_tmp->sort_next */
                     conn->sort_next = sort_tmp->sort_next;
                     sort_tmp->sort_next = conn;
                     break;
                  }
               }

               /* if we didn't insert, put conn at end of list */
               if(sort_tmp->sort_next == NULL)
               {
                  sort_tmp->sort_next = conn;
                  conn->sort_next = NULL;
               }
            }
         }
      }
   }

   /* if "sort" was set, then sorted list should be ready, if sort 
    * is NOT set then items are already printded out.
    */
   if(sort)
   {
      for(sort_tmp = sort_list; sort_tmp; sort_tmp = sort_tmp->sort_next)
      {
         cn_display1(sess, sort_tmp);
      }
   }

   USE_ARG(eofile);

   return 0; /* OK return */
}



/* ht_cnparms()
 *
 * SSI routine.
 *
 * Print the search parameters used for a connection display
 *
 * Returns 0 if OK. 
 */

int
ht_cnparms(wi_sess * sess, EOFILE * eofile)
{
   char parmbuf[100];
   char * prot;
   char * host;

   parmbuf[0] = 0;      /* start with empty string */

   /* See if name/value info came in with the request */
   if(sess->ws_formlist)
   {
      prot = wi_formvalue(sess, "prot");
      if(prot && strcmp(prot, "all"))
         sprintf(parmbuf, " (<font color=#7700BB>%s</font>)", prot);
      host = wi_formvalue(sess, "host");
      if(host)
         sprintf(&parmbuf[strlen(parmbuf)], 
            " host <font color=#BB0077>%s</font>", host);
   }
   wi_printf(sess, parmbuf);

   USE_ARG(eofile);
   return 0; /* OK return */
}


/* fetch and verify a connection pointer from the passed 
 * hp. the connection is given in index/point format in the
 * sess->ws_formlist.
 *
 * returns pointer to connection if found, else NULL.
 */

struct conn_entry *
cn_verify(wi_sess * sess)
{
   int      cnhashindex;
   char *   cp;
   struct conn_entry *  conn;
   struct conn_entry *  tmp;

   /* get the has table index and connection pointer */
   cp = wi_formvalue(sess, "index");
   if(!cp)
      return NULL;
   cnhashindex = atoi(cp);

   cp = wi_formvalue(sess, "ptr");
   if(!cp)
      return NULL;
   conn = (struct conn_entry *)atoi(cp);

   /* verify the pointer */
   if(cnhashindex > HASH_CONN_ENTRIES)
      return NULL;
   tmp = ce_table[cnhashindex];
   while(tmp)
   {
      if(tmp == conn)
         break;
      tmp = tmp->next;
   }
   if(!tmp)    /* conn not in hash bucket? */
      return NULL;

   return conn;
}



/* killcn_cgi()
 *
 * CGI routine.
 *
 * Kills a connection, indicated by attached name/value index and pointer
 *
 */

char *
killcn_cgi(wi_sess * sess, EOFILE * eofile)
{
   char * referrer;        /* Referring page */
   struct conn_entry * conn;

   conn = cn_verify(sess);
   if(!conn)
   {
      dtrap();
      return "Bad connection";
   }

   /* send resets to both ends of the connection */
   br_tcpreset(conn, 0);
   br_tcpreset(conn, 1);
   conn->ce_state = TCPS_KILLED;

#ifdef NOTDEF
   referrer = wi_formvalue(sess, "referrer");
   if(!referrer)
#endif
      referrer = "connlist.htm";

   /* return browser to the connection page */
   wi_redirect(sess, referrer);
   return NULL;
}



#define bool int

/* config_cgi()
 *
 * CGI routine.
 *
 */

char *
config_cgi(wi_sess * sess, EOFILE * eofile)
{
   int      err;		   /* numeric parse errors */
   long     tcptmo;		/* to be set from form */
   long     udptmo;		/* to be set from form */
   long     icmptmo;		/* to be set from form */
   long     iptmo;		/* to be set from form */
   bool     logtcp;		/* to be set from form */
   bool     logudp;		/* to be set from form */
   bool     logicmp; 	/* to be set from form */
   bool     log_ip;		/* to be set from form */
   char *   sortfield;	/* to be set from form */
   int      refresh;		/* to be set from form */
   int      logsize;		/* to be set from form */

   long     mactmo;		/* to be set from form */
   bool     log_mac;		/* to be set from form */
   char *   delmac;     /* to be set from form */

	/* auto-generated code: */

	err = wi_formint(sess, "tcptmo", (long*)&tcptmo);
	if(err)
	{
		return("bad tcptmo");
	}
	err = wi_formint(sess, "udptmo", (long*)&udptmo);
	if(err)
	{
		return("bad udptmo");
	}
	err = wi_formint(sess, "icmptmo", (long*)&icmptmo);
	if(err)
	{
		return("bad icmptmo");
	}
	err = wi_formint(sess, "iptmo", (long*)&iptmo);
	if(err)
	{
		return("bad iptmo");
	}

   err = wi_formint(sess, "mactmo", (long*)&mactmo);
	if(err)
	{
		return("bad mactmo");
	}

   logtcp = wi_formbool(sess, "logtcp");
	logudp = wi_formbool(sess, "logudp");
	logicmp = wi_formbool(sess, "logicmp");
	log_ip = wi_formbool(sess, "log_ip");
	log_mac = wi_formbool(sess, "log_mac");

   delmac = wi_formvalue(sess, "delmac");
	if(delmac == NULL)
	{
		return("bad delmac");
	}

   if(wi_formbool(sess, "show_local"))
      br_cfg.hide_parm &= ~HP_LOCAL;
   else
      br_cfg.hide_parm |= HP_LOCAL;

   if(wi_formbool(sess, "show_buster"))
      br_cfg.hide_parm &= ~HP_BUSTER;
   else
      br_cfg.hide_parm |= HP_BUSTER;

   if(wi_formbool(sess, "cc_toggle"))
   {
      if(br_cfg.cflags & BC_CONTENT)
         br_cfg.cflags &= ~BC_CONTENT;
      else
         br_cfg.cflags |= BC_CONTENT;
   }
   if(wi_formbool(sess, "hostonly"))
      br_cfg.cflags |= BC_HOSTONLY;
   else
      br_cfg.cflags &= ~BC_HOSTONLY;

   if(wi_formbool(sess, "localonly"))
      wi_localhost = 1;
   else
      wi_localhost = 0;

   sortfield = wi_formvalue(sess, "sortfield");
	if(sortfield == NULL)
	{
		return("bad sortfield");
	}
	err = wi_formint(sess, "refresh", (long*)&refresh);
	if(err)
	{
		return("bad refresh");
	}
	err = wi_formint(sess, "logsize", (long*)&logsize);
	if(err)
	{
		return("bad logsize");
	}

   /* set configuration preferences from the form: */
   br_cfg.keep_tcpconn = tcptmo * MINUTE;
   br_cfg.keep_udpconn = udptmo * MINUTE;
   br_cfg.keep_icmpconn = icmptmo * MINUTE;
   br_cfg.keep_iphost = iptmo * HOUR;
   br_cfg.mactmo = mactmo * 60;

   if(logtcp)
      br_cfg.logevents |= LE_TCPTMO;
   else
      br_cfg.logevents &= ~LE_TCPTMO;

   if(logudp)
      br_cfg.logevents |= LE_UDPTMO;
   else
      br_cfg.logevents &= ~LE_UDPTMO;

   if(logicmp)
      br_cfg.logevents |= LE_ICMPTMO;
   else
      br_cfg.logevents &= ~LE_ICMPTMO;

   if(log_ip)
      br_cfg.logevents |= LE_IPHTMO;
   else
      br_cfg.logevents &= ~LE_IPHTMO;

   if(log_mac)
      br_cfg.logevents |= LE_LOGDELMAC;
   else
      br_cfg.logevents &= ~LE_LOGDELMAC;

   if(*delmac == 'd')
      br_cfg.cflags |= BC_DELIDLE;
   else
      br_cfg.cflags &= ~BC_DELIDLE;

   br_cfg.sortparm = *sortfield;
   br_cfg.screen_refresh = refresh * TPS;
   br_cfg.logsize = logsize;


   /* set the protocol filter partameter bit mask from checkboxes. TCP 
    * is first bit - either set only TCP bit or clear mask 
    */
   if(wi_formbool(sess, "fp_tcp"))
       br_cfg.filterparm = FP_TCP;
   else
       br_cfg.filterparm = 0;

   if(wi_formbool(sess, "fp_udp"))
       br_cfg.filterparm |= FP_UDP;

   if(wi_formbool(sess, "fp_icmp"))
       br_cfg.filterparm |= FP_ICMP;

   /* looks like a winner, let save the br_cfg to nvparams */
   br_saveparms();

   wi_redirect(sess, "saveset.htm");
   return NULL;
}


/* unblock_cgi()
 *
 * CGI routine.
 *
 * Handle the form submission when a user wants to unblock a filtered
 * Item. The unblock may be a permanent deletion, a "whitelist", or
 * a timed access. The item may be IP address, DNS name, or URL path.
 *
 */

char *
unblock_cgi(wi_sess * sess, EOFILE * eofile)
{
   int      err;        /* numeric parse errors */
   char *   msg;		   /* IP address parse error msg */
   char *	blkpath;	   /* path to unblock */
   char *   blkname;    /* name to unblock */
   char *	timing;		/* to be set from form */
   unsigned minutes;		/* to be set from form */
   char *   unblock;    /* set from form */
   void *   object;     /* controlled object */
   ip_addr	blkip = 0L; /* IP address to unblock */
   struct   ctdesc * ctl = NULL;    /* use NULL as "unset" flag */
   char *   referral;
   struct b_path *      bpath;
   struct control_ip *  cip;
   struct control_dn *  dnp;
   struct conn_entry *  conn;
   enum unblock_op { REMOVE, BLACKLIST, WHITELIST, TIMEDACCESS } access;

   minutes = 0;     /* default 0 == access forever (or no timer) */

   timing = wi_formvalue(sess, "timing");
	if(timing == NULL)
	{
		return("bad timing");
	}
	err = wi_formint(sess, "minutes", (long*)&minutes);
	if(err)
	{
		return("no 'minutes' field");
	}
	conn = get_form_conn(sess);

   /* The "timing" control may be "timed", "perm", or "block" */
   if(strcmp(timing, "block") == 0)
      access = BLACKLIST;
   else if(strcmp(timing, "perm") == 0)
      access = REMOVE;
   else if(strcmp(timing, "remove") == 0)
      access = REMOVE;
   else if(strcmp(timing, "timed") == 0)
   {
      access = TIMEDACCESS;
      minutes &= 0x000007FF;     /* limit to reasonable value */
   }
   else
   {
      dtrap();
      goto unblock_error;
   }

   /* At least ONE of the following controls OR the connection 
    * should be set. 
    */
   blkpath = wi_formvalue(sess, "blkpath");
   blkname = wi_formvalue(sess, "blkname");
   unblock = wi_formvalue(sess, "unblock");
	referral = wi_formvalue(sess, "referral");

   blkip = 0;
   msg = wi_formipaddr(sess, "blkhost", &blkip);

   /* If the connection is set then extract the DB record for the 
    * blocked item, set the lcal variable, and jump (goto) the case
    * statement logic for that kind of item. If conn is not set then
    * blkip, blkname or blkpath should be set.
    */
   if(conn && conn->block.ptr)
   {
      switch(conn->block.type)
      {
      case CTF_TYPEIP:     /* blocked because of IP address */
         cip = (struct control_ip *)conn->block.ptr;
         goto got_cip;
      case CTF_TYPEPORT:   /* TCP or UDP port number */
         dtrap();
         break;
      case CTF_TYPENAME:   /* DNS name */
         dnp = (struct control_dn *)conn->block.ptr;
         goto got_dnp;
      case CTF_TYPEPATH:   /* path/name potion of URL */
         bpath = (struct b_path *)conn->block.ptr;
         goto got_bpath;
      case CTF_TYPEWORD:   /* Web content word */
         dtrap();
         break;
      case CTF_TYPESIG:    /* Binary signature (virus) */
         dtrap();
         break;
      default:
         dtrap();
         return "unknown reason";
      }
   }

   if((blkip != 0) && (msg == NULL))   /* badserver IP address is set */
   {
      cip = ipe_lookup(htonl(blkip));    /* lookup IPs block record */
got_cip:
      if(!cip)
         goto unblock_error;
      object = cip;
      ctl = &cip->ctl;
      if(access == REMOVE)
      {
         del_ipe(cip, TRUE);
         goto unblock_done;
      }
   }
   else if(blkpath && *blkpath)  /* blocked path may be set.*/
   {
      bpath = path_lookup(blkpath);
got_bpath:
      if(!bpath)
         goto unblock_error;
      object = bpath;
      ctl = &bpath->ctl;
      if(access == REMOVE)
      {
         del_path(bpath);
         goto unblock_done;
      }
   }
   else if(blkname && *blkname)  /* blocked domain name may be set.*/
   {
      dnp = dn_lookup(blkname);
got_dnp:
      if(!dnp)
         goto unblock_error;
      object = dnp;
      ctl = &dnp->ctl;
      if(access == REMOVE)
      {
         TH_LOCK_MUTEX(br_dns_mutex);
         del_dname(dnp);
         TH_UNLOCK_MUTEX(br_dns_mutex);
         goto unblock_done;
      }
   }
   else
   {
unblock_error:
      dtrap(); /* fubar */
      return("unblock.cgi: no data");
   }

   /* Fall to here if access != REMOVE. ctl should have been 
    * set by the logic above. If a time is specified, set timer; 
    * else add ctl's parent to whitelist 
    */
   if(!ctl)
   {
      dtrap();
      goto unblock_error;
   }

   if(access == BLACKLIST)
      ctl_setaccess(ctl, 0, object);  /* No access */
   else if(access == WHITELIST)
      ctl_setaccess(ctl, -1, object);   /* add to whitelist by setting flags */
   else if(access == TIMEDACCESS)
      ctl_setaccess(ctl, minutes, object);  /* Access for limited time */
   else  
      dtrap();


unblock_done:

   /* Set form result page. */
   if(referral && (strstr(referral, "unblock.htm")))
   {
     wi_redirect(sess, "unblocked.htm");
   }
   else
   {
      /* reload the last file which lead the user to ths form */
      if(blkip)
         wi_redirect(sess, "bipaddr.htm");
      else if(blkpath && *blkpath)
         wi_redirect(sess, "bfiletp.htm");
      else
         wi_redirect(sess, "urls.htm");
   }

   return NULL;
}


/* sendbug_cgi()
 *
 * CGI routine.
 *
 * Reports a bug from web server form.
 *
 */

char *
sendbug_cgi(wi_sess * sess, EOFILE * eofile)
{
   char *   errmsg;
   char *   cp;
   char *	username;   /* to be set from form */
   char *	email;	   /* to be set from form */
   char *	problem;	   /* to be set from form */
   int      len;

   /* auto-generated code: */

	username = wi_formvalue(sess, "username");
	if(username == NULL)
	{
		return("bad username");
	}
	email = wi_formvalue(sess, "email");
	if(email == NULL)
	{
		return("bad email");
	}
	problem = wi_formvalue(sess, "problem");
	if(problem == NULL)
	{
		return("bad problem");
	}

   cp = problem;
   while(*cp)
   {
      if(*cp == '+')
         *cp = ' ';
      cp++;
   }
   sprintf(bugbuf, "%s bug report, v%u.%u.%u.%u  User Name: ", 
      br_appname, PUSH_VERSION(br_versnum), email);

   strcat(bugbuf, username);
   strcat(bugbuf, "\r\nEmail: ");
   strcat(bugbuf, email);

   cp = &bugbuf[strlen(bugbuf)];
   len = strlen(problem);
   sprintf(cp, "\r\nProblem; Length=%d\r\n", len);
   strcat(cp, problem);
   strcat(cp, "\r\n");

   errmsg = br_sendbug();
   if(errmsg)
   {
      br_log(0, "error \"%s\" connecting to %u.%u.%u.%u for bug report\n",
         errmsg, PASS_IPADDR(br_bugserver) );
      return(errmsg);
   }

   return("Your report has been received by our server.<br><br>"
          "<center><input type=button onClick='window.close()' "
          "value='Close Window'>");
}



/* br_sendbug() - Try to send a bug report.
 *
 * Parameters are email address and problem decription
 *
 * Returns NULL if OK, else text describing error.
 */

char *
br_sendbug()
{
   int      err;
   int      len;
   FILE *   fp;         /* for reading our system files */
   SOCKTYPE bugsock;    /* connection to bug report server */
   struct sockaddr_in bugaddr;
   char *   senderr = "error sending bug report";

   /* move this to another thread and add code to handle delays later -JB- */
   bugsock = socket(AF_INET, SOCK_STREAM, 0);
   if(bugsock == INVALID_SOCKET)
      return "Unable to get connection for bug report\n";

   bugaddr.sin_addr.s_addr = br_bugserver;
   bugaddr.sin_port = htons(br_bugport);
   bugaddr.sin_family = AF_INET;
   err = connect(bugsock, (struct sockaddr *)&bugaddr, sizeof(bugaddr) );
   if(err  == -1)
      return "failed to connect to bug report server";

   err = send(bugsock, bugbuf, strlen(bugbuf), 0);
   if(err != (int)strlen(bugbuf))
      return senderr;

   fp = fopen("webport.nv", "rb");
   len = ftell(fp);
   sprintf(bugbuf, "\r\nWebport.nv; Length=%d\r\n", len);
   err = send(bugsock, bugbuf, strlen(bugbuf), 0);
   if(err != (int)strlen(bugbuf))
      return senderr;
   while( (len = fread(bugbuf, 1, sizeof(bugbuf), fp)) > 0)
   {
      err = send(bugsock, bugbuf, len, 0);
      if(err != len)
         return senderr;
   }
   fclose(fp);
      
   fp = fopen("buster.log", "rb");
   len = ftell(fp);
   sprintf(bugbuf, "\r\nbuster.log; Length=%d\r\n", len);
   err = send(bugsock, bugbuf, strlen(bugbuf), 0);
   if(err != (int)strlen(bugbuf))
      return senderr;
   while( (len = fread(bugbuf, 1, sizeof(bugbuf), fp)) > 0)
   {
      err = send(bugsock, bugbuf, len, 0);
      if(err != len)
         return senderr;
   }
   fclose(fp);

   /* Append the log file */
   fp = fopen("busterbk.log", "rb");

   fseek(fp, 0L, SEEK_END);    /* seek to end of file */
   len = ftell(fp);
   fseek(fp, 0L, SEEK_SET);    /* restore file pointer to beginning */

   sprintf(bugbuf, "\r\nbusterbk.log; Length=%d\r\n", len);
   err = send(bugsock, bugbuf, strlen(bugbuf), 0);
   if(err != (int)strlen(bugbuf))
      return senderr;
   while( (len = fread(bugbuf, 1, sizeof(bugbuf), fp)) > 0)
   {
      err = send(bugsock, bugbuf, len, 0);
      if(err != len)
         return senderr;
   }
   fclose(fp);
   socketclose(bugsock);

   return NULL;      /* good return */
}



/* ht_whois()
 *
 * SSI routine.
 *
 * Returns 0 if OK. 
 */

int
ht_whois(wi_sess * sess, EOFILE * eofile)
{
   char *   msg;
   ip_addr  ipaddr;
   char *   whois;
   char *   cp;
   int      iptype;
   int      who;

   msg = wi_formipaddr(sess, "host", &ipaddr );
   if(msg)
   {
      wi_printf(sess, "Internal problem: %s", msg);
      return -1;
   }
   iptype = wi_formbool(sess, "iptype");

   /* Run whois session, get result  */
   who = br_whois(ipaddr, &msg, iptype);

   if(who < 0)
   {
      if(!*msg)
      {
         dtrap();    /* bad whois programming */
         return -1;
      }
      wi_printf(sess, "Whois error: %s", msg);
      return 0;
   }

   /* Send whois buffer it to the socket, replacing text CR-LF 
    * ("\r\n") with html "<br>". 
    */
   whois = whoservers[who].whobuf;

   wi_printf(sess, "From whois server: %s<br>", whoservers[who].dnsname );

   /* skip past leading whitespace */
   while(*whois && (*whois <= ' '))
      whois++;

   cp = whois;
   while(*cp)
   {
      if((*cp == '\r') || (*cp == '\n'))
      {
         *cp++ = 0;
         if((cp - whois) < 500)
         {
            wi_printf(sess, "%s<br>", whois);
         }
         else     /* whois string is too long for wi_printf */
         {
            /* check here for SSL socket later??? */
            wi_printf(sess, whois);
            wi_printf(sess, "<br>");
         }
         if((*cp == '\r') || (*cp == '\n'))
            cp++;
         whois = cp;
      }
      cp++;
   }

	return 0; /* OK return */
}

/* ht_cmntyinfo()
 *
 * SSI routine.
 *
 * Returns 0 if OK. 
 */

int
ht_cmntyinfo(wi_sess * sess, EOFILE * eofile)
{
	/* add your code here*/
      USE_ARG(sess);
      USE_ARG(eofile);

	return 0; /* OK return */
}


/* netbar_cgi()
 *
 * CGI routine.
 *
 * Call when user clicks one of the button one the network page 
 * topbar (e.g. "buster_activity_displayed" or "lan_activity_displayed"
 *
 */

char *
netbar_cgi(wi_sess * sess, EOFILE * eofile)
{
   bool	lan_activity_displayed;		/* to be set from form */
   bool	buster_activity_displayed;		/* to be set from form */

	/* auto-generated code: */
	lan_activity_displayed = wi_formbool(sess, "lan_activity_displayed");
	buster_activity_displayed = wi_formbool(sess, "buster_activity_displayed");

   /* Set any changes indicated by the checkbox booleans */
   if(lan_activity_displayed)
      br_cfg.hide_parm &= ~HP_LOCAL;
   else
      br_cfg.hide_parm |= HP_LOCAL;

   if(buster_activity_displayed)
      br_cfg.hide_parm &= ~HP_BUSTER;
   else
      br_cfg.hide_parm |= HP_BUSTER;

   return("netbar.htm");
}


/* Opposite of print_eth() - read all print_eth outputs. 
 *
 * Returns 0 if input string OK, else -1
 *
 */

extern int ishexdigit(char cp);

int
read_eth(char * out_binary, char * in_text)
{
   int      hexchars;   /* hex chars read from input */
   int      outbytes;   /* bytes done in output (0-5) */
   char *   cp;         /* scratch */
   u_char   nibble;
   char     delimiter;  /* delimiter char (eg ' ' or '-'), 0 if none */

   /* See if the third char is a delimiter (xx-xx-xx-xx-xx-xx) or address is 
    * pure hex (xxxxxxxxxxxx) 
    */

   if(ishexdigit(*(in_text + 2)) == FALSE)
      delimiter = (char)*(in_text + 2);
   else
      delimiter = 0;

   hexchars = outbytes = 0;

   for(cp = in_text; *cp; cp++)
   {
      /* All chars must be hex digit or delimited */
      if(!ishexdigit(*cp))
      {
         if(delimiter && (*cp == delimiter))
            continue;

         /* else */
         dtrap();
         return -1;
      }

      nibble = (u_char)(*cp);
      if(nibble <= '9')
         nibble -= '0';
      else if(nibble > 'a')  /* lowercase letter? */
         nibble -= ('a' - 10);
      else              /* uppercase letter */
         nibble -= ('A' - 10);

      if(hexchars & 1)
         out_binary[outbytes++] += nibble;
      else
         out_binary[outbytes] = (nibble << 4);

      hexchars++;
   }

   return 0;   /* OPK return */
}

/* hostname_cgi()
 *
 * CGI routine.
 *
 * Called from form to assign/change name or image 
 * of a local MAC host
 *
 * 
 */

char *
hostname_cgi(wi_sess * sess, EOFILE * eofile)
{
   char *   newname;		/* to be set from form */
   char *   imgname;		/* to be set from form */
   struct ip_hostentry * iphost;
   struct host_entry * machost;

   /* user could be setting name, image, or both */
   newname = wi_formvalue(sess, "newname");
	imgname = wi_formvalue(sess, "imgfile");

   if((newname == NULL) && (imgname == NULL))
   {
      return("Must select new name or image");
   }

   if(br_formhost(sess, &machost, &iphost))
      return("Host lookup failed");

   if(newname && *newname)
   {
      strncpy(machost->locname, newname, MAXDNSNAME);
      machost->h_flags &= ~H_NAMEMASK; /* clear old name type bits */
      machost->h_flags |= H_USERNAME;  /* set new name type */
   }
   if(imgname && *imgname)
   {
      strncpy(iphost->imagefile, imgname, MAXFILENAME);
      strncpy(machost->imagefile, imgname, MAXFILENAME);
   }

   /* This is new info, so trigger a LAN DB write */
   brstates[br_ifview].evflags |= EV_LANSTALE;

   sprintf(br_urlbuf, "machost.htm?host=%s", 
      html_mac(machost->macaddr) );
   wi_redirect(sess, br_urlbuf);

   return(NULL);
}


/* imgload_cgi()
 *
 * CGI routine.
 *
 * routine to handle new MAC image upload completion.
 *
 */

extern char * br_recvimage(struct httpd * hp, char * filename);

char *
imgload_cgi(wi_sess * sess, EOFILE * eofile)
{
   char *   upfile;    /* to be set from form */

   /* auto-generated code: */

   upfile = wi_formvalue(sess, "filename");
   if(upfile == NULL)
   {
      USE_ARG(eofile);
      return("No upload file specified");
   }

   wi_redirect(sess, "pixsel.htm");
   return NULL;
}

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


