/* brpush.c
 *
 * Buster Web server "push" routines.
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 *
 * ROUTINES: ht_showkillcn(), killcn_cgi(), ht_conn1(), new_pushconn(), 
 * ROUTINES: sendwait(), pushconn_tmr(), pushcn_open(), del_pushconn(), 
 */

#include "busterport.h"   /* Praemio */
#include "brnet.h"
#include "ip.h"
#include "brdb.h"
#include "brdns.h"

#include "websys.h"
#include "webio.h"
#include "webfs.h"
#include "wsfdata.h"

long connpush_timer;
void  pushconn_tmr(long parm);   /* timer handler */

extern   struct conn_entry *  cn_verify(wi_sess * sess);
extern   void  cn_display1(wi_sess * sess, struct conn_entry * conn);



/* sendwait() - a utility routine to send lots of odd text bits into a socket,
 * and wait during brief blocks. Usfull for the server-push sockets. This
 * is an approximate replacement for ht_blockingsend(), which cannot be 
 * used on server-push sockets since the httpd struct is usually closed 
 * long before the push socket is.
 */

static int
sendwait(SOCKTYPE so, char * data)
{
   u_long   tmo;        /* send timeout */
   int err;
   int datalen = strlen(data);

   tmo = cticks + (30 * TPS);
   while(tmo > cticks)
   {

#ifdef HT_PERSISTENT
      /* See if we're hurling chunks */
#endif   /* HT_PERSISTENT */

      err = send(so, data, datalen, 0);
      if(err == datalen)
         return 0;   /* good return  sent all data */

      TH_SLEEP(1);    /* let system run a bit... */

      if(err > 0)    /* sent some data but not all */
      {
         data += err;      /* remove what we sent */
         datalen -= err;
         continue;         /* try some more */
      }
      /* fall to here if err < 0 (serious) */
      err = sock_errno(so);
      if(err != EWOULDBLOCK)
         break;
   }
   return 1;   /* error or timeout */
}

/*************** Connction monitor push code *****************/


/* pushcn_open() - Push notification routine for the connection 
 * detail page. 
 *
 * Returns 0 if OK. 
 */

int
pushcn_open(wi_sess * sess, EOFILE * eofile)
{
   dtrap();
#ifdef NOTDEF
   /* if timer is idle, start it up */
   if(connpush_timer == 0)
      connpush_timer = in_timerset(pushconn_tmr, 200, 0L);
   if(connpush_timer == 0)
   {
      dtrap();
      return -1;
   }
#endif
   return 0; /* OK return */
}


/* structures & code to support server push to a list of open connection
 * detail wiondows.
 */

/* to push data to a connection detail page, we need the connection, the 
 * socket, and the index (for verification). The following structure 
 * saves this info in a linked list.
 */

struct connpush 
{
   struct connpush *    next;
   SOCKTYPE             sock;
   struct conn_entry *  conn;
   int                  index;
   /* data counts last time we pushed an update */
   u_long               client_pkts;
   u_long               server_pkts;
};

struct connpush * pushconns;

/* connpush constructor */

struct connpush *
new_pushconn(SOCKTYPE socket, struct conn_entry * conn, int index)
{
   struct connpush * newpc;

   newpc = (struct connpush *)npalloc(sizeof(struct connpush));
   if(!newpc)
      return NULL;

   newpc->conn = conn;
   newpc->index = index;
   newpc->sock = socket;
   newpc->next = pushconns;
   pushconns = newpc;

   return(newpc);
}

/* connpush destructor */

void
del_pushconn(struct connpush * delpc)
{
   struct connpush * tmp;
   struct connpush * last;

   last = NULL;
   for(tmp = pushconns; tmp; tmp = tmp->next)
   {
      if(tmp == delpc)
      {
         if(last)
            last->next = delpc->next;  /* unlink */
         else
            pushconns = delpc->next;
         socketclose(delpc->sock);
         npfree(delpc);
         return;
      }
      last = tmp;
   }
   dtrap();    /* delpc not found in list */
}

/* helper routine for ht_conn1 */

void
ht_cnport(wi_sess * sess, 
          struct endpoint * node, 
          struct conn_entry * conn)
{
   char * portname;

   wi_printf(sess, "<td bgcolor=#FFFFFF >");
   if(conn->ce_type == H_ICMP)
   {
      struct icmp * icmp;
      icmp = (struct icmp *)(node->app);
      wi_printf(sess, "(%s)", icmp_types[icmp->type] );
   }
   else
   {
      wi_printf(sess, " port %d", node->ce_port );
      portname = br_portname(node->ce_port);
      if(portname)
         wi_printf(sess, "<br>(%s)", portname);
   }
   wi_printf(sess, "</td>");
   return;
}

/* ht_conn1() - SSI routine.
 * 
 * Handles first portion  of connection detail window.
 * Draws table contents for connection traffic display
 *
 * Returns 0 if OK.
 */

int
ht_conn1(wi_sess * sess, EOFILE * eofile)
{
   struct conn_entry *  conn;
   struct endpoint * client;
   struct endpoint * server;
   struct ip_hostentry * iphost;
   int      rows, cols;
   char *   portname;      /* remote port name */
   char *   remotename;    /* remote host name */
   long     cnindex;
   
   conn = cn_verify(sess);   /* fetch and verify the pointer */
   if(!conn)
   {
      wi_printf(sess, "</tr></td><tr><td> Connection has timed out.</td></tr>");
      return 0;
   }
   wi_formint(sess, "index", &cnindex);

   if(conn->ce_flags & CEF_LOCALCLIENT)
   {
      client = &conn->loc;
      server = &conn->rem;
   }
   else
   {
      client = &conn->rem;
      server = &conn->loc;
   }

   wi_printf(sess, "<tr><td bgcolor=#FFFFFF ><a href=\"machost.htm?host=%u.%u.%u.%u\" >", 
      PASS_IPADDR(client->ce_addr) );
   wi_printf(sess, "%u.%u.%u.%u <br>", PASS_IPADDR(client->ce_addr) );
   iphost = br_iplookup(conn->ce_iface, client->ce_addr);
   wi_printf(sess, "%s <br>",  br_getname(iphost) );

   /* Generate a 2 x 16 table with a named image in each cell. This 
    * will serve as the objects for javascript to display net acticity.
    */
   wi_printf(sess, "</td><td>\n<table><tr>");
   for(rows = 0; rows < 2; rows++)
   {
      for(cols = 0; cols < 16; cols++)
      {
         wi_printf(sess, 
            "\n   <td class=\"networktint\" > <img src=\"greyled.gif\" name=\"led%d\" width=8, height=14 ></td>",
            (rows * 16) + cols );
      }
      if(rows == 0)
         wi_printf(sess, " \n</tr><tr>");      /* start second row */
   }
   wi_printf(sess, "\n  </tr></table></td>\n");

   wi_printf(sess, "<td ><input type=\"text\" size=9 align=right name=\"client_bytes\" value=\"%d\" ><br>", 
      client->ce_outbytes);

   wi_printf(sess, "<input type=\"text\" size=9 align=right name=\"server_bytes\" value=\"%d\" ></td>\n", 
      server->ce_outbytes);

   wi_printf(sess, "<td><input type=\"text\" size=9 align=right name=\"client_pkts\" value=\"%d\" ><br>", 
      client->ce_outpkts);

   wi_printf(sess, "<input type=\"text\" size=9 align=right name=\"server_pkts\" value=\"%d\" ></td>\n", 
      server->ce_outpkts);


   /* Print out cell with server IP address & (maybe) name */
   wi_printf(sess, "<td bgcolor=#FFFFFF > <a href=\"hostinfo.htm?host=%u.%u.%u.%u\" >", 
      PASS_IPADDR(server->ce_addr) );
   wi_printf(sess, "%u.%u.%u.%u <br>", PASS_IPADDR(server->ce_addr) );

   iphost = br_iplookup(conn->ce_iface, server->ce_addr);
   remotename = br_getname(iphost);
   wi_printf(sess, "%s <br></td></tr><tr>\n", remotename);

   /* NEW ROW: print cell for client port numbers and types */
   ht_cnport(sess, client, conn);

   /* print a "kill" button to kill TCP connections */
   if(conn->ce_type == H_TCP)
   {
      wi_printf(sess, "\n<td colspan=3 class=\"networklt\" >"
         "<a href=\"javascript:are_you_sure(%d, %lu)\">"
         "<table align=center bgcolor=#FFFFFF ><tr><td>",
         cnindex, conn );

      wi_printf(sess, "Kill this connection NOW: </td>"
         "<td class=\"network\"><img height=20 width=20 border=0 src=\"btnkill.gif\" >"
         "</td></tr></table></a></td>");
   }
   else
   {
      wi_printf(sess, "\n<td colspan=3>&nbsp;</td>\n");
   }

   /* print cell for server port numbers and types.  */
   ht_cnport(sess, server, conn);
   wi_printf(sess, "</tr>\n</table>");


   /* end of top table, change rows in outer table and start center table */

   wi_printf(sess, "</td></tr>\n<tr><td align=center><table border=0 width=90%c  >", '%');

   wi_printf(sess, "<tr><td>age:</td><td><input type=text name=\"age\" value=\"%s\" >\n",
      cn_age(cticks - conn->ce_start) );
   wi_printf(sess, "</td>\n<td width=20></td><td>type:</td>\n<td>"
      "<input type=text name=\"ce_type\" value=\"%s\" >",
         discovery_type[conn->ce_type] );
   wi_printf(sess, "</td></tr>\n");

   wi_printf(sess, "<tr><td>idle:</td><td><input type=text name=\"idle\" value=\"%s\" >\n",
      cn_age(cticks - conn->ce_time) );
   wi_printf(sess, "</td>\n<td></td><td>state:</td>\n<td><input type=text name=\"state\" value=\"%s\" >", 
      cn_state(conn) );

   /* Peak speed and latency for TCP connections */
   if(conn->ce_type == H_TCP)
   {
      wi_printf(sess, "<tr><td>RTT:</td><td title=\"Est. Round trip time in microseconds\">"\
         "<input type=text name=\"speed\" value=\"%d\" >\n",
         conn->ce_rtt );
   }
   else
   {
      wi_printf(sess, "<tr><td>RTT:</td><td>"\
         "<input type=text name=\"speed\" value=\"N/A\" >\n");
   }

   wi_printf(sess, "</td>\n<td></td><td>hops:</td>\n"\
      "<td title=\"IP Routes remaining\">"\
      "<input type=text name=\"state\" value=\"%d\" >", 
      conn->ce_hops );

   wi_printf(sess, "</td></tr></table>\n");

   /* end of "Age/idle" table, next is optional content table (URL, DNS query, etc.) */

   if(conn->ce_type == H_ICMP)
   {
      struct icmp * icmp;
      char * type;
      
      if(server->ce_outpkts >= client->ce_outpkts)
         icmp = (struct icmp *)(server->app);
      else
         icmp = (struct icmp *)(client->app);

      if(icmp->type > MAX_ICMP_TYPE)
         type = icmp_types[1];
      else
         type = icmp_types[icmp->type];

      wi_printf(sess, "<table border=1 class=\"networktint\" >\n <tr><td><font face='arial'> &nbsp;");

      if((icmp->type == ECHOREQ) || (icmp->type == ECHOREP))
         wi_printf(sess, "last pkt: %s, Id: %d, seq: %d", 
            type, icmp->id, icmp->seq );
      else
         wi_printf(sess, "last pkt type %d (%s)", 
            icmp->type, type );

      wi_printf(sess, "&nbsp; </td>\n </tr></table><br>\n");

   }
   else if(conn->data[0])       /* is there any saved data? (e.g. URL) */
   {
      wi_printf(sess, "<table border=1 class=\"networktint\" >\n <tr><td> &nbsp;");
      if(conn->ce_type == H_TCP)
      {
         if(conn->rem.ce_port == 80)   /* print a URL row into the display */
         {
            /* conn->data may be long (for ht_printf), so use two calls */
            wi_printf(sess, "URL: </td><td> <a href=%s > ", conn->data);
            wi_printf(sess, "%s </a> \n", conn->data);

            if(filter_web)    /* Are we doing content control? */
            {
               wi_printf(sess, "</td></tr><tr><td>Content score: </td><td>"
                  "%d points in %d words</td>",
                  conn->ce_weight, conn->ce_wcount );
            }
         }
      }
      else if(conn->ce_type == H_UDP)
      {
         if(conn->rem.ce_port == 53)   /* print DNS info row into the display */
            wi_printf(sess, "DNS:</td><td>%s", conn->data);
      }
      wi_printf(sess, "&nbsp; </td>\n </tr></table><br>\n");
   }

   /* end of second table, change rows in outer table and optional controls table */

   wi_printf(sess, "</td></tr>\n<tr><td>");

   /* if conn is TCP, make kill & blacklist button matrix */
   if(conn->ce_type == H_TCP)
   {
      ip_addr  host;
      ip_addr  ips[3];     /* table of IP hosts */
      long     ports[5];     /* Table of ports - high bit is direction */

      /* first paste up the dummies version of port & IP blocking */
      wi_printf(sess, 
         "<center><table border=1 class=\"networktint\" ><tr><td> "
         "<a href='blist_ip.htm?host=%u.%u.%u.%u'> ",
            PASS_IPADDR(server->ce_addr) );

      portname = br_portname(server->ce_port);
      remotename = br_getnametext(server->ce_addr);
      wi_printf(sess, "Click to block server %s </a> </td>\n<td> <a href='blist_ip.htm?port=%d'> "
         "Click to block port %u(%s) </a> </td></tr></table><br>\n",
            remotename, server->ce_port, 
            server->ce_port, portname?portname:"(unknown)");

      /* set up tables for the nested for() loops */
      ips[0] = client->ce_addr;
      ips[1] = server->ce_addr;
      ips[2] = 0L;
      ports[0] = client->ce_port;
      ports[1] = server->ce_port;
      ports[2] = client->ce_port | 0x80000000;
      ports[3] = server->ce_port | 0x80000000;
      ports[4] = 0;       /* all ports */

      wi_printf(sess, "<center><table  width=100%c  border=0 >\n"
         "<tr><td colspan=6 align=center bgcolor=#FFFFFF >"
         "Expert Internet blocking panel - only use these "
         "buttons if you know what you're doing! <br>\n"
         "Click to block the indicated port/IP combinations</td></tr>", '%');

      for(rows = 0; rows < 3; rows++)
      {
         host = ips[rows];
         wi_printf(sess, "<tr>");
         for(cols = 0; cols < 6; cols++)
         {
            unshort  port;
            char * direction;
            if(cols == 0)
            {
               wi_printf(sess, "<tr><td>");
               if(ips[rows] == 0)
                  remotename = "All computers ";
               else
                  remotename = br_getnametext(ips[rows]);
               wi_printf(sess, "%s</td>", remotename );
               continue;
            }
            if((rows == 2) && (cols == 5))
               continue;      /* Don't make a "block everything" button. Ouch! */

            /* fall to here for columns after first */
            direction = (ports[cols-1] & 0x80000000)?"from":"to";
            port = (unshort)(ports[cols-1] & 0x0000FFFF);
            if(port == 0)
               portname = "All ports";
            else
               portname = br_portname(port);
            if(rows == 2)
            {
               wi_printf(sess, "<td title=\"Block port %u (%s) connections to/from all computers\">",
                  port, portname?portname:"unassigned");
            }
            else
            {
               wi_printf(sess, "<td title=\"Block connections %s port %u (%s) on computer %s\">",
                  direction, port, portname?portname:"unassigned", remotename);
            }

            if(port)
            {
               wi_printf(sess, "<input type=button name='btn%d' value='%s port %d'",
                  (cols + (10 * rows)), direction, port );
               wi_printf(sess, " onClick=\"location.href='blist_pt.htm?host=%u.%u.%u.%u&port=%d';\" class=\"button networklt\" ></td>", 
                  PASS_IPADDR(ips[rows]), port);
            }
            else
            {
               wi_printf(sess, "<input type=button name='btn%d' value='all ports' "
                  " onClick=\"location.href='blist_ip.htm?host=%u.%u.%u.%u&port=0';\"  class=\"button networklt\" ></td>", 
                  (cols + (10 * rows)), PASS_IPADDR(ips[rows]) );
            }
         }
         wi_printf(sess, "</tr>");
      }
   }

   wi_printf(sess, "</table>");

   /* If no push connection is required (i.e. connection already closed)
    * then return now.
    */
   if((conn->ce_type == H_TCP) &&
      (conn->ce_state >= 9))        /* 9 is TCPS_FIN_WAIT_2 state */
   {
      return 0;
   }

   /* Create the entry for the server push. */
   new_pushconn(sess->ws_socket, conn, (int)cnindex);

   return 0; /* OK return */
}



/* Timer driven push routine for the connection detail view. 
 * this pumps javascript into the pipe to update the
 * connection detail Page.
 *
 * Parameter is http conn. conn->form has connection info. 
 */

char * startscript = "<SCRIPT language=JavaScript>\n";
char * stopscript = "</SCRIPT>\n";

void
pushconn_tmr(long parm)
{
   struct connpush * pc;
   struct conn_entry * conn;
   int      level;
   char     jstext[64];
   struct endpoint * client;
   struct endpoint * server;

   if(pushconns == NULL)      /* no push connections? */
   {
#ifdef NOTDEF
      in_timerkill(connpush_timer);     /* stop the interval timer */
#endif
      connpush_timer = 0;
      return;
   }

   for(pc = pushconns; pc; pc = pc->next)
   {
      /* verify connection is still in database */
      for(conn = ce_table[pc->index]; conn; conn = conn->next)
      {
         if(conn == pc->conn)
            break;
      }
      if(!conn)   /* connection timed out? */
      {
         /* clean up screen for close */
         sendwait(pc->sock, startscript);
         sendwait(pc->sock, "document.conn.state.value=\"CLOSED\";");
         sendwait(pc->sock, "client_traffic(0); server_traffic(0);\n");
         sendwait(pc->sock, stopscript);
send_err:
         del_pushconn(pc);    /* delete entry for closed connection */
         return;              /* return to avoid list problems */
      }

      if(conn->ce_flags & CEF_LOCALCLIENT)
      {
         client = &conn->loc;
         server = &conn->rem;
      }
      else
      {
         client = &conn->rem;
         server = &conn->loc;
      }
 
      /* send connection update via push socket */
      if(sendwait(pc->sock, startscript))
         goto send_err;

      /* Update LED row displays if packet counts have changed */
      level = pps_level(client->ce_outpkts - pc->client_pkts);
      sprintf(jstext, "client_traffic(%d);\n", level);
      if(sendwait(pc->sock, jstext))
         goto send_err;

      level = pps_level(server->ce_outpkts - pc->server_pkts);
      sprintf(jstext, "server_traffic(%d);\n", level);
      if(sendwait(pc->sock, jstext))
         goto send_err;

      /* Save the current count values */
      pc->client_pkts = client->ce_outpkts;
      pc->server_pkts = server->ce_outpkts;

      sprintf(jstext, "document.conn.state.value=\"%s\";\n", 
         cn_state(conn) );
      if(sendwait(pc->sock, jstext))
         goto send_err;

      sprintf(jstext, "document.conn.age.value=\"%s\";\n", 
         cn_age(cticks - conn->ce_start) );
      if(sendwait(pc->sock, jstext))
         goto send_err;

      sprintf(jstext, "set_traffic(%d, %d, %d, %d);\n",
         client->ce_outbytes,
         client->ce_outpkts,
         server->ce_outbytes,
         server->ce_outpkts);
      if(sendwait(pc->sock, jstext))
         goto send_err;

      sprintf(jstext, "document.conn.idle.value=\"%s\";\n", 
         cn_age(cticks - conn->ce_time) );
      if(sendwait(pc->sock, jstext))
         goto send_err;
      if(sendwait(pc->sock, stopscript))
         goto send_err;

   }
   USE_ARG(parm);
   return;
}


/* ht_showkillcn() - SSI routine to show a single connection indicated 
 * by attached name/value index and pointer
 * 
 * Returns 0 if OK.
 */

int
ht_showkillcn(wi_sess * sess, EOFILE * eofile)
{
   long     index;
   char *   cp;
   struct conn_entry * conn;

   conn = cn_verify(sess);
   if(!conn)
   {
      dtrap();
      return -1;
   }

   /* get the has table index and connection pointer */
   cp = wi_formvalue(sess, "index");
   if(!cp)
      return -1;
   index = atoi(cp);
   if(conn->hashx != index )
   {
      dtrap();
   }

   wi_printf(sess, "<table border=1 bgcolor=#FFFFFF ><tr><td>");
   cn_display1(sess, conn);
   wi_printf(sess, "</td></tr></table>");

   /* output hidden controls for index, ptr */
   wi_printf(sess, "\n<input type=hidden name=\"index\" value=\"%d\">\n", index );
   wi_printf(sess, "\n<input type=hidden name=\"ptr\" value=\"%ld\">\n", conn );

   return 0; /* OK return */
}


/******************* Blocklog push code *********************/


/* new_blklog(hp) - 
 * constructor for struct blklog
 */

struct blklog *
new_blklog(wi_sess * sess)
{
   struct blklog * newlog;

   newlog = (struct blklog *)npalloc(sizeof(struct blklog));
   if(!newlog)
      return NULL;

   newlog->sock = sess->ws_socket;
   newlog->next = blklogs;
   blklogs = newlog;
   return newlog;
}

/* del_blklog() - 
 * destructor for struct blklog
 */

void
del_blklog(struct blklog * dellog)
{
   struct blklog * tmp;
   struct blklog * last;

   last = NULL;
   for(tmp = blklogs; tmp; tmp = tmp->next)
   {
      if(tmp == dellog)
      {
         if(tmp->sock != INVALID_SOCKET)    /* close network connection */
         {
            socketclose(tmp->sock);
            tmp->sock = INVALID_SOCKET;
         }
         if(last)          /* unlink */
            last->next = tmp->next;
         else
            blklogs = tmp->next;

         npfree((char*)tmp);       /* release memory */
         return;
      }
      last = tmp;
   }
   dtrap();    /* not found */
   return;
}

static char blbuf[512];

void
blklog_log(struct alert_event * evt)
{
   int      err;
   char *   cp;
   struct blklog * log;
   struct blklog * tmp;

   if(blklogs == NULL)
      return;

   /* Build script to send event text line to javascript newblock() 
    * routine. This routine will put the line at the bottom og the screen
    */

   strcpy(blbuf, "<SCRIPT LANGUAGE=\"JavaScript\">\nnewblock(\"");
   cp = blbuf + strlen(blbuf);

   sprintf(cp, evt->data.log.format, 
      evt->data.log.va_parms[0], evt->data.log.va_parms[1],
      evt->data.log.va_parms[2], evt->data.log.va_parms[3],
      evt->data.log.va_parms[4], evt->data.log.va_parms[5],
      evt->data.log.va_parms[6], evt->data.log.va_parms[7],
      evt->data.log.va_parms[8], evt->data.log.va_parms[9]
      );

   /* Go through the string overwriting quotes & newlines. */
   while(*cp > 0)
   {
      switch (*cp)
      {
      case '\"':
         *cp = '\'';
         break;
      case '\n':
         *cp = ' ';
         break;
      default:
         break;
      }
      cp++;
      if(cp >= (blbuf + sizeof(blbuf)))
         panic("blklog 1");
   }

   cp = blbuf + strlen(blbuf);
   strcat(cp, "\");\n</SCRIPT>\n");

   if(strlen(blbuf) >= sizeof(blbuf))
      panic("blklog 2");

   /* Stuff message out all the log sockets */
   log = blklogs;
   while(log)
   {
      err = sendwait(log->sock, blbuf);   /* send the block event html */

      /* if socket send had error, delete log session */
      if(err)
      {
         tmp = log->next;
         del_blklog(log);
         log = tmp;
      }
      else
         log = log->next;
   }
   return;
}


/* blocklog_close() - Exit_hook routine to close all the log connections.
 *
 * Returns 0 if OK. 
 */

static int bl_closeset = FALSE;  /* true when exit hook is set */

static char * js_closecode = "<script language=javascript> window.close() </script>";


void
blocklog_close(void)
{
   int      err;
   struct blklog * log;
   struct blklog * tmp;

   /* Stuff message out all the log sockets */
   log = blklogs;
   while(log)
   {
      err = sendwait(log->sock, js_closecode);   /* send the close script */

      /* delete log session */
      tmp = log->next;     /* preserve list link */
      del_blklog(log);     /* kill socket, free mem */
      log = tmp;           /* continue along the list */
   }
   bl_closeset = FALSE;
   return;
}


/* blocklog_open() - Push notification routine for the blocking log
 * open button.
 *
 * Returns 0 if OK. 
 */

int
blocklog_open(wi_sess * sess, EOFILE * eofile)
{
   int      err;
   int      len;
   char *   msg;
   struct blklog *      log;
   struct sockaddr_in   so;
   ip_addr  watcher;
   ip_addr  watchee;

   /* Set the exit hook to close open logs when Buster exits */
   if(bl_closeset == FALSE)
   {
      bl_closeset = TRUE;
   }

   /* There are two IP addresses involved - the host whose blocking 
    * is being logged, and the host with the logging window open.
    * The may often be the same host, but not always.
    */
   
   msg = wi_formipaddr(sess, "host", &watchee);
   if(msg)
   {
      /* no host, use wildcard */
      watchee = 0;
   }

   /* Need SSL support later? */
   len = sizeof(so);
   err = getsockname(sess->ws_socket, (struct sockaddr *)&so, &len);
   watcher = so.sin_addr.s_addr;

   /* find or create the blklog object for this session */
   for(log = blklogs; log; log = log->next)
   {
      if((log->watcher == watcher) && (log->watchee == watchee))
      {
         /* Found an open log socket. Make sure it still works. This
          * message also brings it to the front of the Z-order.
          */
         err = sendwait(log->sock, 
            "<SCRIPT LANGUAGE=\"JavaScript\">\nnewblock(\"Log window "
            "is already open.\");\n</SCRIPT>\n");
         if(err == 0)
         {
            /* Old log session is still OK, so close the new socket 
             * outright (it's useless) and return.
             */
            socketclose(sess->ws_socket);
            return 0;   /* Not an error */
         }
         else     /* old log session is dead */
         {
            del_blklog(log);     /* Clean up old session */
            log = NULL;          /* Indicate no log found */
            break;               /* fall to make new session */
         }
      }
   }

   /* Fall to here if no useable session is found. */
   if(log != NULL)   /* sanity check */
   {
      dtrap();
   }

   /* blklog not found, create a new one */
   log = new_blklog(sess);
   if(log == NULL)
      return ENP_NOMEM;
   log->watchee = watchee;
   log->watcher = watcher;
   sendwait(log->sock, "Connection ready...<br>");

   return 0; /* OK return */
}

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/



