/* brssi.c
 *
 * Generic SSI for Buster Web GUI interface.
 * 
 * ROUTINES: ischecked(), ht_ssi_cvar(), ht_printf(), 
 * ROUTINES: act_color(), ht_hostlist(), ht_mpip(), ht_showblip()
 * ROUTINES: ht_showblport(), ht_blkreason(), ht_viruslist(), 
 * ROUTINES: ht_last_update(), editword_cgi(), addword_cgi()
 * ROUTINES: ht_listwords(), word_cgi(), host_show1(), 
 * ROUTINES: ht_inetlist(), ht_hostmenu(), ht_gethost(), 
 * ROUTINES: ht_commlist(), comm_option(), 
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 */

#include "busterport.h"   /* Praemio */
#include "brnet.h"
#include "ip.h"
#include "brdb.h"
#include "brdns.h"

#include "websys.h"
#include "webio.h"
#include "webfs.h"
#include "wsfdata.h"

#include "busthtml.h"


extern   u_long   br_minpkts(struct host_entry * host);
extern   char *   bgstyles[];

/* Flag to permit connections by the localhost only (security). */
extern   int   wi_localhost;

char * idlecell = "<td class=\"%s\" width=40 height=11 align=right "
   "title=\"%u %s since last network activity\" >%u</td><td></td>\n";

/* ischecked(int boolean_exp) - helper for ht_ssi_cvar(), generates "checked" 
 * or blank text for checkboxes based on the boolean expression passed. 
 */

static char * 
ischecked(int boolean_exp)
{
   if(boolean_exp)
      return "checked"; /* TRUE */
   else
      return "";  /* not TRUE, return empty string */
}

#ifdef STRISTR_ALREADY
extern   char *   stristr(char*, char*);
#else
char *   
stristr(char * s1, char * s2)
{
   /* look for first occurance of s2 in s1 */
   while(*s1)
   {
      if((*s1 & 0x20) == (*s2 & 0x20))
      {
         if(strnicmp(s1, s2, strlen(s2)) == 0)
            return s1;
      }
      s1++;
      s2++;
   }
   return NULL;
}
#endif


int
wi_cvariables(wi_sess * sess, int token)
{
   int   e;

   switch(token)
   {
   case TCPTMO213:
      e = wi_putlong(sess, (u_long)(br_cfg.keep_tcpconn/MINUTE));
      break;
   case UDPTMO214:
      e = wi_putlong(sess, (u_long)(br_cfg.keep_udpconn/MINUTE));
      break;
   case ICMPTMO215:
      e = wi_putlong(sess, (u_long)(br_cfg.keep_icmpconn/MINUTE));
      break;
   case IPTMO216:
      e = wi_putlong(sess, (u_long)(br_cfg.keep_iphost/HOUR));
      break;
   case LOGSIZE217:
      e = wi_putlong(sess, (int)(br_cfg.logsize));
      break;
   case REFRESH218:
      e = wi_putlong(sess, (int)(br_cfg.screen_refresh/TPS));
      break;
   case BUSTERNAME219:
      e = wi_putstring(sess, (char*)(&systemname[0]));
      break;
   case NUMSIGS220:
      e = wi_putlong(sess, (u_long)(sigct));
      break;
   case MACTMO221:
      e = wi_putlong(sess, (u_long)((br_cfg.mactmo/60)));
      break;
   case URLBUF222:
      e = wi_putstring(sess, (char*)(br_urlbuf));
      break;
   case MS_ENAB223:
      e = wi_putlong(sess, (int)(br_srcserving));
      break;
   case PROGNAME224:
      e = wi_putstring(sess, (char*)(br_appname));
      break;
   case SET_IP225:
      e = wi_putstring(sess, (char*)(print_ipad(nets[br_ifview]->n_ipaddr)));
      break;
   case SET_MASK226:
      e = wi_putstring(sess, (char*)(print_ipad(nets[br_ifview]->n_mask)));
      break;
   case SET_ROUTER227:
      e = wi_putstring(sess, (char*)(print_ipad(nets[br_ifview]->n_router)));
      break;
   case SET_DNS1228:
      e = wi_putstring(sess, (char*)(print_ipad(dns_servers[1])));
      break;
   case SET_DNS2229:
      e = wi_putstring(sess, (char*)(print_ipad(dns_servers[2])));
      break;
   case CKLOGTCP230:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.logevents&LE_TCPTMO)));
      break;
   case CKLOGUDP231:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.logevents&LE_UDPTMO)));
      break;
   case CKLOGICMP232:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.logevents&LE_ICMPTMO)));
      break;
   case CKLOGIPH233:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.logevents&LE_IPHTMO)));
      break;
   case CKLOGMAC234:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.logevents&LE_LOGDELMAC)));
      break;
   case CKSORTIP235:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.sortparm=='i')));
      break;
   case CKSORTPT236:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.sortparm=='p')));
      break;
   case CKSORTTR237:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.sortparm=='t')));
      break;
   case CKSORTST238:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.sortparm=='s')));
      break;
   case CKSORTID239:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.sortparm=='I')));
      break;
   case CKSORTNM240:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.sortparm=='n')));
      break;
   case CKDELMAC241:
      e = wi_putstring(sess, (char*)(ischecked((br_cfg.mactmo>0)&&(br_cfg.cflags&BC_DELIDLE))));
      break;
   case CKHIDEMAC242:
      e = wi_putstring(sess, (char*)(ischecked((br_cfg.mactmo>0)&&!(br_cfg.cflags&BC_DELIDLE))));
      break;
   case CK_IP243:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.cflags&BC_USER_IP)));
      break;
   case CK_MASK244:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.cflags&BC_USER_MASK)));
      break;
   case CK_ROUTER245:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.cflags&BC_USER_ROUTER)));
      break;
   case CK_DNS1246:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.cflags&BC_USER_DNS1)));
      break;
   case CK_DNS2247:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.cflags&BC_USER_DNS2)));
      break;
   case FP_TCPCK248:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.filterparm&FP_TCP)));
      break;
   case FP_UDPCK249:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.filterparm&FP_UDP)));
      break;
   case FP_ICMPCK250:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.filterparm&FP_ICMP)));
      break;
   case FP_ALLCK251:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.filterparm==FP_ALL)));
      break;
   case FP_BUSTER252:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.hide_parm&HP_BUSTER)));
      break;
   case HIDEBUSTER253:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.hide_parm&HP_BUSTER)));
      break;
   case FP_LOCAL254:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.hide_parm&HP_LOCAL)));
      break;
   case HIDELOCAL255:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.hide_parm&HP_LOCAL)));
      break;
   case CC_ENAB256:
      e = wi_putstring(sess, (char*)(((br_cfg.cflags&BC_CONTENT)?"disable":"enable")));
      break;
   case CC_CHECKED257:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.cflags&BC_CONTENT)));
      break;
   case HO_CHECKED258:
      e = wi_putstring(sess, (char*)(ischecked(br_cfg.cflags&BC_HOSTONLY)));
      break;
   case DHCP_CHECKED259:
      e = wi_putstring(sess, (char*)(ischecked(nets[br_ifview]->n_flags&NF_DHCPC)));
      break;
   case LO_CHECKED260:
      e = wi_putstring(sess, (char*)(ischecked(wi_localhost)));
      break;
   default:
      e = WIE_BADPARM;
      break;
   }
   return e;
}


int   buster_execfunc(wi_sess * sess, char * args)
{
   char  result[128];
   int   err;

   MEMSET(result, 0, sizeof(result));
   if(strncmp(args, "version", 7) == 0)
   {
      sprintf(result, "%s", br_version);
   }
   else if(strncmp(args, "host", 4) == 0)
   {
      char * hostinfo = nextarg(args);

      if((sess->ws_formlist == NULL) ||
         (sess->ws_formlist->paircount < 1))
         return ENP_PARAM;
      
      return(br_hostparm(sess, hostinfo));
   }
   else if(strncmp(args, "update", 6) == 0)
   {
      ht_last_update(sess, NULL);
   }
   else if(strncmp(args, "mode", 4) == 0)
   {
      strncpy(result, br_modenames[br_mode], sizeof(result) );
   }
   else if(strncmp(args, "ipstat", 6) == 0)
   {
      return(br_ipstats(sess, args));
   }
   else
   {
      dtrap();
   }

   err = wi_putstring(sess, result);

   return err;
}

static char htmlmac[20];

/* print a mac address in a format which goes through HTML name/value encoding*/

char *
html_mac(u_char * mac)
{
   sprintf(htmlmac, "%02X-%02X-%02X-%02X-%02X-%02X", 
      *mac, *(mac+1), *(mac+2), *(mac+3), *(mac+4), *(mac+5));

   return htmlmac;
}


/* get an activity level color string for the passed metric. */

char *
act_color(int pkts,        /* bool - metric is pkts or bytes */
          int interval,    /* time interval, MINUTE, HOUR */
          u_long metric)   /* The value */
{
   u_long   pps;     /* adjusted metric for pps */

   if((interval < TPS) || (metric == 0))
      return bgstyles[0];

   pps = metric / (interval/TPS);
   if(!pkts)
      pps >>= 5;     /* convert for bytes */

   return ( bgstyles[pps_level(pps)] );
}



char * iconcell  = 
"</td>\n<td><a href=\"%s\"><img height=20 width=20 border=0 src=\"%s\" ></a>";

static   host_line = 0;   /* for host list color toggleing */

void
host_show1(wi_sess * sess, struct ip_hostentry *  iphost, u_long  pkts)
{
   char     iptext[16];
   char     urltext[64];
   char *   name;
   char *   timeunits;
   int      timecount;
   int      idle;
   char *   htclass;    /* color, etc, for row */
   int      colorx;     /* index into traffic colors */


   if(host_line++ & 1)
      htclass = "blanktint";
   else
      htclass = "networktint";

   name = br_getname(iphost);
   sprintf(iptext, "%u.%u.%u.%u", PASS_IPADDR(iphost->ipaddr) );

   wi_printf(sess, "<tr class=\"network\" ><td height=1 colspan=8 ></td></tr>");

   wi_printf(sess, "<tr class=\"%s\" ><td>", htclass);
   wi_printf(sess, name);
   wi_printf(sess, "</td>\n<td>");
   wi_printf(sess, iptext );
   wi_printf(sess, "</td>\n<td class=\"%s\" title=\"Packets sent to or from this computer and LAN\" align=right >%d", 
      act_color(1, MINUTE, pkts), pkts);

   /* get color index for connections cell.
    * add TPS to cticks to avoid divide by zero 
    */
   colorx = pps_level(iphost->conns / ((cticks + TPS)/TPS) );
   wi_printf(sess,  "</td>\n<td class=\"%s\"  title=\"Total connections between this computer and local computers\" align=right >%d", 
      bgstyles[colorx], iphost->conns);

   idle = cticks - iphost->tm_lastpkt;
   htclass = br_idleinfo(idle, &timeunits, &timecount );
   wi_printf(sess, idlecell, htclass, timecount, timeunits, timecount );


   sprintf(urltext, "machost.htm?host=%s", iptext);
   wi_printf(sess, iconcell, urltext, "btnview.gif");

   sprintf(urltext, "connlist.htm?prot=all&host=%s", iptext);
   wi_printf(sess, iconcell, urltext, "btnview.gif");
   wi_printf(sess, "</td>\n");

   return;
}

/* ht_hostlist() - SSI routine.
 *
 * List MAC hosts to passed http session, one host per line. 
 *
 * Returns NULL if OK.
 */

int
ht_hostlist(wi_sess * sess, EOFILE * eofile)
{
   int      i;
   int      count;
   u_long   pkts;
   struct host_entry * host;
   struct ip_hostentry * iphost;

   /* loop through hosts database */
   count = 0;
   host_line = 0;
   for(i = 0; i < HASH_HOSTS; i++)
   {
      host = hosttab[i];
      while(host)
      {
         count++;    /* count number of hosts */

         /* Why did I do this funny lookup instead of using host->iphost? Sanity check? JB */
         iphost = br_iplookup(br_ifview, host->iphost->ipaddr);
         if(iphost != host->iphost)
         {
            dtrap();    /* ever happen? */
         }

         pkts = br_minpkts(host);
         host_show1(sess, iphost, pkts);
         host = host->next;
      }
   }
   if(count == 0)
   {
      wi_printf(sess, "<tr><td></td><td>No Hosts detected yet.</td></tr>" );
      USE_ARG(eofile);
   }

   return 0; /* OK return */
}

/* ht_commlist() - SSI routine.
 *
 * Display list of current communities for management page.
 *
 * Returns NULL if OK. 
 */

int
ht_commlist(wi_sess * sess, EOFILE * eofile)
{
   int      err;
   u_long   timeval;
   char *   cssclass;
   char     datebuf[32];
   struct comm_dbud * comm;
   int      rownum = 0;

   for(comm = br_communities; comm; comm = comm->next)
   {
      if(++rownum & 1)
         cssclass = "communitytint";
      else
         cssclass = "blanktint";

      wi_printf(sess,  "<tr class=\"community\" height=1 ><td colspan=9 height=1 ></td></tr>");

      wi_printf(sess,  "<tr class=\"%s\" ><td title=\"name of Community\" >%s</td>",
         cssclass, br_prtcomm(comm) );

      if(comm->cm_url)
      {
         wi_printf(sess,  "<td title=\"Link for Community updates\" class=\"smaller\" >%s</td>",
            comm->cm_url );
      }
      else
      {
         wi_printf(sess,  "<td>(No update link)</td>");
         USE_ARG(eofile);
      }

      err = br_writedatetime(comm->cm_update, datebuf, sizeof(datebuf) );
      if(err)
         strcpy(datebuf, "Not Set");
      wi_printf(sess,  "<td>%s</td>", datebuf);

      if(comm->cm_state == FAILED)
         wi_printf(sess,  "<td title=\"Last attempted update was unsuccessful\" >Failed</td>");
      else if((comm->cm_state == IDLE) && (comm->cm_interval > 2))
         wi_printf(sess,  "<td title=\"Update is current\" >OK</td>");
      else
         wi_printf(sess,  "<td title=\"Update is currently in progress\" >Updating...</td>");

      timeval = comm->cm_interval + comm->cm_update;
      err = br_writedatetime(timeval, datebuf, sizeof(datebuf) );

      if((comm->cm_interval == 0) || err)
         strcpy(datebuf, "Not Set");
      wi_printf(sess,  "<td>%s</td><td>%d</td>", datebuf, comm->cm_revision );

      wi_printf(sess,  "<td>%d</td>", comm->cm_records);

      if(comm->cm_url[0])
      {
         wi_printf(sess,  
         "<td><input type=button class=\"buttons bcommunity\" "
         "onmouseover=\"this.className='buttons buttonsover bcommunityover'\" "
         "onmouseout=\"this.className='buttons bcommunity'\" "
         "onClick=\"cm_update('%s')\" value='Update' ></td>",
         br_prtcomm(comm) );
      }
      else
      {
         wi_printf(sess,  "<td>No Url</td>");
      }

      if(comm == &local_comm)
      {
         wi_printf(sess,  "<td>Permanent</td>");
      }
      else
      {
         wi_printf(sess,  
         "<td><input type=button class=\"buttons bcommunity\" "
         "onmouseover=\"this.className='buttons buttonsover bcommunityover'\" "
         "onmouseout=\"this.className='buttons bcommunity'\" "
         "onClick=\"cm_delete('%s')\" value='Delete' ></td></tr>",
         br_prtcomm(comm) );
      }
   }

	return 0; /* OK return */
}


/* comm_option
 * 
 * Invoked when user clicks on a GUI community button.
 *
 * Returns NULL if OK, else short error text
 */

char *
comm_option(wi_sess * sess, EOFILE * eofile)
{
   char *	update;		/* to be set from form */
   char *	delcomm;		/* to be set from form */
   struct comm_dbud * comm;

   /* Only one of these will be set, depending on button: */
	update = wi_formvalue(sess, "update");
	delcomm = wi_formvalue(sess, "delete");

   if(update)
   {
      comm = br_cmbyfile(update);
      br_cmupdate(comm);
   }
   else if(delcomm)
   {
      comm = br_cmbyfile(delcomm);
      br_cmdel(comm);
   }
   else
   {
      dtrap();
      return("bad button");
   }

   /* Reload the community page */
   wi_redirect(sess, "comms.htm");
   return NULL;
}


/* ht_inetlist()
 *
 * SSI routine stub
 *
 * Lists all the known IP (Internet) hosts.
 *
 * Returns NULL if OK. 
 */

int
ht_inetlist(wi_sess * sess, EOFILE * eofile)
{
   int      i;
   int      count;
   struct ip_hostentry * iphost;

   /* loop through hosts database */
   count = 0;
   host_line = 0;

   for(i = 0; i < HASH_IPHOSTS; i++)
   {
      for(iphost = ip_hosttab[i]; iphost; iphost = iphost->next)
      {
         if(iphost->flags & IH_BROADCAST)
            continue;

         if(islocal_ip(br_ifview, iphost->ipaddr))
            continue;

         count++;    /* count number of hosts */
         host_show1(sess, iphost, iphost->tot_pkts);
      }
   }
   if(count == 0)
      wi_printf(sess, "<tr><td class=\"networktint\" ></td><td>No Hosts detected yet.</td></tr>" );

   return 0; /* OK return */
}


/* ht_mpip()
 *
 * SSI routine stub
 *
 * Inserts our IP address in dot notation.
 *
 * Returns 0 if OK. 
 */

int
ht_mpip(wi_sess * sess, EOFILE * eofile)
{
	char buf[16];

   sprintf(buf, "%u.%u.%u.%u", PASS_IPADDR(nets[br_ifview]->n_ipaddr));
   wi_printf(sess,  buf);

   USE_ARG(eofile);
	return 0; /* OK return */
}


/* ht_showmac()
 *
 * SSI routine stub
 *
 * Output text for mac host contained the form.
 * Format is "host=xx-xx-xx-xx-xx-xx"
 *
 * Returns 0 if OK. 
 */

int
ht_showmac(wi_sess * sess, EOFILE * eofile)
{
   char *   hosttext;
   u_char   macaddr[6];

   /* get the IP address of host to block */
   hosttext = wi_formvalue(sess, "host");
   if(!hosttext)
   {
      USE_ARG(eofile);
      dtrap();
      return -1;
   }
   parse_mac(hosttext, macaddr);
   wi_printf(sess,  "%02x-%02x-%02x-%02x-%02x-%02x", 
      macaddr[0],
      macaddr[1],
      macaddr[2],
      macaddr[3],
      macaddr[4],
      macaddr[5] );

	return 0; /* OK return */
}


/* ht_showblip() - SSI routine.
 *
 * This routine is invoked by three SSI files. The name of the SSI
 * file determines the format of the blocked host's IP information
 *
 * show_blip.ssi - output IP address in dot notation
 * show_blname.ssi -  output DNS name if know, else "(unknown)"
 * show_blhost.ssi -  output DNS if known, else return IP
 *
 * Returns 0 if OK. 
 */

int
ht_showblip(wi_sess * sess, EOFILE * eofile)
{
   struct ip_hostentry * iphost;
   char *   name;
   char *   msg;
   ip_addr  ipaddr;

   /* get the IP address of host to block */
   msg = wi_formipaddr(sess, "host", &ipaddr);
   if(msg)
   {
      /* maybe we have a name instead */
      name = wi_formvalue(sess, "host");

      ipaddr = 0;    /* clear this in case mac lookup fails */

      /* maybe name is a MAC address */
      if(name && 
         (*(name+2) == '-') &&
         (strlen(name) == 17))
      {
         struct host_entry * machost;
         u_char   macaddr[6];

         if(parse_mac(name, macaddr) == 0)
         {
            machost = br_hostlookup((char*)macaddr, br_ifview);
            if(machost)
            {
               iphost = machost->iphost;
               ipaddr = iphost->ipaddr;
            }
         }

      }
   }
   else
   {
      iphost = br_iplookup(br_ifview, ipaddr);    /* see if we have info on this host */
      if(iphost)
         name = br_getname(iphost);    /* get optional name */
      else
         name = "(unknown)";
   }

   if(strstr (eofile->eo_emfile->em_name, "show_blip"))
   {
      wi_printf(sess,  "%u.%u.%u.%u", PASS_IPADDR(ipaddr));
      return 0;
   }

   if(strstr (eofile->eo_emfile->em_name, "show_blname"))
   {
      wi_printf(sess,  "%s", name);
      return 0;
   }

   /* default case for show_blhost */
   if(name && (*name != '('))
      wi_printf(sess,  "%s", name);
   else
      wi_printf(sess,  "%u.%u.%u.%u", PASS_IPADDR(ipaddr));

	return 0; /* OK return */
}


/* ht_showblport() - SSI routine.
 *
 * This routine is invoked by three SSI files. The name of the SSI
 * determines the format of the blocked port information
 *
 * show_blport - output port as a number
 * show_blapp -  output application type (protocol) if known, else "(unknown)"
 * show_blprot -  output application type if known, else number
 *
 * Returns 0 if OK. 
 */

int
ht_showblport(wi_sess * sess, EOFILE * eofile)
{
   int      port;
   char *   name;

   /* get pointer to port text in form */
   name = wi_formvalue(sess, "port");
   if(name == NULL)
   {
      dtrap();
      return -1;
   }
   port = atoi(name);

   if(strstr (eofile->eo_emfile->em_name, "show_blport"))
   {
      wi_printf(sess,  "%u", port);
   	return 0; /* OK return */
   }
   name = br_portname((unshort)port);     /* get service name */

   if(name)
      wi_printf(sess,  "%s", name);
   else if( strstr(eofile->eo_emfile->em_name, "show_blapp"))
      wi_printf(sess,  "(unknown)");
   else
      wi_printf(sess,  "%u", port);

	return 0; /* OK return */
}


/* ht_blkreason() - SSI routine.
 *
 * Output the reason a file or server was blocked. There should be a 
 * single name/value pair attached to the calling URL to indicate the reason. 
 * If it was blocked due to the IP address, then the name is "host" and the 
 * value is the IP address, otherwise it was blocked because of the URL path.
 * In this case name is "path" and the value is the errant file/path.
 *
 * Outpout is html, with two hidden controls added.control names are "blkhost"
 * and "blkpath". They corrospond to "host" and "path" (above) respectivly.
 *
 * Returns 0 if OK. 
 */

int
ht_blkreason(wi_sess * sess, EOFILE * eofile)
{
   ip_addr  badhost = 0;
   char *   badpath;
   struct conn_entry * conn;

   /* If there is no form data return error */
   if(sess->ws_formlist == NULL)
      return -1;

   wi_formipaddr(sess, "host", &badhost);
   badpath = wi_formvalue(sess, "path");

   if(badhost && badpath)
   {
      dtrap();    /* had both? Try to recover */
   }
   if(!badhost && !badpath)
   {
      dtrap();
      return -1;     /* can't recover */
   }
   /* auto-generated code: */
	conn = get_form_conn(sess);
	if(conn && (conn->block.type !=0))
	{
      if(conn->block.ptr == NULL)
      {
         dtrap();
         return -1;
      }
      switch(conn->block.type)
      {
      case CTF_TYPEIP:     /* blocked because of IP address */
         {
            struct control_ip * cip = (struct control_ip *)conn->block.ptr;
            if(cip->ci_name && cip->ci_name->cu_name)
            {
               wi_printf(sess,  "Site <b>%s</b> is blocked due to: <b>%s</b>\n",
                  cip->ci_name->cu_name, cip->ctl.offense->desc);
            }
            else
            {
               wi_printf(sess,  "IP address <b>%u.%u.%u.%u</b> is blocked due to: <b>%s</b>\n",
                  PASS_IPADDR(cip->ci_ipaddr), cip->ctl.offense->desc);
            }
            break;
         }
         break;
      case CTF_TYPEPORT:   /* TCP or UDP port number */
         dtrap();
         break;
      case CTF_TYPENAME:   /* DNS name */
         {
            struct control_dn * dnp = (struct control_dn *)conn->block.ptr;
            wi_printf(sess,  "Server <b>%s</b> is blocked due to: <b>%s</b>\n",
               dnp->cu_name, dnp->ctl.offense->desc);
            break;
         }
      case CTF_TYPEPATH:   /* path/name potion of URL */
         {
            struct b_path * bpath = (struct b_path *)conn->block.ptr;
            wi_printf(sess,  "Page <b>%s</b> is blocked because it contains <b>'%s'</b>",
               conn->data, bpath->pathname );
            wi_printf(sess,  ", which is often used for <b>%s</b>\n",
               bpath->ctl.offense->desc);
            break;
         }
      case CTF_TYPEWORD:   /* Web content word */
         dtrap();
         break;
      case CTF_TYPESIG:    /* Binary signature (virus) */
         dtrap();
         break;
      default:
         dtrap();
         return -1;
      }
	}
   else if(badhost)
   {
      char * name;
      struct control_ip * ctlip;

      /* Look up the blocked IP address and see if we know why it was blocked. If
       * If the user sat looking at the page for a long time the IP address may
       * no longer be in the block list, so allow for this.
       */
      ctlip = ipe_lookup(htonl(badhost));
      if(ctlip)
      {
         if(ctlip->ci_name)      /* Use DNS name if known */
            name = ctlip->ci_name->cu_name;
         else
            name = br_getnametext(badhost);
         wi_printf(sess,  "Server <b>%s</b> is blocked due to: <b>%s</b>\n",
            name, ctlip->ctl.offense->desc);
      }
      else
      {
         /* IP address is not in block list. Tell user, and go on to the unblock page */
         wi_printf(sess,  "Unable to find current information on server <b>%u.%u.%u.%u</b>.",
            PASS_IPADDR(badhost) );
      }
      wi_printf(sess,  "<input type=hidden name='blkhost' value='%u.%u.%u.%u' >",
         PASS_IPADDR(badhost));
   }
   else
   {
      struct b_path * bpath;

      /* convert the path parameter back into a local pointer */
      bpath = (struct b_path * )atol(badpath);
      if(!bpath)     /* test for valid bpath later? Violate heap? -JB- */
         return -1;

      wi_printf(sess,  "The URL contains the text <b>\"%s\"</b>\n",
         bpath->pathname);
      wi_printf(sess,  "<input type=hidden name='blkpath' value='%s' >",
         bpath->pathname);
   }


   USE_ARG(eofile);

	return 0; /* OK return */
}



/* ht_viruslist() - SSI routine.
 *
 * Display list of Binary (virus) signatures 
 *
 * Returns 0 if OK. 
 */

#define SDLEN  48

extern int   def_limit;           /* Max entries per page */

static   int   expert_mode = FALSE;

int
ht_viruslist(wi_sess * sess, EOFILE * eofile)
{
   int      i;
   int      hits = 0;
   char *   text;
   char *   lastentry;
   u_char * cp;
   char     sigdata[SDLEN];
   char *   mode;
   int      gotlast;
   struct virussig * virusdb;

   text = wi_formvalue(sess, "virusname");
   if(!text)
   {
      USE_ARG(eofile);
      text = "";     /* match everything? */
   }
   mode = wi_formvalue(sess, "mode");
   if(mode)
   {
      if(stricmp(mode,  "expert") == 0)
         expert_mode = TRUE;
      else
         expert_mode = FALSE;
   }

   /* If a "last entry" was passed, it means the viewer is paging through 
    * a large number of sigs (maybe thousands) a screenfull (def_limit) at 
    * a time. If this parameter is set then also clear the gotlast flag, 
    * which will be set to true when the table scanning loop gets to 
    * the gotlast name.
    */
   lastentry = wi_formvalue(sess, "lastentry");
   if(lastentry)
      gotlast = FALSE;
   else
      gotlast = TRUE;   /* Avoid compiler "unset var" warnings */

   /* Make invisible control with virus name */
   wi_printf(sess,  "<input type=hidden name=\"oldname\" value=\"%s\" >\n",
      text);

   /* Display a list heading line: */
   wi_printf(sess,  
      " Results for \"%s\":<br><center><table border=0 width=90% "
      "cellspacing=0 cellpadding=2  >", 
      text);

   /* scan the virus database looking for names which match */
   for(virusdb = viruslist; virusdb; virusdb = virusdb->next)
   {
      if(stristr(virusdb->name, text))
      {
         if(lastentry && (gotlast == FALSE))
         {
            if(strcmp(virusdb->name, lastentry) == 0)
               gotlast = TRUE;
            continue;
         }

         hits++;
         /* print heading lines */
         if(expert_mode)
         {
            if(hits == 1)
            {
               wi_printf(sess,  "<tr><td>Name</td><td width=15%>sig.len.</td>"
                  "<td> &nbsp; Start of signature data (hex)</td></tr>");
            }

            /* convert binary signature to hex display */
            cp = virusdb->sig;
            for(i = 0; i < (SDLEN - 2); i += 2)
            {
               sigdata[i] = hexdigit((u_char)((*cp) >> 4));
               sigdata[i+1] = hexdigit((u_char)((*cp++) & 0x0F));
               /* Stop if at end of signature data */
               if((i >> 1) >= virusdb->siglen)
                  break;
            }
            sigdata[i] = 0;

            wi_printf(sess,  "<tr class=\"%s\"><td><pre>%s</td><td align=right >"
               "<pre> %d </td><td><pre>%s</td></tr>",
               hits&1?"blockingtint":"blanktint", 
               virusdb->name, virusdb->siglen, sigdata );
         }
         else  /* Dummy mode */
         {
            if((hits % 3) == 1)
            {
               wi_printf(sess,  "<tr class=\"%s\" >", 
                  (!((hits/3)&1))?"blockingtint":"blanktint");
            }
            wi_printf(sess,  "<td><pre>&nbsp; %s &nbsp;</td><td width=20 bgcolor=#FFFFFF ></td>", 
               virusdb->name);
            if((hits % 3) == 0)
            {
               wi_printf(sess,  "</tr>\n");
            }
         }
         /* See if that's enough for one page */
         if(hits >= def_limit)
            break;
      }
   }
   if(hits == 0)
      wi_printf(sess,  "<tr><td>No matching virus names found.</td></tr>");

   wi_printf(sess,  "</table>");

   /* If not all entrys are shown then make a control with current 
    * entry virus name for viewing next batch of entries 
    */
   if(hits >= def_limit)
   {
      wi_printf(sess,  "<input type=button name=\"more\" value=\"Next %u Entries\" \n",
         def_limit );
      wi_printf(sess,  " onClick=\"document.location='virus.htm?lastentry=%s'\"" 
         " class=\"button blockinglt\" >\n", virusdb->name );
   }

	return 0; /* OK return */
}

/* ht_last_update() - SSI routine.
 * Prints out date of last update to database.
 *
 * Returns 0 if OK. 
 */


int
ht_last_update(wi_sess * sess, EOFILE * eofile)
{
   int      err;
   char     datetxt[48];   /* tmp buffer for date writes */

   if(br_dbdate == 0)
   {
      wi_printf(sess,  "No updates.");
      return 0;
   }

   err = br_writedatetime( br_dbdate, datetxt, strlen(datetxt) );
   if(err)
   {
      wi_printf(sess,  "Not set.");
      USE_ARG(eofile);
   }
   else
   {
      wi_printf(sess,  "%s, GMT", datetxt);
   }

   return 0; /* OK return */
}


/* ht_worddata() - SSI routine.
 *
 * The Content words GUI has a concept of selecting a word. 
 * for re-weighting or deletion. This routine extracts the 
 * info about the word from the passed form.
 *
 * Returns 0 if OK. 
 */

int
ht_worddata(wi_sess * sess, EOFILE * eofile)
{
   struct vword * wordp;
   char * word;
   char * args;

   /* If no vocabulary was passed in name/values, just return */
   if(sess->ws_formlist == NULL)
      return 0;

   /* Arg tells use which detail (word, weight, ???) to ht_printf */
   args = wi_formvalue(sess, "arg");
   if(!args)
   {
      dtrap();
      return ENP_PARAM;
   }

   dtrap();   // watch first time on new server

   while(args && (*args == ' '))
      args++;

   switch(*args)
   {
   case 'w':   /* may be "word" or "weight" */
      word = wi_formvalue(sess, "thisword");
      if(word == NULL)
         return 0;
      switch(*(args+1))
      {
      case 'o':   /* "word" (currently selected) */
         wi_printf(sess,  "%s", word);
         break;
      case 'e':   /* "weight of word */
         wordp = word_lookup(word);
         if(!wordp)
         {
            /* this can happen if we just deleted the active word */
            return 0;
         }
         /* see if this sword is a "showstopper" or not */
         if(wordp->weight == -1)
            wi_printf(sess,  "X");
         else
            wi_printf(sess,  "%d", wordp->weight );
         break;
      default:
         dtrap();
         break;
      }
      break;
   default:
      dtrap();
      break;
   }

   return 0; /* OK return */
}


/* ht_listwords() - SSI routine.
 *
 * List all the content-blocking Words to ht_printf
 *
 * Returns 0 if OK. 
 */

extern struct comm_dbud * def_comm; /* filter for community in ctlcgi.c */

int
ht_listwords(wi_sess * sess, EOFILE * eofile)
{
   struct vword * wordp;
   struct vword * tmp;
   struct vword * list;
   int      i;
   int      wordct;

   wordct = 0;

   /* Insertion sort the vocabulary's word list */
   tmp = list = NULL;

   for(i = 0; i < VOCABHASHSIZE; i++)     /* Scan hash table */
   {
      for(wordp = wordhashtab[i]; wordp; wordp = wordp->nexthash)
      {
         if((def_comm) && (wordp->ctl.comm != def_comm))
            continue;

         if(list == NULL)     /* no list yet */
         {
            list = wordp;     /* Make wordp head of list */
            wordp->answerlist = NULL;
         }
         else if(wordcmp( wordp->word, list->word) < 0)
         {
            wordp->answerlist = list;  /* insert at head of list */
            list = wordp;
         }
         else  /* new word goes somewhere after head of list */
         {
            tmp = list;

            /* Scan through list until wordp goes between current 
             * and next, or we get to end. 
             */
            while((tmp->answerlist != NULL) && 
               (wordcmp( wordp->word, tmp->answerlist->word) > 0))
            {
               tmp = tmp->answerlist;
            }
            /* Insert word between tmp entry and tmp->next */
            wordp->answerlist = tmp->answerlist;
            tmp->answerlist = wordp;
         }
      }
   }

   /* now print the sorted list */
   for(wordp = list; wordp; wordp = wordp->answerlist)
   {
      char weightstring[4];

      if(wordp->weight == -1)
         strcpy(weightstring, "X");
      else
         sprintf(weightstring, "%d", wordp->weight);

      /* Build a link, with javascript, to set word & weight in form */
      wi_printf(sess,  "<td bgcolor=#FFFFFF >&nbsp;</td><td>"
         "<a href=\"foo.htm\" onClick=\"clickword('%s', '%s')\" >",
         wordp->word, weightstring );

      wi_printf(sess,  "%s</a></td>", wordp->word);

      /* see if this sword is a "showstopper" or not */
      if(wordp->weight == -1)
         wi_printf(sess,  "<td title=\"Any page containing this word is blocked\">X</td>\n");
      else
         wi_printf(sess,  "<td title=\"range: 1-10\">%d</td>\n", wordp->weight);

      wordct++;
      if((wordct % 5) == 0)
      {
         if(!(wordct & 1))
            wi_printf(sess,  "</tr><tr class=\"blockingtint\" >");
         else
            wi_printf(sess,  "</tr><tr class=\"blanktint\" >");
      }
   }

   USE_ARG(eofile);
   return 0;
}


/* editword_cgi() - CGI routine.
 *
 * Delete or change weight of a word on the content blocking list
 *
 *
 * Returns bitmask (see manual). 
 */

char *
editword_cgi(wi_sess * sess, EOFILE * eofile)
{
   char *   editword;   /* to be set from form */
   char *   weight;     /* to be set from form */
   char *   opcode;     /* to be set from form */
   char *   onoff;
   struct vword * wordp;

   /* Overloaded form: if submit came form content_onoff form, then
    * handle the toggle and return 
    */
   onoff = wi_formvalue(sess, "cc_toggle");
	if(onoff != NULL)
	{
      /* toggle the content control bit */
      if(br_cfg.cflags & BC_CONTENT)
         br_cfg.cflags &= ~BC_CONTENT;
      else
         br_cfg.cflags |= BC_CONTENT;

      br_saveparms();    /* Save new config */

      wi_redirect(sess, "content.htm");    /* reload the content page */
      return NULL; /* OK return, show file */
	}

   /* auto-generated code: */

   editword = wi_formvalue(sess, "thisword");
	if(editword == NULL)
		return("bad editword");

	opcode = wi_formvalue(sess, "opcode");
	if(opcode == NULL)
		return("bad opcode");


   weight = wi_formvalue(sess, "weight");  /* May be NULL */

   /* Start of hand-written code */
   switch(*opcode)
   {
   case 's':   /* set_weight */
   	if(weight == NULL)
	   	return("Weight value not set");

      wordp = word_lookup(editword);
      if(!wordp)
         goto internal_error;
      if((*weight == 'X') || (*weight == 'x'))
         wordp->weight = -1;
      else
         wordp->weight = atoi(weight);
      break;
   case 'w':   /* word_del */
      wordp = word_lookup(editword);
      if(!wordp)
         goto internal_error;
      del_word(wordp);
      break;
   default:
		return("bad opcode");
   }

   wi_redirect(sess, "content.htm");

internal_error:
   return("Content word database error");
}

/* addword_cgi() - CGI routine.
 *
 * Add a new Content blocking word. Word and weight are 
 * in passed form
 *
 */

char *
addword_cgi(wi_sess * sess, EOFILE * eofile)
{
   int      err;
   char *	add_word;      /* to be set from form */
   int      add_weight;    /* to be set from form */
   struct vword * newword;

	/* auto-generated code: */

	add_word = wi_formvalue(sess, "add_word");
	if(add_word == NULL)
		return("bad add_word");

   err = wi_formint(sess, "add_weight", (long*)&add_weight);
	if(err)
		return("bad add_weight");

   /* Make a new word in this vocabulary. We can pass NULL to 
    * new_word() instead of a DB space since the vocabulary 
    * already exists. 
    */
   newword = new_word(add_word, &local_comm);
   if(newword == NULL)
   {
      /* Word is already in vocabulary, or heap is out of memory.
       * Assume the former.
       */
      return("Word is already in database");
   }
   newword->weight = add_weight;

   wi_redirect(sess, "content.htm");
   return NULL; /* OK return, show file */
}


/* ht_hostmenu() - SSI routine.
 *
 * output local host list, formatting as Javascript Menu
 *
 * Returns 0 if OK. 
 */

/* Javascript Menu item formats for host menus: */
char * hmtext1 = /* This string gets number of hosts */
   "Menu1_2=new Array(\"LAN List\", "
   "\"javascript:menulink('machosts.htm', "
   "'netbar.htm')\",'',%u,24,130);\n";

char * hmtext2 = /* This string gets index, hostname, MAC address */
   "Menu1_2_%u=new Array(\"%s\", "
   "\"javascript:menulink('connmap.htm?host="
   "%s', 'netbar.htm')\",\"\",0,24,130);\n";

int
ht_hostmenu(wi_sess * sess, EOFILE * eofile)
{
   int      hosts = 0;     /* number of hosts */
   int      i;
   struct host_entry ** host_array;    /* sorted hosts */
   struct host_entry * host;
   char *   hostname;      /* host IP or name for menu label */
   char *   hostmac;       /* host MAC address (text for html link) */

   if(brstates[br_ifview].machosts == 0)  /* No hosts known? */
      return 0;      /* Not an error */

   /* get array containing all the MAC hosts */
   host_array = br_sorthosts(&hosts);
   if(!host_array)
   {
      USE_ARG(eofile);
      dtrap();
      return ENP_RESOURCE;
   }

   wi_printf(sess,  hmtext1, hosts);

   for(i = 0; i < hosts; i++)
   {
      host = *(host_array + i);      /* get next host from sorted list */

      /* skip Multicast (and broadcast) hosts */
      if((htonl(host->iphost->ipaddr) & 0xFF000000) >= 0xE0000000)
      {
         dtrap();
         continue;
      }

      hostname = br_getnametext(host->iphost->ipaddr);
      hostmac = html_mac(host->macaddr);

      wi_printf(sess,  hmtext2, i+1, hostname, hostmac);
   }

   return 0; /* OK return */
}

/* ht_gethost() - SSI routine.
 *
 * Show "host" IP address from the current form  in dot notation.
 *
 * Returns 0 if OK.
 */

int
ht_gethost(wi_sess * sess, EOFILE * eofile)
{
   ip_addr  host;
   char *   msg;

   msg = wi_formipaddr(sess, "host", &host);
   if(msg)
   {
      USE_ARG(eofile);
      dtrap();       /* bad HTML? */
      return -1;
   }
   wi_printf(sess,  "%u.%u.%u.%u", PASS_IPADDR(host));

	return 0; /* OK return */
}

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


