/* cfgcgi.c
 *
 * Buster Configuration pages SSI and CGI routines
 *
 * ROUTINES: ht_alertchk(), textlong(), alertcfg_cgi(), 
 * ROUTINES: ht_email(), emailcfg_cgi(), br_pwdigest(), 
 * ROUTINES: check_permit(), user_ok(), ht_userlist(), ht_userdata(), 
 * ROUTINES: deluser_cgi(), usercfg_cgi(), ht_lanman(), lanman_cgi(), 
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 */

#include "busterport.h"   /* Praemio */
#include "brnet.h"
#include "ip.h"
#include "brdb.h"
#include "brdns.h"

#include "websys.h"
#include "webio.h"
#include "webfs.h"
#include "wsfdata.h"

#include "busthtml.h"


typedef int bool;

extern char *  smtp_server_name;


/* ht_alertchk() - SSI routine.
 *
 * Generate a table of all the alarm types and methods. Use the following format:.
 *
 *
<tr>
<td bgcolor=#f0eeff >New computer on network</font></td>
<td align=center ><input type=checkbox name=newmac_email <!--#include file="alertchk.ssi?name=newmac_email" --> ></td>
<td align=center ><input type=checkbox name=newmac_popup <!--#include file="alertchk.ssi?name=newmac_popup" --> ></td>
<td align=center ><input type=checkbox name=newmac_cons  <!--#include file="alertchk.ssi?name=newmac_cons" --> ></td>
<td align=center ><input type=checkbox name=newmac_audio <!--#include file="alertchk.ssi?name=newmac_audio" --> ></td>
</tr>
 *
 *
 * Returns 0 if OK.
 */


int
ht_alertchk(wi_sess * sess, EOFILE * eofile)
{
   int   alarm;
   int   method;

   for(alarm = 0; alarm < alertcount; alarm++)
   {
      wi_printf(sess, "<tr><td  class=\"alertcheck\" title=\"Click for info about this alert\" >"
         "<a href=\"http:////%s\" target=top >%s</a></td>\n", 
         alarms[alarm]->al_url, alarms[alarm]->longname);

      for(method = 0; method < alert_methodct; method++)
      {

         wi_printf(sess, "<td  class=\"alertcheck\" ><input type=checkbox name=\"%s_%s\" ",
            alarms[alarm]->name, a_methods[method].name );

         /* see if this method is enabled for this alarm. */
         if(alarms[alarm]->actions & a_methods[method].bitmask )
            wi_printf(sess, " CHECKED ");

         wi_printf(sess, " ></td>\n");
      }
      wi_printf(sess, "</tr>\n");
   }
   USE_ARG(eofile);
   return 0; /* OK return */
}


/* convert first four bytes of passed string to a long, with first 
 * byte in the high order byte of the long.
 */

u_long
textlong(char * parm)
{
   int      i;
   u_long   retval = 0;

   for(i = 0; i < 4; i++)
   {
      if(*parm <= ' ')
         retval = (retval << 8);
      else
         retval = (retval << 8) + (*parm++ | 0x20);
   }
   return retval;
}




/* alertcfg_cgi()
 *
 * CGI routine.
 *
 */


char *
alertcfg_cgi(wi_sess * sess, EOFILE * eofile)
{
   int      i, j;
   char *   tmp;
   char     cname[24];
   
   /* the nested loops below rotate through every event/method control 
    * name by combining the name fragment in the above tables. The code
    * then fetchs the event from the event tab;e, and sets or clears the 
    * bit using the value in the methods table.
    */
   for(i = 0; i < alertcount; i++)
   {
      for(j = 0; j < alert_methodct; j++)
      {
         /* build the control name */
         strcpy(cname, alarms[i]->name);
         strcat(cname, "_");
         strcat(cname, a_methods[j].name);

         /* get control name's boolean from form data */
         tmp = wi_formvalue(sess, &cname[0]);

         /* set or clear bit as needed.Some browsers don;t put unchecked 
          * checkboxes in the name/value pairs, so we have to assume these
          * missing controls (tmp == NULL) are in fact unchecked. 
          */
         if(tmp && (*tmp == 'o') && (*(tmp+1) == 'n'))
            alarms[i]->actions |= a_methods[j].bitmask;
         else
            alarms[i]->actions &= ~a_methods[j].bitmask;
      }
   }
   br_saveparms();    /* Save new config */

   wi_redirect(sess, "saveset.htm");
   return NULL;
}

/* ht_email() - SSI routine for email configuration page.
 *
 * Returns 0 if OK.
 */

/* HTML model for recipient list 

<tr><td></td><td bgcolor=#ffffff ><font face="Arial,Helvetica">
jbartas@clear2net.com
</td><td><input type=button value="delete" name="jbartas@clear2net.com" ></td></tr>

*/

#ifdef SMTP_ALERTS
int
ht_email(wi_sess * sess, EOFILE * eofile)
{
   char *   parm;   /* email parameter form form */
   int      i;
   u_long   parmval;

   parm = wi_formvalue(sess, "parm");

   if(!parm)
   {
      USE_ARG(vfp);
      USE_ARG(args);
      dtrap();
      return -1;
   }

   parmval = textlong(parm);

   switch(parmval)
   {
   case 0x73657276:     /* "server" */
      if(smtp_server_name)
         wi_printf(sess, "%s", smtp_server_name );
      else
         wi_printf(sess, "%u.%u.%u.%u", PASS_IPADDR(smtp_serverip) );
      break;
   case 0x75736572:     /* "user" */
      wi_printf(sess, smtp_sendername );
      break;
   case 0x656e6162:     /* "enabled" */
      if(smtpalert_enabled)
         wi_printf(sess, "checked" );
      break;
   case 0x72656369:     /* "reci" */
      if((smtp_rcptcount == 0) ||
         (*(char*)(&smtp_rcpt[0]) == 0) )
      {
         wi_printf(sess, 
            "<tr><td colspan=2 align=center><font face=\"Arial,Helvetica\"><b>"
            "No recipients in the list</b></td></tr>");
      }
      else     /* have a recipient list */
      {
         for(i = 0; i < smtp_rcptcount; i++)
         {
         wi_printf(sess, "<tr %s ><td></td><td>", 
            (i&1==1)?"class=\"preferencetint\"":"bgcolor=#FFFFFF");
         wi_printf(sess, smtp_rcpt[i]);
         wi_printf(sess, 
      "</td>\n<td><input type=submit value=\"Delete\" name=\"del_recipient\" \n"
      " onClick=\"value='%s'\";  class=\"button preferencelt\" ></td></tr>\n", 
            smtp_rcpt[i]);
         wi_printf(sess, "<tr class=\"preferences\" ><td height=1 colspan=3></td></tr>\n");
         }
      }
      break;
   case 0x70617373:     /* "password" */
      wi_printf(sess, "%s", smtp_password );
      break;
   default:
      dtrap();
      return -1;
   }

   return 0; /* OK return */
}


/* emailcfg_cgi()
 *
 * CGI routine.
 */

char *
emailcfg_cgi(wi_sess * sess, EOFILE * eofile)
{
   char *   msg;                 /* IP address parse error msg */
   char *   server_name;         /* DNS name of server */
   ip_addr  smtp_server;         /* to be set from form */
   char *   login_name;          /* to be set from form */
   char *   password;            /* to be set from form */
   char *   new_recipient;       /* to be set from form */
   char *   del_recipient;       /* to be set from form */
   bool     enable_email;        /* to be set from form */
   int      err;

   /* auto-generated code: */

   msg = get_form_ipaddr(form, "email_server", &smtp_server);
   if(msg)
   {
      /* See if it's a DNS name */
      server_name = wi_formvalue(sess, "email_server");
      if((server_name == NULL) || (*server_name <= ' '))
      {
         USE_ARG(hp);
         wi_redirect(sess, msg);
         return NULL;
      }
      if(strlen(server_name) >= sizeof(smtp_server_name))
      {
         wi_redirect(sess, "SMTP (email) server name is too long");
         return NULL;
      }
      /* Make the server 0.0.0.0 for now and use this as a 
       * flag to save the name instead. We also start a DNS 
       * query for it and hopeit completes OK before the server
       * is needed. The DNS query MAY return a valid IP address, 
       * but we DONT save it, since the user (or his ISP) 
       * prefers a name to a potentially transient number.
       */
      if(dns_servers[0] != 0)
         dns_query(server_name, &smtp_server);
 
      strcpy(smtp_server_name, server_name);
      smtp_server = 0;
   }
   else
   {
      /* SMTP sertver is hard-coded IP address; clear name */
      MEMSET(smtp_server_name, 0, sizeof(smtp_server_name));
   }

   login_name = wi_formvalue(sess, "email_user");
   if(login_name == NULL)
      return("bad login_name");

   password = wi_formvalue(sess, "email_password");

   /* The following are optional */
   new_recipient = wi_formvalue(sess, "new_recipient");
   del_recipient = wi_formvalue(sess, "del_recipient");
   enable_email = wi_formbool(sess, "email_enabled");

   if(new_recipient)    /* Add new recipient to list */
   {
      err = smtpalert_setrcpt(new_recipient);
   }
   else if (del_recipient) /* delete recipient from list */
   {
      smtpalert_delrcpt(del_recipient);
   }
   else     /* Just update the other data */
   {
      smtp_serverip = smtp_server;
      strncpy(smtp_sendername, login_name, MAX_RCPT_LEN);
      smtpalert_enabled = enable_email;
      if(password)
         strncpy(smtp_password, password, MAX_RCPT_LEN);
   }

   br_saveparms();

   wi_redirect(sess, "saveset.htm");
   return NULL;
}

#else /* SMTP_ALERTS */
int
ht_email(wi_sess * sess, EOFILE * eofile)
{
   dtrap();
   return ENP_PARAM;
}

char *
emailcfg_cgi(wi_sess * sess, EOFILE * eofile)
{
   dtrap();
   return ("no smtp alerts");
}
#endif   /* SMTP_ALERTS */



/* FUNCTION: check_permit()
 *
 * CRIBBED from original in ../misclib/userpass.c -JB-
 *
 * check_permit() - This is called from applciations which want to 
 * verify a user/password combination. Two extra parameters are 
 * provided for the application's use. These are: - appcode is one of 
 * the defines in userpass.h. this will identify the calling module. 
 * - permissions. This may be used by apps as needed. 
 *
 * 
 * PARAM1: char *    username
 * PARAM2: char *    password
 * PARAM3: int       appcode
 * PARAM4: void *    permissions
 *
 * RETURNS: TRUE if use is allowed, FALSE if not.
 */

int
check_permit(char * username, 
   char *   password, 
   int      appcode,       /* application which called us */
   void *   permissions)   /* NULL if not used */
{
   int   i;

   dtrap();

   for (i = 0; i < BR_MAXUSERS; i++)
   {
      if (strcmp(users[i].username, username) == 0)
         break;
   }

   if (i >= BR_MAXUSERS)
      return FALSE;


   if (strcmp(users[i].password, password) != 0)
      return FALSE;

   /* add any further checking based on appcode & permissions here: */
   USE_ARG(appcode);
   USE_ARG(permissions); 

   return TRUE;   /* this user is allowed in */
}


struct file_permit
{
   char * filename;
   u_long mask;
} 
file_permits[] = 
{
"addcomm.cgi", UF_EDITBLOCK,
"addcomm.htm", UF_EDITBLOCK,
"addword.cgi", UF_CONTENT,
"alertcfg.cgi", UF_ALERTS,
"alertcfg.htm", UF_ALERTS,
"bfiletp.htm", UF_EDITBLOCK,
"bipaddr.htm", UF_EDITBLOCK,
"blockbar.htm", 0,
"blocklog.htm", UF_VIEWBLOCK,
"bl_ip.cgi", UF_EDITBLOCK,
"bl_port.cgi", UF_EDITBLOCK,
"bl_name.cgi", UF_EDITBLOCK,
"blklog.htm", UF_VIEWBLOCK,
"bport.htm", UF_VIEWBLOCK,
"cm_opt.cgi", 0,
"comms.htm", UF_VIEWBLOCK,
"commbar.htm", 0,
"config.cgi", UF_VIEWBLOCK,
"config.htm", UF_VIEWPREF,
"connlist.htm", UF_VIEWHOST | UF_VIEWSELF,
"connmap.htm", UF_VIEWHOST | UF_VIEWSELF,
"connview.htm", UF_KILLCONN,
"content.htm", UF_CONTENT,
"delmac.htm", UF_KILLCONN,
"del_hide.cgi", UF_VIEWMAP,
"deluser.cgi", UF_EDITUSER,
"editbip.htm", UF_EDITBLOCK,
"editft.cgi", UF_EDITBLOCK,
"editft.htm", UF_EDITBLOCK,
"editurl.cgi", UF_EDITBLOCK,
"editurl.htm", UF_VIEWBLOCK,
"editword.cgi", UF_EDITBLOCK,
"emailcfg.cgi", UF_EDITBLOCK,
"email.htm", UF_SPAMEDIT,
"emailbar.htm", 0,
"emailcfg.htm", UF_VIEWPREF,
"helpbar.htm", 0,
"helpmain.htm", 0,
"hostinfo.htm", UF_VIEWPREF,
"hostname.cgi", UF_EDITPREF,
"index.htm", UF_VIEWMAP,
"iphosts.htm", UF_VIEWMAP,
"killcn.cgi", UF_KILLCONN,
"lanman.cgi", UF_EDITBLOCK,
"lanman.htm", UF_EDITBLOCK,
"logo.htm", 0,
"logo2.htm", 0,
"machost.htm", UF_VIEWMAP,
"machosts.htm", UF_VIEWMAP,
"menus.htm", 0,
"myserver.htm", UF_VIEWPREF,
"netbar.htm", 0,
"netbar.cgi", UF_VIEWMAP,
"netcfg.cgi", UF_EDITBLOCK,
"netcfg.htm", UF_VIEWIP,
"netmap.htm", UF_VIEWMAP,
"prefbar.htm", 0,
"pixsel.htm", UF_EDITPREF,
"recent.htm", UF_VIEWHOST | UF_VIEWSELF,
"saveset.htm", 0,
"setcip.cgi", UF_EDITBLOCK,
"set_defip.cgi", UF_EDITBLOCK,
"setnet.htm", UF_EDITPREF,
"unbkform.htm", UF_EDITBLOCK,
"unblkip.htm", UF_EDITBLOCK,
"unblk_path.htm", UF_EDITBLOCK,
"unblock.cgi", UF_EDITBLOCK,
"unblock.cgi", UF_EDITBLOCK,
"unblock.htm", UF_EDITBLOCK,
"unblocked.htm", UF_EDITBLOCK,
"urls.htm", UF_VIEWBLOCK,
"usercfg.cgi", UF_EDITUSER,
"usercfg.htm", UF_VIEWBLOCK,
"viewnet.cgi", UF_EDITBLOCK,
"virus.htm", UF_VIEWBLOCK,
};


#define FP_ENTRIES sizeof(file_permits)/sizeof(struct file_permit)

int
user_ok(char * user,
   char * password,
   void * vfp)
{
   char *   cp;
   int      i;
   int      j;
   EOFILE * eofile;
   char *   filename;

   /* Look up username */
   for (i = 0; i < BR_MAXUSERS; i++)
   {
      if (strcmp(users[i].username, user) == 0)
         break;
   }

   /* If user name not found, refuse */
   if (i >= BR_MAXUSERS)
      return FALSE;

   /* Run password through encryptor and check it: */
   cp = br_pwdigest(password, users[i].usermask);
   if(strcmp(cp, users[i].password) != 0)
      return FALSE;

   eofile = (EOFILE *)vfp;
   if(!eofile || !eofile->eo_emfile)
   {
      dtrap();       /* Bad file parameret */
      return FALSE;  /* Deny access */
   }
   filename = eofile->eo_emfile->em_name;


   /* check users[i].usermask against file restrictions */
   for(j = 0; j < FP_ENTRIES; j++)
   {
      if( *file_permits[j].filename == *filename)
      {
         /* do full compare of file names */
         if(strcmp(file_permits[j].filename, filename) == 0)
         {
            /* See if user has at least one of the needed permission bits */
            if((file_permits[j].mask & users[i].usermask) ||
               (file_permits[j].mask == 0))
            {
               /* Almost there - do the viewself test */
               if(file_permits[j].mask & UF_VIEWSELF)
               {
                  if((users[i].usermask & (UF_VIEWHOST | UF_VIEWMAP)) == 0)
                  {
                     /* make sure request is coming from host to be viewed */
                     dtrap();
                  }
               }
               return TRUE;      /* let user see the file */
            }
            else
            {
               br_log(0, "Valid user %s (password OK) failed to access file %s\n",
                  user, filename);
               return FALSE;
            }
         }
      }
   }

   /* Fall to here if file name not in list */
   dtrap();
   return FALSE;
}

/* titles for cells with user permission bits */
char * uf_titles[] = 
{
"Allow viewing map pages",
"Allow viewing host map for self",
"Allow editing blocked URLs, IPs, etc.",
"Allow viewing host maps",
"Allow viewing IP settings",
"Allow viewing preference settings",
"Allow editing preference settings",
"Allow editing IP settings",
"Allow editing blocked URLs, IPs, etc.",
"Allow editing content vocabularies",
"Allow editing own spam settings",
"Allow viewing email passwords",
"Allow editing alert settings",
"Allow killing conmections",
"Allowed access via telnet",
"Allow editing other users",
};


/* ht_userlist() - SSI routine.
 *
 * List Buster users in usercfg.htm page.
 *
 * Returns 0 if OK. 
 */

int
ht_userlist(wi_sess * sess, EOFILE * eofile)
{
   int      i, j;
   u_long   mask;

   for(i = 0; i < BR_MAXUSERS; i++)
   {
      if(users[i].username[0])
      {
         wi_printf(sess, "<tr><td colspan=18 class=\"preferences\" height=1 ></td></tr>\n");
         wi_printf(sess, "<tr class=\"preferencetint\" ><td><a "
            "href=\"javascript:loadUser('%s', 0x%lx)\" >%s</a></td>\n",
            users[i].username, users[i].usermask, users[i].username );

         mask = users[i].usermask;
         for(j = 0; j < NUM_UFBITS; j++)
         {
            wi_printf(sess,"<td bgcolor=%s width=12  title=\"%s\" ><font size=-1>%c</td>\n",
               (mask&1)?"white":"lightgrey", uf_titles[j], (mask&1)?'Y':'N' );

            mask >>= 1;
         }
         wi_printf(sess, "<td ><input type=button name=deluser "
            "value=\"Delete\" class=\"buttons preferencelt\" " 
            "onMouseOver=\"this.className='buttons bns buttonsover bpreferencesover'\"" 
            "onMouseOut=\"this.className='buttons bns bpreferences'\"" );

         wi_printf(sess, " onClick=\"delete_user( '%s');\" ></td></tr>",
            users[i].username  );
      }
   }
	return 0; /* OK return */
}


/* ht_userdata() - SSI routine.
 *
 * Returns 0 if OK. 
 */

int
ht_userdata(wi_sess * sess, EOFILE * eofile)
{
   char *   username;		/* to be set from form */

   /* print user information into hp connection */
   username = wi_formvalue(sess, "user");
   if(username == NULL)
   {
      return -1;
   }

   wi_printf(sess, "%s", username);

	return 0; /* OK return */
}

/* deluser_cgi()
 *
 * CGI routine.
 *
 * Handle the "yes" click from the "Are you sure?" deluser.htm page
 */

char *
deluser_cgi(wi_sess * sess, EOFILE * eofile)
{
   int      i;
   char *   username;		/* to be set from form */

   /* auto-generated code: */

   username = wi_formvalue(sess, "user");
   if(username == NULL)
      return("bad username");

   /* see if user is already in database */
   for(i = 0; i < BR_MAXUSERS; i++)
   {
      if(strcmp(users[i].username, username ) == 0)
      {
         /* Found user entry, clear name to free entry */
         MEMSET(users[i].username, 0, sizeof(users[i].username) );
         br_saveparms();
         wi_redirect(sess, "usercfg.htm");
         return NULL;
      }
   }

   return("Unable to find user name in database");
}


/* usercfg_cgi()
 *
 * CGI routine.
 * 
 * Extract the user data from form and set it in C structure.
 *
 * Returns bitmask (see manual). 
 */

char * 
usercfg_cgi(wi_sess * sess, EOFILE * eofile)
{
   int      i;
   char *   new_password;  /* to be set from form */
   char *   pw_digest;     /* digested password */
   char *   new_user;      /* to be set from form */
   u_long   mask = 0;

	/* auto-generated code: */

	new_password = wi_formvalue(sess, "new_password");
   /* Wait until we know user before checking password */

   new_user = wi_formvalue(sess, "new_user");
	if(new_user == NULL)
		return("bad new_user");

   if(wi_formbool(sess, "edituser"))
      mask |= UF_EDITUSER;
   if(wi_formbool(sess, "telnet"))
      mask |= UF_TELNET;
   if(wi_formbool(sess, "killconn"))
      mask |= UF_KILLCONN ;
   if(wi_formbool(sess, "alerts"))
      mask |= UF_ALERTS  ;
   if(wi_formbool(sess, "emailpw"))
      mask |= UF_EMAILPW;
   if(wi_formbool(sess, "spamedit"))
      mask |= UF_SPAMEDIT;
   if(wi_formbool(sess, "content"))
       mask |= UF_CONTENT ;
   if(wi_formbool(sess, "editblock"))
       mask |= UF_EDITBLOCK;
   if(wi_formbool(sess, "editip"))
       mask |= UF_EDITIP ;
   if(wi_formbool(sess, "editpref"))
       mask |= UF_EDITPREF;
   if(wi_formbool(sess, "viewpref"))
       mask |=  UF_VIEWPREF;
   if(wi_formbool(sess, "viewip"))
       mask |= UF_VIEWIP;
   if(wi_formbool(sess, "viewhost"))
       mask |= UF_VIEWHOST ;
   if(wi_formbool(sess, "viewblock"))
       mask |= UF_VIEWBLOCK ;
   if(wi_formbool(sess, "viewself"))
       mask |= UF_VIEWSELF;
   if(wi_formbool(sess, "viewmap"))
       mask |= UF_VIEWMAP;


   /* see if user is already in database */
   for(i = 0; i < BR_MAXUSERS; i++)
   {
      if(strcmp(users[i].username, new_user) == 0)
      {
         /* Found user, set mask & password */
         users[i].usermask = mask;

         /* Javascript in the form may allow a null password
          * for existing users. IN this case, DON'T try to save it.
          */
         if(new_password)
         {
            /* Got new password, convert it to digest */
            pw_digest = br_pwdigest(new_password, mask);
            strcpy(users[i].password, pw_digest);
         }
         br_saveparms();
         wi_redirect(sess, "usercfg.htm");
         return NULL;
      }
   }

   /* New users require passwords */
   if(new_password == NULL)
	{
		return("bad new_password");
	}
   /* convert password to digest version */
   pw_digest = br_pwdigest(new_password, mask);

   if(add_user(new_user, pw_digest, mask) == FALSE)
   {
      return("Sorry, can't add user. <br>System alreay has maximum number of users.");
   }

   br_saveparms();
   wi_redirect(sess, "usercfg.htm");
   return NULL;
}

/**************** SSIs & CGI for lanman page ****************/

static char * warning = 
"<br>The current LAN is <b>NOT named,</b> and "
"so information about local computers"
"will not be saved when you shut down Buster. "
"To name the LAN, enter a name (such as \"Home\" "
"or \"My Office\") into the "
"\"Current LAN\" text field below, and then "
"click the \"Set LAN Name\" button.<br>";

/* ht_lanman() - SSI routine.
 *
 * Returns 0 if OK. 
 */

int
ht_lanman(wi_sess * sess, EOFILE * eofile)
{
   char *   parm;
   int      iface;
   int      displayed;
   struct laninfo * ln;

   parm = wi_formvalue(sess, "parm");
   if(!parm)
   {
      dtrap();
      return -1;
   }

   iface = br_ifview;

   switch(*parm)
   {
   case 'l':      /* lans (list of) */
      displayed = 0;
      for(ln = br_lans; ln; ln = ln->next)
      {
         if(ln->flags & LN_TEMP)
            continue;
         wi_printf(sess, " &nbsp;%s<br>\n", ln->name);
         displayed++;
      }
      if(displayed == 0)
         wi_printf(sess, "(None yet)<br>\n");
      break;
   case 'w':      /* warning */
      if(brstates[iface].current_lan[0] == 0)
         wi_printf(sess, warning);
      else
         wi_printf(sess, "&nbsp;");
      break;
   case 'n':      /* name (of current LAN) */
      if(brstates[iface].current_lan[0])
         wi_printf(sess, "%s", brstates[iface].current_lan );
      else
         wi_printf(sess, "(unknown)" );
      break;
   case 'r':      /* router??? */
      if(strcmp("routerIP", parm) == 0)
      {
         if((brstates[iface].br_router) &&
            (brstates[iface].br_router->iphost))
         {
            wi_printf(sess, "%u.%u.%u.%u", 
               PASS_IPADDR(brstates[iface].br_router->iphost->ipaddr) );
         }
         else
            wi_printf(sess, "(unknown)"); 
      }
      else if(strcmp("routerMAC", parm) == 0)
      {
         u_char * mac;
         if(brstates[iface].br_router)
         {
            mac = &brstates[iface].br_router->macaddr[0];
            wi_printf(sess, "%02X%02X%02X%02X%02X%02X", 
               *mac,
               *(mac + 1),
               *(mac + 2),
               *(mac + 3),
               *(mac + 4),
               *(mac + 5) );
         }
         else
            wi_printf(sess, "(unknown)"); 
      }
      else     /* unknown parameter */
      {
         dtrap();
         return -1;
      }
      break;
   default:       /* bad HTML or logic */
      dtrap();
      return -1;
   }

   return 0; /* OK return */
}


/* lanman_cgi()
 *
 * CGI routine.
 *
 */

char *
lanman_cgi(wi_sess * sess, EOFILE * eofile)
{
   char *   lanname;       /* to be set from form */
   char *   setname;       /* to be set from form */
   char *   getrouter;     /* to be set from form */
   char *   deletelan;     /* to be set from form */
   char *   msg;           /* error message */
   char *   cp;            /* scratch */
   u_char   routerMAC[8];  /* to be set from form */
   ip_addr  routerIP;      /* to be set from form */

	/* auto-generated code: */

	lanname = wi_formvalue(sess, "lanname");
	if(lanname == NULL)
	{
		return("bad lanname");
	}
   msg = wi_formipaddr(sess, "routerIP", &routerIP);
	if(msg)
	{
		return("bad routerIP");
	}
	cp = wi_formvalue(sess, "routerMAC");
   if(!cp)
	{
badmac:
      dtrap();
		return("bad routerMAC");
	}
   if(hex2bin(cp, routerMAC, 6))
      goto badmac;


   /* try for each of the submit button values. The one present 
    * is the one that was clicked.
    */
   setname = wi_formvalue(sess, "setname");
   getrouter = wi_formvalue(sess, "getrouter");
   deletelan = wi_formvalue(sess, "delete");

	if(setname)
	{
      struct laninfo * ln;

      /* User wants to set LAN name. First make sure a reasonable
	    * name was typed.
	    */
      if((*lanname < 'A') || (*lanname > 'z'))
      {
   		return("Lan name must start with a letter.");
      }

      for(cp = lanname; *cp; cp++)
      {
         if(*cp == '+')   /* convert plus back to space */
            *cp = ' ';

         /* Check that name is legal chars */
         if(*cp == ' ')
            continue;
         else if(*cp < ' ')
            goto badname;
         else if(*cp > 'z')
            goto badname;
      }

      if((cp - lanname) > MAXNAMELEN)
      {
   		return("LAN name too long");
      }

      /* see if lan name already exists */
      for(ln = br_lans; ln; ln = ln->next)
      {
         if(stricmp(ln->name, lanname) == 0)
         {
            return("LAN by that name already exists");
         }
      }

      if(brstates[br_ifview].br_router == NULL)     /* no router? */
      {
    	   return("LAN router is not known. Please try again later.");
      }

      /* Make sure the router IP address in the form matches the 
       * current router IP address.
       */
      if( MAC_EQ(routerMAC, brstates[br_ifview].br_router->macaddr) == FALSE )
      {
    	   return("LAN (or router) has changed. Please refresh LAN page.");
      }

      /* Find current LAN based on router MAC */
      for(ln = br_lans; ln; ln = ln->next)
         if( MAC_EQ(ln->router_mac, routerMAC) )
            break;

      if(!ln)
      {
         ln = new_lan(lanname, routerMAC, routerIP);
         if(!ln)
       	   return("Error creating LAN.");
      }

      /* passed all test, name the LAN and record it. */
      ln->flags &= ~LN_TEMP;        /* It's no longer temporary */
      strcpy(ln->name, lanname);    /* set perm. name */
      strcpy(brstates[br_ifview].current_lan, lanname);
      brstates[br_ifview].evflags |= EV_NETNAMED;
      br_saveparms( );              /* write LAN info to NV file */
      br_writelan();                /* force immediate network.db update */

      wi_redirect(sess, "netmap.htm");     /* Return to the main map page */

      return NULL;
	}
	else if(deletelan)
   {
      struct laninfo * currentlan;
      u_char router_mac[8];      /* save router while LAN is cleared */

      br_cleardbs();    /* Clear the in-ram database */

      /* If the current LAN is not temporary, delete it. */
      currentlan = br_lanlookup(brstates[br_ifview].current_lan);
      if((currentlan) && 
         ((currentlan->flags & LN_TEMP) == 0))
      {
         MEMCPY(router_mac, currentlan->router_mac, 6);
         del_lan(currentlan);
         brstates[br_ifview].current_lan[0] = 0;

         /* Replace the nuked LAN with a "temp" lan. */
         currentlan = new_lan("temp", router_mac, 0);
         if(currentlan)
            currentlan->flags |= LN_TEMP;      /* Mark as temporary */
         strcpy(brstates[br_ifview].current_lan, "temp");
      }

      wi_redirect(sess, "netmap.htm");  /* Return to the newly cleared map page */
      return NULL;
   }
	else if(getrouter)
	{
	   /* User wants to get or test the router IP address */
      dtrap(); /* deprecated */
	}
	else
	{
		return("Bad button clicked");
	}


   /* just reload the LAN manage page */
   wi_redirect(sess, "lanmap.htm");
   return NULL;

badname:
   return("Lan name may be only letters, numbers or a space");
}


/* ht_imglist() - SSI routine.
 *
 * Produce table with list of available MAC image files.
 *
 * <table ... > tag already printed, this should output <tr>
 * tags and below.
 *
 * For now, we use two sources: 
 * 1) static list of files in the GUI vfs
 * 2) the contents from the br_imagedir directory
 *
 * Returns 0 if OK. 
 */


static char * imagelist = 
"pc.gif\nlaptop.gif\nrte.jpg\nlinux.gif\n"
"msworm.gif\nmailbox1.gif\nmailbox2.gif\niphost.gif\nmspopmsg.gif\nunknown.gif";

static char    thisimage[MAXFILENAME];

int
ht_imglist(wi_sess * sess, EOFILE * eofile)
{
   char * nextimage;
   char * cp;
   int   len;
   int imagect = 0;
   int   source = 0;    /* 0 - static list, 1 - images directory */

   wi_printf(sess, "\n<tr>\n");
   nextimage = imagelist;

   while(nextimage && *nextimage)
   {
      cp = strchr(nextimage, '\n');
      if(cp)
         len = cp - nextimage;
      else
         len = strlen(nextimage);

      if(source > 0)     /* reading the user images directory? */
         len += (strlen(br_imagedir) + 1);

      if(len > MAXFILENAME)
      {
         dtrap();
         return ENP_RESOURCE;
      }

      if(source > 0)     /* reading the user images directory? */
      {
         strcpy(thisimage, br_imagedir);
         strcat(thisimage, "/");
         strcat(thisimage, nextimage);
      }
      else
      {
         strncpy(thisimage, nextimage, len);
      }
      thisimage[len] = 0;

      while(*nextimage >= ' ')
         nextimage++;
      while(*nextimage == '\n')
         nextimage++;

      wi_printf(sess, "<td><img src=\"%s\" height=50 width=70 border=0 >", 
         thisimage );
      wi_printf(sess, "<br>\n <input type=radio name=\"imgfile\" value=\"%s\" "
         "onClick=\"document.forms.pixlist.imgfile.value ='%s'\">\n",
         thisimage, thisimage );
      wi_printf(sess, " %s</td>\n", thisimage);
            
      /* If we were reading the static built-in images list, switch to the 
       * user-uploaded images directory 
       */
      if((*nextimage == 0) && (source == 0))
      {
         nextimage = br_getdir(br_imagedir);
         source++;
      }

      imagect++;
      if((imagect % 5) == 0)
         wi_printf(sess, "</tr><tr>\n");
   }   
   wi_printf(sess, "</tr>\n");

   return 0; /* OK return */
}


/* ht_myservers() - SSI routine.
 *
 * List all the Advertising servers into the srcserver.htm page.
 *
 * Should look something like this (per server)
 *
<tr class="blanktint" >
<td>Praemio Ad Server</td>
<td>http://www.praemio.com/images/Praemio-logo.gif</td>
<td>10</td>
<td>Active</td>

<td align=center > <input type=button name="disable" value="Disable" class="bpreferences" 
   OnClick="toggle_server('Praemio Ad Server')" ></td>

<td align=center > <input type=button name="delete"  value="Delete"  class="bpreferences" 
   OnClick="delete_server('Praemio Ad Server')" ></td>

</tr><tr class="preferences" height=1 ><td colspan=6 height=1></td></tr>

  * Returns 0 if OK. 
 */

struct srcserv srcserver = 
{
   NULL,
   "Praemio SRC Server",
   "http://www.praemio.info/srctest/srcserve.html",
   10,
   TRUE
};
struct srcserv * myservers = &srcserver;

int   br_srcserving = 0;   /* Boolean */

int
ht_myservers(wi_sess * sess, EOFILE * eofile)
{
   int   servers;
   struct srcserv * srcserver;

   servers = 0;
   for(srcserver = myservers; srcserver; srcserver = srcserver->next)
   {
      wi_printf(sess, "<tr class=\"blanktint\" >\n<td>%s</td>\n",
         srcserver->ms_name);
      wi_printf(sess, "<td>%s</td>\n<td>%d</td>\n", 
         srcserver->ms_url, srcserver->ms_weight);
      wi_printf(sess, "<td>%s</td>\n", srcserver->ms_active?"Active":"Disabled" );

      wi_printf(sess, "<td align=center > <input type=button "
         "value=\"%s\" class=\"bpreferences\" OnClick=\"toggle_server('%s')\" ></td>\n", 
         srcserver->ms_active?"disable":"enable", 
         srcserver->ms_name );

      wi_printf(sess, "<td align=center > <input type=button "
         "value=\"Delete\"  class=\"bpreferences\" OnClick=\"delete_server('%s')\" ></td>\n", 
         srcserver->ms_name);

      wi_printf(sess, "</tr><tr class=\"preferences\" height=1 >"
         "<td colspan=6 height=1></td></tr>\n");
      servers++;
   }

   if(servers == 0)
   {
      wi_printf(sess, "<tr class=\"blanktint\" ><td>No Ad Servers Set</td></tr>\n");
   }

   return 0;   /* OK return */
}

/* myserv_cgi()
 *
 * CGI routine.
 */

char *
myserv_cgi(wi_sess * sess, EOFILE * eofile)
{
   char *	server_name;      /* to be set from form */
   char *	operation;        /* to be set from form */
   char *	onoff_button;     /* to be set from form */
   struct srcserv * srcserver;

   /* auto-generated code: */

   operation = wi_formvalue(sess, "operation");
   if(operation == NULL)
   {
      return("bad operation");
   }
   onoff_button = wi_formvalue(sess, "onoff_button");

   if( strcmp(operation, "ms_toggle") == 0 )
   {
      if(strcmp(onoff_button, "Disable") == 0 )
         br_srcserving = 0;
      else
         br_srcserving = 1;

      br_saveparms();

      wi_redirect(sess, "myserver.htm");
      return NULL;
   }
   server_name = wi_formvalue(sess, "server_name");
   if(server_name == NULL)
   {
      return("bad MyServer name");
   }

   for(srcserver = myservers; srcserver; srcserver = srcserver->next)
   {
      if(strcmp(srcserver->ms_name, server_name) == 0 )
         break;
   }
   if(!srcserver)
   {
      dtrap();
      return("Name not found in srcserver list");
   }

   if(srcserver->ms_active == TRUE)
      srcserver->ms_active = FALSE;
   else
      srcserver->ms_active = TRUE;
   
   wi_redirect(sess, "srcserver.htm");
   return NULL;
}

/* ht_msinfo()
 *
 *  SSI routine.
 * 
 * This is called to insert information about the SRC server
 * into the output stream.
 *
 * Returns 0 if OK. 
 */

int
ht_msinfo(wi_sess * sess, EOFILE * eofile)
{
	/* add your code here */
   USE_ARG(sess);
   USE_ARG(eofile);

   dtrap();

	return 0; /* OK return */
}

/* get_form_conn() - CGI helper routine.
 *
 * Gets a connectio pointer from a passed form, where the form 
 * contains the name/value pair "conn=1234" and "1234" is decimal
 * for the memory location of an Buster pointer structure.  
 *
 * Returns pointer if valid connection was passed in form, 
 * NULL if not.
 */

struct conn_entry * 
get_form_conn(wi_sess * sess)
{
   int   err;
   struct conn_entry * conn; 
   struct conn_entry * tmp; 

   err = wi_formint(sess, "conn", (long*)&conn );
   if(err || (conn == NULL))
   {
      dtrap();
      return NULL;
   }

   if((conn < conn_lowest) || 
      (conn > conn_highest))
   {
      /* Stale connection from previous Buster run? */
      dtrap();
      return NULL;
   }

   /* Lookup connection to make sure it didn;t time out while the 
    * page was displayed (could have been days!)
    */
   tmp = br_cnlookup( conn->loc.ce_addr, conn->rem.ce_addr, 
      conn->loc.ce_port, conn->rem.ce_port, conn->ce_type);

   if(tmp != conn)
      return NULL;

   return conn;
}


/* blockpage_cgi()
 *
 * CGI routine.
 *
 * This is called when the user clicks a submit control on 
 * the form page that is used to replace blocked full-screen 
 * web pages.
 */

static char unblock_file[50];

char * 
blockpage_cgi(wi_sess * sess, EOFILE * eofile)
{
   char *   request;
   char *   blocked_page;     /* referrer which was blocked */
   struct conn_entry * conn;

   /* auto-generated code: */
	conn = get_form_conn(sess);
	if(conn == NULL)
	{
		return("Connection info timed out");
	}
   
   /* If blocked page is known, save it in conn->data buffer for unblock.htm, */
   blocked_page = wi_formvalue(sess, "blocked_page");
   if(blocked_page)
   {
      /* Got referrer page, copy it into conneciton */
      strncpy(conn->data, blocked_page, MAXDATALEN);
   }
   else
   {
      sprintf( unblock_file, "http://%u.%u.%u.%u", 
         PASS_IPADDR(conn->rem.ce_addr) );
      strncpy(conn->data, unblock_file, MAXDATALEN);
   }

   request = wi_formvalue(sess, "request");
   if(request == NULL)
   {
      dtrap();
		return("bad request;");
   }

   if(*request == 'i')     /* info */
   {
      sprintf( unblock_file, 
         "hostinfo.htm?host=%u.%u.%u.%u&conn=%d", 
         PASS_IPADDR(conn->rem.ce_addr), conn );
      wi_redirect(sess, unblock_file);
      return NULL;
   }
   else if (*request == 'u')  /* unblock */
   {
      sprintf( unblock_file, 
         "unblock.htm?host=%u.%u.%u.%u&conn=%d", 
         PASS_IPADDR(conn->rem.ce_addr), conn );
      wi_redirect(sess, unblock_file);
      return NULL;
   }

   dtrap();
   
   return("unknown request"); 
}


/* ht_getads()
 *
 *  SSI routine.
 *
 * Get the ad server content for an ad replacement. 
 *
 * This should do nothing if user has ad serving disabled
 *
 * Returns 0 if OK. 
 */

char * madhen = "MyServer content inserted here";


int
ht_getsrc(wi_sess * sess, EOFILE * eofile)
{
	/* add your code here*/

   if(br_srcserving == 0)
      return 0;

   // tmp hack - shove in madhens
      wi_printf(sess, madhen);


   return 0; /* OK return */
}



/* ht_conninfo()
 *
 *  SSI routine.
 * 
 * Inserts information about a connection (usually blocked) into the 
 * invoking html page. .
 * 
 * Returns 0 if OK. 
 */

int
ht_conninfo(wi_sess * sess, EOFILE * eofile)
{
   struct conn_entry * conn; 
   struct ctdesc * ctl;
   char *   bitem;
   char     biptext[21];
   char *   args;

   args = wi_formvalue(sess, "arg");
   if(!args)
   {
      dtrap();
      return ENP_PARAM;
   }

   conn = get_form_conn(sess);
   if(conn == NULL)
   {
      wi_printf(sess, "\"><font color=#FF0000> Web connection info not found - timed out? </font>");
      return -1;
   }

   switch(conn->block.type)
   {
   case  CTF_TYPEIP:
      {  struct control_ip* ipe;
         ip_addr tmp;
         ipe = (struct control_ip*)(conn->block.ptr);
         ctl = &ipe->ctl;
         if(ipe->ci_name)
         {
            bitem = ipe->ci_name->cu_name;
         }
         else
         {
            tmp = htonl(ipe->ci_ipaddr);
            sprintf(biptext, "IP %u.%u.%u.%u", PASS_IPADDR(tmp));
            bitem = biptext;
         }
         break;
      }
   case  CTF_TYPEPATH:
      {
         struct b_path* bip;
         bip = (struct b_path*)(conn->block.ptr);
         ctl = &bip->ctl;
         bitem = bip->pathname;
         break;
      }
   case  CTF_TYPEPORT:
      {
         struct control_port * cpe;
         cpe = (struct control_port*)(conn->block.ptr);
         ctl = &cpe->ctl;
         sprintf(biptext, "Port %u", cpe->cp_port);
         bitem = biptext;
         break;
      }
   case  CTF_TYPENAME:
      {
         struct control_dn * dne;
         dne = (struct control_dn*)(conn->block.ptr);
         ctl = &dne->ctl;
         bitem = dne->cu_name;
         break;
      }
   default:    
      /* This may be unknown type or damaged conn, but 
       * it also happens if the Browser user clicks reload on 
       * inserted ad block page. 
       */
      return 0;
   }

   switch(*args)
   {
   case 'c':  /* Community */
      /* Make sure we don't fault in printing */
      if(!ctl->comm)
      {
         dtrap();
         return -1;
      }
      wi_printf(sess, "%s", br_prtcomm(ctl->comm) );
      break;
   case  'm':   /* motive */
      /* Make sure we don;t fault in printing */
      if(!ctl->offense || !ctl->offense->desc )
      {
         dtrap();
         return -1;
      }
      wi_printf(sess, "%s", ctl->offense->desc );
      break;
   case 'p':   /* pointer to struct */
      wi_printf(sess, "%u", conn );
      break;
   case 'b':   /* blocked item */
      wi_printf(sess, "%s", bitem );
      break;
   case 'u':   /* urlblocked */
      wi_printf(sess, "%s", conn->data );
      break;
   default:
      dtrap(); /* Bad SSI parameter from web page */
      return -1;
   }

   return 0; /* OK return */
}


/* addcomm_cgi()
 *
 * CGI routine.
 *
 * Called when community add form is submitted
 *
 */

char *
addcomm_cgi(wi_sess * sess, EOFILE * eofile)
{
   int      err;     /* numeric parse errors */
   char *   name;    /* to be set from form */
   char *   url;     /* to be set from form */
   int      priority;   /* to be set from form */
   struct comm_dbud *   comm;

	/* auto-generated code: */
	name = wi_formvalue(sess, "name");
	if(name == NULL)
	{
		return("bad name");
	}
	url = wi_formvalue(sess, "url");
	if(url == NULL)
	{
		return("bad url");
	}
	err = wi_formint(sess, "priority", (long*)&priority);
	if(err)
	{
		return("bad priority");
	}


   /* Make sure name & URL and resonable */
   if(*name <= ' ')
   {
      return("Community name must start with a letter.");
   }
   if( strncmp(url, "http://", 7) )
   {
      return("Community URL must start with \"http://\".");
   }

   /* Make sure this community is not a duplicate - either 
    * in name or by URL. If there's a priority duplication then 
    * just decrement the priority of the new community and try 
    * again.
    */

comm_again:
   for(comm = br_communities; comm; comm = comm->next)
   {
      if( stricmp(comm->cm_url, url) == 0)
      {
         return("The URL for the community is already in use.");
      }
      if( stricmp(comm->cm_name, name) == 0)
      {
         return("The name for the community is already in use.");
      }
      if((priority == 0) || (comm->cm_priority == priority))
      {
         priority -= (cticks % 100);
         if(priority < 0)
            priority += 1000;
         goto comm_again;
      }
   }

   /* Fall to here if the form data seems OK */
   comm = br_cmnew(url, name, priority);
   if(!comm)
   {
      return("Unexplained error creating new community.");
   }

   br_saveparms();

   wi_redirect(sess, "comms.htm");
   return NULL;
}

/* ht_commvar()
 *
 *  SSI routine to write out variables related to a community.
 *
 * Returns 0 if OK. 
 */

int
ht_commvar(wi_sess * sess, EOFILE * eofile)
{
   char *   cp;
   char *   args;

   args = wi_formvalue(sess, "arg");
   if(!args)
   {
      dtrap();
      return ENP_PARAM;
   }

   /* add your code here */
   if(strstr(args, "name"))
   {
   	cp = wi_formvalue(sess, "name");
      if(!cp)
         goto error_exit;
      wi_printf(sess, "%s", cp );

   }
   else if(strstr(args, "url"))
   {
	   cp = wi_formvalue(sess, "url");
      if(!cp)
         goto error_exit;
      wi_printf(sess, "%s", cp );
   }
   else if(strstr(args, "priority"))
   {
	   long priority;
      int err = wi_formint(sess, "priority", &priority);

      if(err || !priority)
      {
         priority = 1000 + (cticks % 900);
      }
      wi_printf(sess, "%d", priority );
   }
	return 0; /* OK return */

error_exit:

   /* The addcomm.htm page looks better if we just return without 
    * printing anything - the form controls just show up blank. 
    * Doing error trapping for control default values 
    * problably does does more harm than good.
    */
	return 0;
}

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/

