/* ctlcgi.c
 *
 * Buster Controlled resource pages SSI and CGI routines
 *
 * ROUTINES: ht_docip(), ctl_notes(), br_ctlstats(), bt_section(), 
 * ROUTINES: dntree_helper(), ht_urllist(), tptree_helper(), ht_ftlist(), 
 * ROUTINES: ht_ftbkinfo(), editft_cgi(), iptree_helper(), ht_biplist(), 
 * ROUTINES: listpop(), ht_pcllist(), ht_pcluser(), ht_cip(), setcip_cgi(), 
 * ROUTINES: ht_urlbkinfo(), ht_list_motives(), editurl_cgi(), bl_ip_cgi()
 * ROUTINES: ht_bportlist(), porttree_helper(), 
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 */

#include "busterport.h"   /* Praemio */
#include "brnet.h"
#include "ip.h"
#include "brdb.h"
#include "brdns.h"

#include "websys.h"
#include "webio.h"
#include "webfs.h"
#include "wsfdata.h"

#include "busthtml.h"


static int odd_even;    /* tracker for odd/even line entries */

static char xbuf[100];  /* scratch */

/* Defaults for filtering display */
int   def_reason = 0xFFFFFFFF;   /* filter for spyware, ads, content, etc. */
int   def_limit = 100;           /* Max entries per page */
char  def_name[MAXURL] = {"a"};  /* filter for name. */
int   def_ipbytes[4] = { 256,256,256,256 };
struct comm_dbud *  def_comm;    /* filter for community. */


/* helper routine to output IP address ranges */

void
ht_docip(wi_sess * sess, struct control_ip * cip)
{
   ip_addr  ipaddr;

   if(cip)
   {
      ipaddr = htonl(cip->ci_ipaddr);

      wi_printf(sess, "%u.%u.%u.%u", PASS_IPADDR(ipaddr) );
   }
   else
   {
      wi_printf(sess, "(unknown)");
   }

   return;
}

/* ctl_notes - output the "notes" field to a blocked URL/IP/path 
 * entry line.
 */

void
ctl_notes(wi_sess * sess, 
          struct ctdesc * ctl, 
          char * item)
{
   int   access = ctl_getaccess(ctl);

   if(access == 2)
   {
      /* show minutes left in "allow timer". round tick count upwards */
      wi_printf(sess, "<td title=\"This %s is enabled for a limited number "
         "of minutes, then it reverts to being blocked \" > %d minutes</td>\n", 
            item, ((ctl->timeout - cticks)/MINUTE) +1 );
   }
   else if(access == 1)
   {
      wi_printf(sess, "<td title=\"Access is explicitly PERMITTED"
         " to this %s.\" > whitelist</td>\n", item );
   }
   else
   {
      if(access != 0)
      {
         dtrap();
      }
      wi_printf(sess, "<td title=\"Access is blocked\" >"
         " blacklist</td>\n" );
   }
   return;
}

/* br_settmmetric()
 *
 * Convert a passed cticks count into printable units, and return 
 * a little string for the tyoe of units 
 */

void
br_settmmetric(u_long tickct,    /* IN - cticks to measure */
               int * amount,     /* OUT - rounded time unit count */
               char ** units)    /* OUT - string describing units */
{

   if(tickct < (120 * SECOND))
   {
      *amount = tickct/SECOND;
      *units = "seconds";
   }
   else if(tickct < (120 * MINUTE))
   {
      *amount = tickct/MINUTE;
      *units = "minutes";
   }
   else if(tickct < (120 * HOUR))
   {
      *amount = tickct/HOUR;
      *units = "hours";
   }
   else     /* time is more than 2 hours */
   {
      *amount = tickct/DAY;
      *units = "days";
   }

   return;
}

void
br_ctlstats(wi_sess * sess, u_long actions, u_long lasttick)
{
   char *   cp;
   int      age;     /* age of last action, in cticks */

   wi_printf(sess, "<td title=\"Number of times this entry caused a block\" >"
      "%u</td>", actions );

   if(lasttick == 0)
   {
      wi_printf(sess, "<td title=\"time since this entry caused a block\" >N/A</td>");
   }
   else
   {
      br_settmmetric(cticks - lasttick, &age, &cp);
      wi_printf(sess, "<td title=\"time since this entry caused a block\" >"
         "%u <font size=-2>%s</td>", 
            age, cp );
   }
   return;
}

/************* DNS name blocking routines routines ****************/

/* helper routine for SSI routine below. This examines the passed bt node (which 
 * had better be in the dn_tree) and prints web page a formatted line if it 
 * matches the current filter settings.
 *
 * returns 1 if it printed a line, 0 if it did not.
 */

int
dntree_helper(struct btnode * n, long param, int depth)
{
   wi_sess * sess;
   struct control_dn * dnp;
   struct control_ip * cip;
   struct ctdesc * ctl;
   char *   cssclass;
   u_long   actions;       /* for totaling DNS & IP blocking stats */
   u_long   lasttick;

   sess = (wi_sess *)param;
   dnp = (struct control_dn *)n->data;
   if(!dnp || !dnp->ctl.offense)
   {
      dtrap();    /* CGI somewhere forgot to set DNS offense???? */
      return 0;
   }
   ctl = &dnp->ctl;

   /* skip entries which don;t match the class being displayed */
   if(def_reason && ((ctl->offense->code & def_reason) == 0))
   {
      return 0;
   }

   /* Skip entries not in the default display community */
   if(def_comm && (def_comm != ctl->comm))
   {
      return 0;
   }

   /* Skip entries not matching the default display pattern */
   if(def_name[0] && (strncmp(dnp->cu_name, def_name, strlen(def_name))))
   {
      return 0;
   }

   if(odd_even++ & 1)
      cssclass = "blockingtint";
   else
      cssclass = "blanktint";

   /* Make a pretty trim line */
   wi_printf(sess, "<tr class=\"blocking\" height=1 ><td height=1 colspan=8 ></td></tr>"); 

   /* Print domain name */
   wi_printf(sess, "<tr class=\"%s\" ><td title=\"Domain being blocked, "
      "Click to Edit\" ><a href=\'editurl.htm?url=%s\'>%s</a></td>\n",
         cssclass, dnp->cu_name, dnp->cu_name);

   /* print motive for blocking it */
   wi_printf(sess, "<td title=\"Reason for blocking this domain\" >%s</a></td>\n", 
      ctl->offense->desc);

   /* Print IP address. */
   cip = dnp->cu_cip;
   wi_printf(sess, "<td title=\"IP address(es) used by this domain\" ><font size=-1>");
   ht_docip(sess, cip);
   wi_printf(sess, "</font></td>\n");

   /* Call ctl_getaccess to force update of access timer. */
   ctl_getaccess(ctl);

   wi_printf(sess, "<td title=\"Community this URL came from\" >%s</td>", 
      br_prtcomm(ctl->comm) );

   /* print stats for this controlled item. For DNS entries, factor in the 
    * actions count and last tick time of the associates IP entries.
    */
   actions = ctl->actions;
   lasttick = ctl->lasttick;
   if(dnp->cu_cip) 
   {
      cip = dnp->cu_cip;
      actions = cip->ctl.actions;
      lasttick = cip->ctl.lasttick;
   }
   br_ctlstats(sess, actions, lasttick);

   /* print any flags or odd conditions */
   ctl_notes(sess, ctl, "domain");

   wi_printf(sess, "<td class=\"buttons bblocking\" ><a href='unblkip.htm?host=%s'>",
      dnp->cu_name );

   /* If it's whitelisted make a button to block it, otherwise unblock */
   if( WHITELISTED(ctl) )
   {
      wi_printf(sess,"Block</a></td></tr>\n");
   }
   else
   {
      wi_printf(sess,"Unblock</a></td></tr>\n"); 
   }

   /* If there is a list of additional ranges, loop through it */
   if(dnp->cu_cip && dnp->cu_cip->ci_dnlist)
   {
      for(cip = dnp->cu_cip->ci_dnlist; cip; cip = cip->ci_dnlist)
      {
         /* Sanity test for looped list */
         if(cip->ci_dnlist == dnp->cu_cip)     /* list got looped! */
         {
            dtrap();    /* logic error somewhere... */
            return 1;
         }
         if(odd_even++ & 1)
            cssclass = "blockingtint";
         else
            cssclass = "blanktint";


         /* Make a pretty trim line */
         wi_printf(sess, "<tr class=\"blocking\" height=1 ><td height=1 colspan=8 ></td></tr>"); 

         wi_printf(sess, "<tr class=\"%s\" ><td align=center >&quot;</td><td></td>"
            "<td title=\"More IP address(es) used by this domain\"><font size=-1>",
            cssclass);
         ht_docip(sess, cip);
         wi_printf(sess, "</font></td>");

         br_ctlstats(sess, cip->ctl.actions, cip->ctl.lasttick);
         wi_printf(sess, "<td></td><td></td></tr>\n");
      }
   }

   USE_ARG(depth);
   return 1;
}

/* bt_section() - version of bt_depthfirt() which only process 
 * a portion of a tree. Start item is indicated by passed key, 
 * we then process (*count) items. This is a helper 
 * for displaying the btrees of blocklist data in 
 * increments of 100 lines or so. 
 *
 * A NULL is passed to start at the 
 * first item (the lowest in the tree.)
 *
 * Returns next node to do work on, NULL if done.
 */


struct btnode *
bt_section(
   struct btnode * n,   /* tree to work in */
   int (* workfunc)(struct btnode *, long param, int depth),
   int (* cmpfunc) (struct btnode *, u_long key),
   long     param,      /* parameter for work function */
   u_long   key,
   int      depth,
   int *    doit,       /* flag for "are we working yet?" */
   int *    count)      /* number of items remaining to process */
{
   enum BT_REL    avlrel;
   struct btnode * next;

   if(!n)
      return NULL;

   if(*count <= 0)
      return n;

   /* If n is below key range Don;t bother with left branch */
   avlrel = cmpfunc(n, key);
   if(avlrel != KEY_GT)
   {
      next = bt_section(n->left, workfunc, cmpfunc, param, key, depth++, doit, count);
      if(*count <= 0)
      {
         if(next)
            return next;
         else
            return n;
      }
   }

   if(avlrel != KEY_GT)
   {
      *count -= workfunc(n, param, depth++);
      if(*count <= 0)
      {
         /* Find the next item. "n" and n->left are done, so get the 
          * leftmost item below n->right.
          */
         next = n->right;
         while(next && next->left)
            next = next->left;
         return next;
      }
   }

   next = bt_section(n->right, workfunc, cmpfunc, param, key, depth++, doit, count);
   if(*count <= 0)
   {
      if(next)
         return next;
      else
         return NULL;
   }
   return next;
}

/* ht_urllist()
 *
 * SSI routine.
 *
 * Returns 0 if OK.
 */

int
ht_urllist(wi_sess * sess, EOFILE * eofile)
{
   char *   type;
   char *   commname;
   char *   match;
   char *   last;
   int      workflag;
   int      lines;
   struct btnode * next;

   /* See if the page has a name/value qualifier to filter types */
   type = wi_formvalue(sess, "type");
   if(type)
   {
      if(*type == 's')
         def_reason = OF_SPYWARE;
      else if(*type == 'a')
         def_reason = OF_ADVERTS;
   }
   match = wi_formvalue(sess, "match");
   if(match)
   {
      if(*match)
      {
         strcpy(def_name, match);   /* This may be "" */
      }
      else 
      {
         /* "match" was specified in form but has null value */
         def_name[0] = 0;
      }
   }
   last = wi_formvalue(sess, "lastentry");

   /* Set filter community is specified by caller */
   commname = wi_formvalue(sess, "community");
   if(commname)
   {
      def_comm = br_cmbyfile(commname);
   }

   odd_even = 1;     /* tint first row */
   lines = def_limit;

   /* Resume printing at last entry printed. If no "last" then 
    * start at lowest value item - an empty string. 
    */
   if(last == NULL)
   {
      last = "";
      workflag = TRUE;  /* start with forst item */
   }
   else
      workflag = FALSE; /* find last before starting working */

   /* traverse the btree, printing info in lines */
   next = bt_section((struct btnode *)dn_tree, 
      dntree_helper, 
      bt_dntest, 
      (u_long)sess, 
      (long)(last), 
      0,
      &workflag,
      &lines );

   if(next)
   {
      wi_printf(sess, "<td align=center colspan=8 >"
         "<input type=button name=\"more\" value=\"Next %u Entries\" ",
            def_limit);
      wi_printf(sess, " onClick=\"document.location='urls.htm?lastentry=%s'\" " 
         " class=\"buttons bblocking\" ></td>\n", 
         ((struct control_dn *)next->data)->cu_name );
   }
   else if(lines == def_limit)
   {
      wi_printf(sess, "<td align=center colspan=8 >No matching items found.</td>");
   }

   return 0; /* OK return */
}

/* ht_commsel()
 *
 * SSI routine.
 *
 * Build a select control for the communities.
 *
 * Returns 0 if OK. 
 */

static 
char * cm_selected(struct comm_dbud * comm)
{
   if(comm == def_comm)
      return "selected";
   else
      return "";
}

int
ht_commsel(wi_sess * sess, EOFILE * eofile)
{
   char * name;
   struct comm_dbud * comm;

   /* See if a new community is being selected */
   name = wi_formvalue(sess, "community");
   if(name)
   {
      /* Establish new default selection before displaying list */
      comm = br_cmbyfile(name);
      if(comm)
         def_comm = comm;
      else  /* comm may be NULL becasue name was "--All--" */
      {
         if(strcmp(name, "--All--") == 0)
            def_comm = NULL;      /* comm==NULL indicates all communities */
      }
   }

   wi_printf(sess, "<select name=\"community\"  class=\"pulldowns\" >\n");
   wi_printf(sess, "<option value=\"--All--\" %s> All Communities</option>\n",
      cm_selected(&local_comm) );
   /* read in database files for the various Communities */
   for(comm = br_communities; comm; comm = comm->next)
   {
      name = br_prtcomm(comm);
      wi_printf(sess, "<option value=\"%s\" %s > %s</option>\n",
         name, cm_selected(comm), name);
   }
   wi_printf(sess, "</select>");

	return 0; /* OK return */
}

/* ht_ipfilter()
 *
 * SSI routine.
 *
 * Display the current IP filter value. Use blanks for wildcard.
 *
 * 
 * Returns 0 if OK. 
 */

int
ht_ipfilter(wi_sess * sess, EOFILE * eofile)
{
   char * byteptr;      /* This is called 4 times for bytes 1-4 */
   int   bytenum;

   byteptr = wi_formvalue(sess, "byte");
   if(!byteptr)
   {
      dtrap();
      return -1;
   }

   bytenum = (int)(*byteptr);

   /* On first byte, see if the user is changing default */
   switch(bytenum)
   {
   case '1':
      /* lastly, output the value of first (high order) byte */
      if(def_ipbytes[0] > 255)
         wi_printf(sess, " ");
      else
         wi_printf(sess, "%u", def_ipbytes[0] );
      break;
   case '2':
      if(def_ipbytes[1] > 255)
         wi_printf(sess, " ");
      else
         wi_printf(sess, "%u", def_ipbytes[1] );
      break;
   case '3':
      if(def_ipbytes[2] > 255)
         wi_printf(sess, " ");
      else
         wi_printf(sess, "%u", def_ipbytes[2] );
      break;
   case '4':
      if(def_ipbytes[3] > 255)
         wi_printf(sess, " ");
      else
         wi_printf(sess, "%u", def_ipbytes[3] );
      break;
   default:
      dtrap(); /* Bad HTTP construction */
   }
	return 0; /* OK return */
}


/* ht_listdef()
 *
 * SSI routine.
 * 
 * Get one of the display defaults - def_comm, def_name, etc.
 * 
 * Returns 0 if OK. 
 */

int
ht_listdef(wi_sess * sess, EOFILE * eofile)
{
   char * deftype;

   deftype =  wi_formvalue(sess, "deftype");
   if(!deftype)
   {
      dtrap();
      return -1;
   }
   if(strstr(deftype, "name"))
      wi_printf(sess, "%s", def_name);
   else if(strstr(deftype, "comm"))
      wi_printf(sess, "%s", br_prtcomm(def_comm));
   else if(strstr(deftype, "reason"))
      wi_printf(sess, "%s", motives[def_reason].desc );

	return 0; /* OK return */
}

/************* Protocol PORT routines ****************/

void
porttree_helper(struct btnode * n, long param, int depth)
{
   wi_sess * sess;
   struct control_port * pcp;
   char *   cssclass;
   char *   appname;
   char *   direction;
   struct ctdesc * ctl;

   sess = (wi_sess *)param;
   pcp = (struct control_port *)n->data;
   ctl = &pcp->ctl;

   if(odd_even++ & 1)
      cssclass = "blockingtint";
   else
      cssclass = "blanktint";

   /* Make a pretty trim line */
   wi_printf(sess, "<tr class=\"blocking\" height=1 ><td height=1 colspan=9 ></td></tr>"); 

   wi_printf(sess, "<tr class=\"%s\" height=21 ><td>%u</td><td>%s</td>\n", 
      cssclass, pcp->cp_port, pcp->cp_flags & CPF_TCP?"TCP":"UDP" );

   appname = br_portname(pcp->cp_port);
   if(!appname)
      appname = "Unknown";
   if(pcp->cp_flags & CPF_LOCAL)
   {
      if(pcp->cp_flags &  CPF_REMOTE)
         direction = "Both";
      else
         direction = "Outgoing";
   }
   else
      direction = "Incoming";

   wi_printf(sess, "<td>%s</td><td>%s</td>\n", 
      appname, direction);

   wi_printf(sess, "<td title=\"Community this Port block came from\" >%s</td>", 
      br_prtcomm(ctl->comm) );

   wi_printf(sess, "<td title=\"Reason for blocking this type of file\" >"
      "%s</td>", ctl->offense->desc );

   /* print stats for this controlled item */
   br_ctlstats(sess, ctl->actions, ctl->lasttick);

   /* Output special notes, like timer or whitelist settings */
   ctl_notes(sess, ctl, "file name text");

   wi_printf(sess, "</tr>\n");

   USE_ARG(depth);

   return;
}


/* ht_bportlist()
 *
 * SSI routine.
 *
 * List the blocked ports. Obey the default community filter. 
 * Protocol filter may be passed.
 *
 * Returns 0 if OK. 
 */

int
ht_bportlist(wi_sess * sess, EOFILE * eofile)
{
   char *   protocol;

   /* See if a new community is being selected */
   protocol = wi_formvalue(sess, "prot");
   if(!protocol)
   {
      protocol = "both";   /* default to both UDP and TCP */
   }

   odd_even = 1;     /* tint first row */

   /* traverse the btree, printing info in lines */
   if((*protocol == 'u') || (*protocol == 'b'))
      bt_depthfirst( (struct btnode *)udpp_tree, porttree_helper, (u_long)sess, 0);
   if((*protocol == 't') || (*protocol == 'b'))
      bt_depthfirst( (struct btnode *)tcpp_tree, porttree_helper, (u_long)sess, 0);

	return 0; /* OK return */
}



/************* type/pathname routines ****************/
/* helper routine for ht_ftlist below */

void
tptree_helper(struct btnode * n, long param, int depth)
{
   wi_sess * sess;
   struct b_path * bp;
   char *   cssclass;
   char *   btn_name;
   struct ctdesc * ctl;

   sess = (wi_sess *)param;
   bp = (struct b_path *)n->data;
   ctl = &bp->ctl;

   if(odd_even++ & 1)
      cssclass = "blockingtint";
   else
      cssclass = "blanktint";

   /* Make a pretty trim line */
   wi_printf(sess, "<tr class=\"blocking\" height=1 ><td height=1 colspan=8 ></td></tr>"); 

   wi_printf(sess, "<tr class=\"%s\" ><td title=\"URLs containing this "
      "text are blocked. Asteriscs (*) are 'wildcards'.\" >"
      "<a href='unblk_path.htm?path=%s' > %s%s%s</a></td>",
      cssclass,
      bp->pathname,
      bp->flags & BP_LEADWILD?"*":"",
      bp->pathname,
      bp->flags & BP_TAILWILD?"*":""
      );
   if(bp->ctl.offense == NULL)
      bp->ctl.offense = &motives[0];
   wi_printf(sess, "<td title=\"Reason for blocking this type of file\" >"
      "%s</td>", bp->ctl.offense->desc );

   wi_printf(sess, "<td title=\"Community this pattern came from\" >%s</td>", 
      br_prtcomm(ctl->comm) );

   /* print stats for this controlled item */
   br_ctlstats(sess, bp->ctl.actions, bp->ctl.lasttick);

   /* Output special notes, like timer or whitelist settings */
   ctl_notes(sess, ctl, "file name text");

   /* If it's whitelisted make a button to block it, otherwise unblock */
   if( WHITELISTED(ctl) )
      btn_name = "Block";
   else
      btn_name = "Unblock";

   wi_printf(sess, "<td  class=\"buttons bblocking\" ><a href='unblk_path.htm?path=%s' >"
      "%s</a></td></tr>\n",
      bp->pathname, btn_name);

   USE_ARG(depth);
   return;
}


/* ht_ftlist()
 *
 * SSI routine.
 * 
 * Displays list of blocked blocked pathnames
 *
 * Returns 0 if OK. 
 */

int
ht_ftlist(wi_sess * sess, EOFILE * eofile)
{
   odd_even = 1;     /* tint first row */

   /* traverse the btree, printing info in lines */
   bt_depthfirst( (struct btnode *)path_tree, tptree_helper, (u_long)sess, 0);

	return 0; /* OK return */
}

/* ht_ftbkinfo()
 *
 * SSI routine.
 *
 * Displays one of several kinds of information about a blocked pathname
 * (aka filetype). Types displayed are controlled by a single char arg:
 * 'n' - name with optional wildcard asteriscs
 * 'o' - name by itself (no asteriscs)
 * 't' - temp. unblock timer, displayed in minutes
 * 
 * Returns 0 if OK. */

int
ht_ftbkinfo(wi_sess * sess, EOFILE * eofile)
{
   char * path;
   struct b_path * bp;
   char *   args;

   args = wi_formvalue(sess, "arg");
   if(!args)
   {
      dtrap();
      return ENP_PARAM;
   }

   /* If this is a new entry, there will be no previous info to 
    * display, and thus no form. Just return and let these be blank.
    */
   path = wi_formvalue(sess, "path");
   if(path == NULL)
      return 0;

   bp = path_lookup(path);
   if(!bp)
   {
      dtrap();
      return -1;
   }

   args++;
   switch(*args)
   {
   case 'n':   /* name */
      wi_printf(sess, "%s%s%s", 
         (bp->flags & BP_LEADWILD)?"*":"",
         path,
         (bp->flags & BP_TAILWILD)?"*":"");
      break;
   case 'o':   /* oldname - just plain "name" */
      wi_printf(sess, path);
      break;
   case 't':   /* timer */
      if(bp->ctl.timeout > cticks)
         wi_printf(sess, "%d", (bp->ctl.timeout - cticks)/MINUTE );
      else
         bp->ctl.timeout = 0;
      break;
   default:
      dtrap();
   }

	return 0; /* OK return */
}


/* editft_cgi()
 *
 * CGI routine.
 *
 * Adds a new "blocked" file type to path list. 
 *
 * 3/15/05 - Removed references to a vaiable called "oldname". I'm not 
 * sure why this was there, but it may be leftover from a previous 
 * use of this form to edit existing blocked file type info. Anyway
 * "oldname" was not set in the form (it was a hidden input in the html)
 * and was causing this code to return a "bad oldname" error. I removed 
 * all references to oldname and now intend this form to be solely for
 * entering new name data - not edititing existing data. -JB-
 *
 */

char *
editft_cgi(wi_sess * sess, EOFILE * eofile)
{
   char *   name;       /* path/name after used editing */
   char *   motive;     /* motive for blocking */
   struct b_path *   bp;

	/* auto-generated code: */

	name = wi_formvalue(sess, "name");
	if(name == NULL)
	{
		return("bad name");
	}
	motive = wi_formvalue(sess, "reason");
	if(motive == NULL)
	{
      dtrap();    /* HTML should not allow missing motive */
   	motive = "Other";
   }

   /* See if the name is already in the database */
   bp = path_lookup(name);
   if(bp)
   {
      dtrap();
      return("This file name already exists in the database. \
          If you want to change it, please delete the existing \
          entry first.");
   }

   /* build a new b_path structure */
   bp = new_path(name, &local_comm);   /* local community */
   if(!bp)
      return("alloc error");

   bp->ctl.offense = mot_byname(motive);

/* what about these???:
   bp->ctl.comment = ctl.comment;
   bp->ctl.action_mask |= ctl.action_mask;
*/

   /* send user back to the blocked "file types" (path) list */
   wi_redirect(sess, "bfiletp.htm");
   return NULL;
}


/************* IP address blocking routines ****************/

/* helper routine for ht_biplist(
 *
 * SSI routine below.
 *
 * returns 1 if it printed a line, 0 if it did not.
 */

int
iptree_helper(struct btnode * n, long param, int depth)
{
   wi_sess * sess;
   struct control_ip * cip;
   struct control_dn * dnp;
   struct ctdesc * ctl;
   char *   cssclass;
   char *   btn_value;
   ip_addr  ipaddr;

   cip = (struct control_ip *)(n->data);
   ctl = &cip->ctl;
   sess = (wi_sess * )param;
   ipaddr = htonl(cip->ci_ipaddr);

   /* If we're have a community filter, check it */
   if(def_comm)
   {
      if(def_comm != ctl->comm)
         return 0;
   }

   /* Check against the blocked IP address display filter */
   if(def_ipbytes[0] < 256)
   {
      if( (int)((cip->ci_ipaddr & 0xFF000000) >> 24) != def_ipbytes[0])
         return 0;
   }
   if(def_ipbytes[1] < 256)
   {
      if( (int)((cip->ci_ipaddr & 0xFF0000) >> 16) != def_ipbytes[1])
         return 0;
   }
   if(def_ipbytes[2] < 256)
   {
      if( (int)((cip->ci_ipaddr & 0xFF00) >> 8) != def_ipbytes[2])
         return 0;
   }
   if(def_ipbytes[3] < 256)
   {
      if( (int)(cip->ci_ipaddr & 0xFF) != def_ipbytes[3])
         return 0;
   }

   if(odd_even++ & 1)
      cssclass = "blockingtint";
   else
      cssclass = "blanktint";

   /* Make a pretty trim line */
   wi_printf(sess, "<tr class=\"blocking\" height=1 ><td height=1 colspan=8 ></td></tr>"); 

   wi_printf(sess, "<tr class=\"%s\" ><td title=\"IP address being blocked\" >"
      " <a href=\"editbip.htm?ip=%u.%u.%u.%u\" ><font size=-1>", 
      cssclass, PASS_IPADDR(ipaddr) );
   wi_printf(sess, " %u.%u.%u.%u </font></a></td>", PASS_IPADDR(ipaddr) );

   /* print motive for blocking it */
   if(cip->ctl.offense)
   {
      wi_printf(sess, "<td title=\"Reason these IPs are blocked\" > %s </a></td>\n", 
         cip->ctl.offense->desc);
   }
   else
   {
      wi_printf(sess, "<td></td>\n");
   }

   dnp = cip->ci_name;
   if(dnp)
   {
      wi_printf(sess, "<td title=\"Domain (site) using these addresses\" >"
         "<a  href=\"editurl.htm?url=%s\" > &nbsp; %s &nbsp; </td></a>", 
         dnp->cu_name, dnp->cu_name  );
   }
   else
      wi_printf(sess, "<td title=\"No domain name info for these addresses\" >- - -</td>");


   /* If temp. access timer expired, clear before printing */
   if(ctl->timeout)
   {
      if(ctl->timeout <= cticks)   /* has access timer expired? */
         ctl->timeout = 0;         /* clear it, fall to tcp reset */
   }

   wi_printf(sess, "<td title=\"Community this Port block came from\" >%s</td>", 
      br_prtcomm(ctl->comm) );

   /* print stats for this controlled item */
   br_ctlstats(sess, ctl->actions, cip->ctl.lasttick);

   /* If this was created via blocked domain, show timeout */
   if(cip->ci_flags & CIF_USER)
   {
      char * units;
      int    tmo;

      br_settmmetric(cip->ci_timeout, &tmo, &units);
      wi_printf(sess, "<td title=\"IP Addess set by DNS, time out shown.\" >"
         "%d <font size=-2 > %s </font) </td>", tmo, units );
   }
   else
   {
      /* print any flags or odd conditions */
      ctl_notes(sess, ctl, "IP address");
   }

   /* If it's whitelisted make a button to block it, otherwise unblock */
   if( WHITELISTED(ctl) )
      btn_value = "Block";
   else
      btn_value = "Unblock";

   wi_printf(sess, "<td class=\"buttons bblocking\" ><a href='unblkip.htm?host=%u.%u.%u.%u' >"
      "%s</td></tr>\n", 
      PASS_IPADDR(ipaddr), btn_value );

   USE_ARG(depth);
   return 1;
}
   
/* ht_biplist()
 *
 * SSI routine.
 *
 * Returns 0 if OK. 
 */

int
ht_biplist(wi_sess * sess, EOFILE * eofile)
{
   struct btnode * last_bip;
   char *   msg;
   ip_addr  last_ip; /* Last IP address displayed */
   int      workflag;
   int      lines;


   /* See if there is a "last IP" parameter passed */
   msg = wi_formipaddr(sess, "nextip", &last_ip);
   if(msg)
   {
      last_bip = 0;
      workflag = TRUE;
   }
   else
   {
      workflag = FALSE;
   }
   lines = def_limit;   /* set numb er of lines to display */
   odd_even = 1;        /* tint first row */

   /* traverse the btree, printing info in lines */
   last_bip = bt_section((struct btnode *)ip_tree, 
      iptree_helper, 
      bt_iptest, 
      (u_long)sess, 
      last_ip, 
      0,
      &workflag,
      &lines );

   /* Print a line with "next n IPs" button, if needed */
   if(last_bip)
   {
      wi_printf(sess, "<td align=center colspan=8 >"
         "<input type=button name=\"more\" value=\"Next %u Entries\" ",
            def_limit);
      wi_printf(sess, " onClick=\"document.location='bipaddr.htm?nextip=%s'\" " 
         " class=\"buttons bblocking\" ></td>\n", 
         print_ipad(((struct control_ip *)last_bip->data)->ci_ipaddr));
   }
   else if(lines == def_limit)
   {
      wi_printf(sess, "<td align=center colspan=8 >No matching items found.</td>");
   }

   return 0; /* OK return */
}

/* set_defip_cgi()
 *
 * CGI routine.
 *
 * Read the 4 bytes of the filter for blobked IP address display.
 * 
 * Returns bitmask (see manual). 
 */

char *
set_defip_cgi(wi_sess * sess, EOFILE * eofile)
{
   int	err;		/* numeric parse errors */
   int	ipbyte1;		/* to be set from form */
   int	ipbyte2;		/* to be set from form */
   int	ipbyte3;		/* to be set from form */
   int	ipbyte4;		/* to be set from form */

	/* auto-generated code: */

	err = wi_formint(sess, "ipbyte1", (long*)&ipbyte1);
	if(err)
      ipbyte1 = 256;
	err = wi_formint(sess, "ipbyte2", (long*)&ipbyte2);
	if(err)
      ipbyte2 = 256;
	err = wi_formint(sess, "ipbyte3", (long*)&ipbyte3);
	if(err)
      ipbyte3 = 256;
	err = wi_formint(sess, "ipbyte4", (long*)&ipbyte4);
	if(err)
      ipbyte4 = 256;

   def_ipbytes[0] = ipbyte1;
   def_ipbytes[1] = ipbyte2;
   def_ipbytes[2] = ipbyte3;
   def_ipbytes[3] = ipbyte4;

   wi_redirect(sess, "bipaddr.htm");
   return NULL;
}

/************* POP list routines ****************/

/* listpop() - helper routine fo
 *
 * SSI routines listing POP user info.
 */

void
listpop(wi_sess * sess, struct   pop_client * tmp, char * htclass)
{
   wi_printf(sess, "<tr class=\"email\" height=1 ><td colspan=5 ></td></tr>\n");
   wi_printf(sess, "<tr class=\"%s\" height=19 ><td>%c</td>\n",
      htclass, (tmp->pflags & POPF_FILTER)?'Y':'N' );
   wi_printf(sess, "<td><a href=\"popuser.htm?pcl=%d\" >%s</a></td>",
      tmp, tmp->pcl_name );
   wi_printf(sess, "<td >%s</td>\n",
      br_getnametext(tmp->pcl_client->ipaddr));
   wi_printf(sess, "<td >%s</td>\n", 
      br_getnametext(tmp->pcl_server) );
   wi_printf(sess, "<td>%s</td></tr>",  
      (tmp->pflags & POPF_APOP)?"secure (APOP)":"unsecured" );
}

/* ht_pcllist()
 *
 * SSI routine.
 *
 * Print the POP client list
 *
 * Returns 0 if OK. 
 */

int
ht_pcllist(wi_sess * sess, EOFILE * eofile)
{
   struct   pop_client * tmp;
   int      line = 0;

   if(pop_clients == NULL)
   {
      wi_printf(sess, "<tr><td class=\"emailtint\" align=center colspan=5 >"
         "No POP clients detected</td></tr>");
      return 0;
   }
   /* Loop though pop client list */
   for(tmp = pop_clients; tmp; tmp = tmp->next )
   {
      /* alternate colors */
      if(line++ & 1)
         listpop(sess, tmp, "blanktint" );
      else
         listpop(sess, tmp, "emailtint" );
   }

   return 0; /* OK return */
}


/* ht_pcluser()
 *
 * SSI routine.
 *
 * Returns 0 if OK.
 */

int
ht_pcluser(wi_sess * sess, EOFILE * eofile)
{
   struct   pop_client * pcl;

   pcl = (struct pop_client *)atoi(wi_formvalue(sess, "pcl"));
   if(pcl)
      listpop(sess, pcl, "emailtint");

   return 0; /* OK return */
}


/* ht_cip()
 *
 * SSI routine.
 *
 * Returns 0 if OK. 
 */

int
ht_cip(wi_sess * sess, EOFILE * eofile)
{
   char *   err;
   ip_addr  ipaddr;
   struct control_ip * cip;
   char *   args;

   args = wi_formvalue(sess, "arg");
   if(!args)
   {
      dtrap();
      return ENP_PARAM;
   }

   /* If there is no form data, then just make an empty form */
   if(sess->ws_formlist == NULL)
      return 0;         /* not an error */

   err = wi_formipaddr(sess, "ip", &ipaddr);
   if(err)
   {
      wi_printf(sess, "(unknown)");
      return 0;
   }

   cip = ipe_lookup(htonl(ipaddr));
   if(!cip)
   {
      dtrap();
      return -1;
   }

   while(*args == ' ')
      args++;

   switch(*args)
   {
   case 'i':   /* IP address */
      ipaddr = htonl(cip->ci_ipaddr);
      wi_printf(sess, "%u.%u.%u.%u", PASS_IPADDR(ipaddr));
      break;
   case 'd':   /* dns name */
      if(cip->ci_name)
         wi_printf(sess, cip->ci_name->cu_name);
      else
         wi_printf(sess, "(none)");
      break;
   case 't':   /* timeout */
      if(cip->ctl.timeout )
         wi_printf(sess, "%d", (cip->ctl.timeout - cticks)/MINUTE );
      else
         wi_printf(sess, "none");
      break;
   default:
      dtrap();
      return -1;
   }
   
   return 0; /* OK return */
}

/* setcip_cgi()
 *
 * CGI routine.
 * Returns bitmask (see manual). 
 */

char *
setcip_cgi(wi_sess * sess, EOFILE * eofile)
{
   char *   msg;           /* IP address parse error msg */
   ip_addr  ipaddr;        /* to be set from form */
   char *   dnsname;       /* to be set from form */
   struct control_ip * cip;
   struct control_dn * dnp;
   long     timer;         /* tmp access time, in minutes */
   char *   cp;
   char *   reason;

   /* auto-generated code: */

   msg = wi_formipaddr(sess, "ipaddr", &ipaddr);
	if(msg)
		return(msg);

   /* Extra check for empty IP address field */
   if(ipaddr == 0)
   {
		return("msg");
   }

   dnsname = wi_formvalue(sess, "dnsname");   /* OK if this is NULL */

   reason = wi_formvalue(sess, "reason");
   if(reason == NULL)
   {
		return("No motive button was checked");
	}


   cp = wi_formvalue(sess, "timer"); /* may be "none" or NULL */
   if(cp)
      timer = atoi(cp); /* If "none", timer will == 0 */
   else
      timer = 0;

   /* make the new (or get existing) controlled IP entry */
   cip = new_ipe(ipaddr, NULL, CIF_USER, &local_comm);

   /* The user may be changing the IP address, so set the 
    * addresses specified by the user into the structure.
    */
   cip->ci_ipaddr = htonl(ipaddr);
   ctl_setaccess(&cip->ctl, timer, cip); /* set access values */

   cip->ctl.offense = mot_byname(reason);
   if(cip->ctl.offense == NULL)
   {
      dtrap();       /* programming error? */
      cip->ctl.offense = &motives[0];  /* use 1st motive, try to recover */
   }

   if(dnsname && (*dnsname > ' '))     /* see if a DNS name was specifified */
   {
      struct control_ip * tmp;

      /* see if this DNS name already esists */
      dnp = dn_lookup(dnsname);
      if(!dnp)    /* not there, make a new entry for the dns name */
      {
         dnp = new_dn(dnsname, &local_comm);
         if(!dnp)
         {
            dtrap();
      		return("out of memory");
         }
         /* clone control structure from IP struct into new dns struct */
         MEMCPY(&dnp->ctl, &cip->ctl, sizeof(cip->ctl) );
      }
      /* see if IP block is already associated with DNS entry */
      for(tmp = dnp->cu_cip; tmp; tmp = tmp->ci_dnlist)
      {
         if(cip == tmp)
            break;
      }
      if(!tmp)    /* cip was not in associated IP list */
         br_insertcip(dnp, cip);
   }

   return("bipaddr.htm");
}

/* ht_urlbkinfo()
 *
 * SSI routine.
 *
 * output the Blocked IP address info a URL named in a form.
 * This takes the following form args:
 *
 * name - Qualified DNS name of entry
 * ipaddr - IP address - "(unknown)", or IP address "x.x.x.x"
 * replace - replacement URL, blank if none
 * timer - access override timeout if any, else blank
 *
 * Returns 0 if OK.
 */

int
ht_urlbkinfo(wi_sess * sess, EOFILE * eofile)
{
   char * dnname;    /* Domain name from name/value pairs */
   struct control_dn * dnp;
   char *   args;

   args = wi_formvalue(sess, "arg");
   if(!args)
   {
      dtrap();
      return ENP_PARAM;
   }

   /* If there is no form data, then just make an empty form */
   if(sess->ws_formlist == NULL)
      return 0;         /* not an error */

   dnname = wi_formvalue(sess, "url");
   if(!dnname)
   {
      /* no passed url also means make empty form */
      return 0;
   }
   dnp = dn_lookup(dnname);
   if(!dnp)
   {
      wi_printf(sess, "(FORM ERROR)" );
      dtrap();
      return -1;
   }

   switch(*args)
   {
   case 'n':
      wi_printf(sess, dnname );
      break;
   case 'i':      /* print IP address */
      ht_docip(sess, dnp->cu_cip);
      break;
   case 'r':
      wi_printf(sess, "(none)"  );
      break;
   case 't':
      if(dnp->ctl.timeout <= cticks)
         ctl_setaccess(&dnp->ctl, 0, dnp);     /* clear timers */
      else
         wi_printf(sess, "%d", ((dnp->ctl.timeout - cticks)/MINUTE) + 1 );
      break;
   default:
      dtrap();
      return -1;
   }
   return  0;
}


/* ht_list_motives()
 *
 * SSI routine.
 *
 * List motives for blocking a Site or IP  address
 *
 * Returns 0 if OK. 
 */


int
ht_list_motives(wi_sess * sess, EOFILE * eofile)
{
	int      i;
   char *   name;
   char *   msg;
   ip_addr  ipaddr;
   struct bk_motive * motive;
   int      per_row = 3;

   wi_printf(sess, "<table border=0 width=80%c >", '%' );

   motive = NULL;
   if((name = wi_formvalue(sess, "url")) != NULL)
   {
      struct control_dn * dnp;
      dnp = dn_lookup(name);
      if(dnp)
         motive = dnp->ctl.offense;
   }
   else if((msg = wi_formipaddr(sess, "ip", &ipaddr)) == NULL)
   {
      struct control_ip * cip;

      cip = ipe_lookup(htonl(ipaddr));
      if(cip)
         motive = cip->ctl.offense;
   }

   for(i = 0; motives[i].desc != NULL; i++)
   {
      if((i % per_row) == 0)
         wi_printf(sess, "<tr>");
      wi_printf(sess, "<td><input type=radio name=reason value=\"%s\" %s>%s</td>\n",
         motives[i].desc, (&motives[i]==motive)?"CHECKED":"", motives[i].desc );
      if((i % per_row) == 2)
         wi_printf(sess, "</tr>");
   }
   wi_printf(sess, "</table>" );
	return 0; /* OK return */
}




/* editurl_cgi()
 *
 * CGI routine.
 *
 * Returns bitmask (see manual). 
 */

char *
editurl_cgi(wi_sess * sess, EOFILE * eofile)
{
   char *   urlname;    /* to be set from form */
   char *   oldname;    /* name before used edited it */
   char *   replace;    /* to be set from form */
   char *   editurl;    /* to be set from form */
   char *   delurl;     /* to be set from form */
   char *   editip;     /* to be set from form */
   char *   timer;
   char *   reason;
   struct control_dn * dnp;

	/* auto-generated code: */

	urlname = wi_formvalue(sess, "urlname");
	if(urlname == NULL)
	{
		return("bad urlname");
	}

   /* the "replace" URL may be null */
	replace = wi_formvalue(sess, "replace");

   /* try for each of the submit button values. The one present 
    * is the one that was clicked.
    */
	editurl = wi_formvalue(sess, "editurl");
	delurl = wi_formvalue(sess, "delurl");
	editip = wi_formvalue(sess, "editip");

   dnp = dn_lookup(urlname);
   if((!dnp) && (!editurl))
   {
      dtrap();
 	   return("internal error");
   }

   if(editurl)
   {
      timer = wi_formvalue(sess, "timer");    /* optional access timer */
      reason = wi_formvalue(sess, "reason");
      if(!dnp)
      {
         oldname = wi_formvalue(sess, "oldname");
         if(oldname == NULL || (*oldname == 0))
         {
            /* We're adding a new name, not replacing an older one */
            dnp = new_dn(urlname, &local_comm);
            dnp->ctl.offense = mot_byname(reason);
         }
         else  /* we're replacing a pre-existing name */
         {
            dnp = dn_lookup(oldname);     /* get entry based on unedited name */
            if(!dnp || !dnp->cu_name)
          	   return("DNS entry not found");

            npfree(dnp->cu_name);   /* free old name buffer */
            dnp->cu_name = npalloc(strlen(urlname) + 1);    /* get new buffer */
            if(!dnp->cu_name)
       	      return("Out of memory");

            strcpy(dnp->cu_name, urlname);   /* copy in new name */
         }
         /* set timer from form */
         if(timer)
            dnp->cu_time = atoi(timer);
         else
            dnp->cu_time = 0;

         if(reason == NULL)
            reason = "Other";
      }
      else
      {
         if(stricmp(dnp->cu_name, urlname) == 0)
         {
            strcpy(xbuf, "The URL you selected \"");
            strcat(xbuf, urlname);
            strcat(xbuf, "\" is already covered by the blocklist entry \"");
            strcat(xbuf, dnp->cu_name);
            strcat(xbuf, "\". " );
            return(xbuf);
         }
      }

      if(timer)
         ctl_setaccess(&dnp->ctl, cticks + (atoi(timer) * MINUTE), dnp );

      if(strcmp(dnp->ctl.offense->desc, reason) )
      {
         struct bk_motive * newmotive;
         newmotive = mot_byname(reason);
         if(newmotive)
         {
            dnp->ctl.offense = newmotive;
            dnp->ctl.action_mask |= CTF_UPDATE;
            dnp->ctl.comm->cm_flags |= CM_NEWDATA;
         }
      }

      if(replace && (*replace != '('))
      {
         dtrap(); /* more here later */
      }
   }
   else if(delurl)
   {
      TH_LOCK_MUTEX(br_dns_mutex);
      del_dname(dnp);
      TH_UNLOCK_MUTEX(br_dns_mutex);
   }
   else if(editip)
   {
      if((dnp->cu_cip != NULL) &&
         (dnp->cu_cip->ci_ipaddr != 0))
      {
         /* build URL to load IP address edit form */
         sprintf(xbuf, "editbip.htm?host=%u.%u.%u.%u", PASS_IPADDR(dnp->cu_cip->ci_ipaddr) );
         return(xbuf);
      }
      else
      {
         return("Blocked URL Entry has no IP addresses to edit.");
      }
   }

   wi_redirect(sess, "urls.htm");
   return NULL;
}


/* Blacklist CGI routines */

/* bl_ip_cgi() - CGI routine to blacklist IP address (or dns name).
 *
 * Returns bitmask (see manual).
 */

char *
bl_ip_cgi(wi_sess * sess, EOFILE * eofile)
{
   char *   msg;		         /* IP address parse error msg */
   ip_addr	bad_ip;		      /* IP adress to block */
   struct control_ip *  ipe;  /* return from new_ipe */
   char *   bad_dnsname;      /* Set if we are to block by name */
   struct ctdesc        ctl;  /* for pass to new_ipe() */
   struct control_dn * dnp;   /* DNS name to block (if passed) */
   char *   reason;
   int      i;

   /* Either of both name & IP may be set */
   bad_ip = 0;
   bad_dnsname = NULL;
	msg = wi_formipaddr(sess, "bad_ip", &bad_ip);

   bad_ip = htonl(bad_ip);
   
   bad_dnsname = wi_formvalue(sess, "bad_name");

   /* If neither name nor IP may is set, error. */
   if(msg && (bad_dnsname == NULL))
	{
      return(msg);
   }

   reason = wi_formvalue(sess, "reason");
   if(!reason)
      reason = "other";

   /* look up reason in table */
   for(i = 0; motives[i].code; i++)
      if(stricmp(motives[i].desc, reason) == 0)
         break;

   if(motives[i].code == 0)      /* not found */
   {
      dtrap();     /* nuts */
      return("motives");
   }

   MEMSET(&ctl, 0, sizeof(ctl)); 
   ctl.action_mask = CTF_BLOCK;
   ctl.offense = &motives[i];

   if(bad_dnsname)
   {

      /* see if this DNS name is already in blacklist */
      dnp = dn_lookup(bad_dnsname);
      if(!dnp)    /* not there, make a new entry for the dns name */
      {
         dnp = new_dn(bad_dnsname, &local_comm);
         if(!dnp)
         {
            dtrap();
      		return("out of memory");
         }
      }
      /* copy control information into control_dn structure */
      MEMCPY(&dnp->ctl, &ctl, sizeof(ctl));

      /* make sure IP address for newly blocked name is known */
      if(dnp->cu_cip == NULL)
      {
         /* If IP is known, attach it to the name */
         if(bad_ip != 0)
         {
            dnp->cu_cip = new_ipe(bad_ip, &ctl, CIF_USER, &local_comm);
            if(dnp->cu_cip->ci_name == NULL)
            {
               dnp->cu_cip->ci_name = dnp;      /* set back pointer */
            }
         }
      }
      else     /* Add IP Address to name's existing IP address list */
      {
         ipe = new_ipe(bad_ip, &ctl, CIF_USER, &local_comm);   /* Get/Make blocked IP entry */
         if(ipe->ci_name == NULL)      /* see if we can associate it with the name */
         {
            dtrap();
            br_insertcip(dnp, ipe);
         }
      }
      wi_redirect(sess, "urls.htm");       /* display blocked URL list */
   }
   else     /* No DNS name, just add IP to blacklist */
   {
      /* add the IP address to blacklist. */
      ipe = new_ipe(bad_ip, &ctl, CIF_USER, &local_comm);
      if(ipe == NULL)      /* not found */
      {
          dtrap();
          return("out of memory");
      }
      wi_redirect(sess, "bipaddr.htm");    /* display blocked IP list */
   }
   return NULL;
}

/* bl_port_cgi() - CGI routine to blacklist ports. Form input may also
 * contain an IP address to assiciate with the blacklisted port.
 *
 * Returns bitmask (see manual).
 */

char *
bl_port_cgi(wi_sess * sess, EOFILE * eofile)
{
   char *	bad_port;		/* to be set from form */
   char *   msg;
   ip_addr  bad_ip;
   unshort  port;
   struct ip_hostentry * iphost;
   struct control_ip * ipe;
   struct control_port * cpp;

	/* auto-generated code: */

	bad_port = wi_formvalue(sess, "bad_port");
	if(bad_port == NULL)
	{
		return("bad bad_port");
	}

   msg = wi_formipaddr(sess, "bad_ip", &bad_ip);
	if(msg)
	{
		return(msg);
	}

   if(bad_ip)
   {
      iphost = br_ipanyif(bad_ip);
      if(!iphost)
      {
           dtrap();    /* should not happen */
		   return("IP not in database");
      }
   }
   else
      iphost = NULL;

   /* block the port and maybe the host */
   port = (unshort)atoi(bad_port);
   if(iphost)
   {
      /* make the new (or get existing) controlled IP entry */
      ipe = new_ipe(iphost->ipaddr, NULL, CIF_USER, &local_comm);
      ipe->ci_port = port;   /* Add the port to the entry */
      ipe->ctl.offense = &motives[0];
      wi_redirect(sess, "bipaddr.htm");       /* display blocked IP list */
   }
   else
   {
      /* see if this port is already blocked */
      cpp = cpt_lookup(port, CPF_TCP);
      if(!cpp)
      {
         cpp = new_cp(port,CPF_TCP, &local_comm);
         cpp->ctl.offense = &motives[0];
      }
   }

   return NULL;
}

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


