/*
 * brmain.c - C "main" file for windows app version of Buster/Agent.
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 *
 *
 * ROUTINES: 
 *  br_putkey(), br_kbhit(), br_getkey(), 
 *  br_LaunchBrowser(), br_chgpower(), br_lanchanged(), 
 *  br_brcallback(), perr(), brWinHook(), winchar(), winstr(), 
 *  br_consoleWnd(), br_SetTrayIcon(), WinMain(), dputchar(), 
 *  dprintf(), panic(), br_winexit(), 
 *  br_popup(), br_beep(), br_flash(), br_flashTick(), 
 *  br_portthreads(), br_getlogtime(), br_getlogdate(), 
 *  br_portrunning(), br_getdate(), br_newimage(), ug_done(), 
 *  br_user_alert(), br_getdir(), 
 *  br_opencustom(), 
 */


#include "busterport.h"   /* Praemio */

#include "resource.h"    /* Windows */
#include <shellapi.h> 
#include <direct.h> 
#include <time.h>

#include "brnet.h"   /* Praemio */
#include "ip.h"
#include "brdb.h"
#include "brmenus.h"



/* Set expiration date for some eval releases. 
 *   Undef BR_EXPIRATION_DATE to disable.
 *
 *   Remember, 0-based months (2 = march)
 */


#define UTF_2008    1199174400      // UTF at Jan 1, 2008
#define UTF_DAY     86400           // Seconds in a day
#define UTF_MONTH   ((UTF_DAY*30) + (UTF_DAY/2))  // Average - 30.5 days

#define BR_EXPIRATION_DATE \
  ((UTF_2008)       +\
   (UTF_MONTH * 3) +\
   (UTF_DAY * 1))

#undef BR_EXPIRATION_DATE // disable.

#ifdef NNSQUAD
extern char * NN_version;
#endif         /* NNSQUAD or not */      

/* globals in this file */
HWND           consoleWnd;    /* window for current console, if any */
HWND           messageWnd;    /* message handler window */
HHOOK          hook;
OSVERSIONINFO  osVer;         /* for GetVersionEx() */ 
DWORD          dwVersion;     /* derivitives from osVer */
DWORD          dwWindowsMajorVersion;
HINSTANCE      brInstance;
HANDLE         hIconMenu;


static   NOTIFYICONDATA tnd;


#define  QKEYS 64
char     keyq[QKEYS];
int      putqkey, getqkey;

#define ICON_MESSAGE       WM_USER+1
#define ICON_ID_SYSTRAY    1
u_long   TrayIconBlinks;
u_long   TrayIconLast;

char errbuf[512];

static   char *   trayicon;      /* name of icon to use in sys tray */
static   int      flash_level;   /* level passed to br_flash() */

int br_system_shutdown = 0;     /* nonzero when shutting down */

char * cmdline;

/* Index (1 - n) of NDIS driver selected in windrv.c. This is only 
 * important in windows systems with more than one useable 
 * driver, usually laptops. This is set at startup to iface + 1, or to -1 
 * if system should "auto select" based on recent traffic. 
 */
int   ndis_ix = 0;

int   alarm_windows = 0;
int   max_alarm_windows = 8;
u_long cticks;

int   http_port = 88;      /* default web server port */

void  br_consoleWnd(void);

extern   int    br_winmain(void);
extern   void   wintrap();
extern   int    nd_prep(void);
extern   int    br_startdb(void);
extern   int    br_startnet(void);
extern   int    t_errno(long socket);

void    winchar(int c);
void    winstr(char * s);

extern  char * prompt;  /* in brmenus.c */


/* task struct we'll wrap arounf WinMain() thread. "entry" and 
 * "handle" members will be null in this structure, so be wary 
 * of it. 
 */

HANDLE   to_winmain;

struct br_thread mainWinThread = 
{
   &to_winmain,
   "WinMain()",
   NULL,
   APP_PRIORITY,
   APP_STACK_SIZE,
};


extern  int      total_threads;

extern struct br_thread mainWinThread;



/* The next three routines and "edit" control  keyboard IO. These implement 
 * kbhit() and getchar().
 */

/* br_putkey() - keystroke from a WM_CHAR message is inserted into queue. */

void
br_putkey(int key)
{
    if(key == 8)        /* backspace */
    {
        if(putqkey < 1)
            return;
        winchar(key);   /* echo backspace to display */
        putqkey--;      /* remove last char from buf */
        return;
    }

   keyq[putqkey++] = key;
   if(putqkey >= QKEYS)
      putqkey = 0;
   if(putqkey == getqkey)    /* buffer overflow? */
   {
      putqkey--;
      /* dtrap? later... -JB- */
   }
   else
       winchar(key);    /* echo the char */

   /* If we got a return (Enter) send command to menu system */
   if(key == '\r')
   {
       if(putqkey > 1)
       {
           keyq[putqkey - 1] = 0;   /* null terminate cmd string */
           br_menucmd(NULL, keyq);
       }
       else
           winstr(prompt);
       keyq[0] = 0; /* clear string */
       putqkey = 0; /* clear char counter */
   }
   return;
}

/* console window kbhit() work-alike */

int
br_kbhit()
{
   return(putqkey != getqkey);
}

/* br_getkey(0 - getchar() for Expert console */

int
br_getkey()
{
   int key;

   if(putqkey == getqkey)
      return -1;

   key = keyq[getqkey++];
   if(getqkey >= QKEYS)
      getqkey = 0;

   return key;
}


void
br_LaunchBrowser()
{
   char  uri[64];

   /* build a URI to the initial screen */
   if(http_port != 80)
      sprintf(uri, "http://127.0.0.1:%u", http_port );
   else
      sprintf(uri, "http://127.0.0.1" );

   /* launch a browser */
   ShellExecute(messageWnd, "open", uri, NULL, NULL, 0 );
}

/* br_chgpower() - Handle windows message about power changes.
 *
 * On JB's toshiba laptop (cycling through hibernation) we get 
 * 8 messages with lParam always 0 and
 * wParam of: 0,0,4,4,7,7,18,18
 */

void
br_chgpower(long lParam, UINT wParam)
{
   dprintf("WM_POWER message; long: %lu, short :%u\n", lParam, wParam);
   switch(wParam)
   {
      case PBT_APMRESUMEAUTOMATIC:  /* Operation resuming automatically after event.  */
         USE_ARG(lParam);           /* This is harmless here... */
      case PBT_APMRESUMECRITICAL:   /* Operation resuming after critical suspension.  */
      case PBT_APMRESUMESUSPEND:    /* Operation resuming after suspension. */
         /* Restart all network event flags except wakeup flag */
         br_powerresume();
         break;
      case PBT_APMSUSPEND:             /* System is suspending operation. */
         br_powerdown_mode = br_mode;  /* save interrupted mode */
         br_downtime = br_gettime();   /* save time we went offline */
         br_setmode(OFFLINE);          /* broadcast that we're going away */
         break;
      default:
         break;
   }
}

/* br_lanchanged()
 * 
 * portability routine - called if 
 * network changed - complain to user, ask to clear DB. 
 */

void
br_lanchanged()
{
   int   ok;

   ok = MessageBox(NULL,
      "The system has apparently changed LANs. Please press OK \n"
      "to clear the database. You may use Cancel to try to go \n"
      "ahead with the old LAN database, but if you are on another \n"
      "LAN this may disrupt it.\n",
      "Restart for changed LAN",
      MB_OKCANCEL);

   if(ok == 1)    /* User clicked OK */
   {
      br_cleardbs();
   }

   return;
}


/* br_brcallback() - callback for the bug report dialog */

BOOL CALLBACK
br_brcallback(  HWND hwndDlg,  /* handle to dialog box */
      UINT uMsg,     /* message */
      WPARAM wParam, /* first message parameter */
      LPARAM lParam  /* second message parameter */
   )
{
   int   wNotifyCode;
   int   wID;

   switch(uMsg)
   {
   case WM_INITDIALOG:
      break;
   case WM_COMMAND:        /* 0x0111 - got a click on a dialog box control */
      wNotifyCode = HIWORD(wParam); // notification code 
      wID = LOWORD(wParam);         // item, control, or accelerator identifier 
      if(wID == 1)               /* OK button */
      {
         dtrap();
      }
   default:
      return 0;
   }


   return 1;
}

/* br_portsuspendresume()
 *
 * Portability routine, called when the receiver is suspecnded 
 * or resumed.
 *
 */

void
br_portsuspendresume(int disabled)
{
   int err;

   if(disabled)
   {
      tnd.uFlags = NIF_ICON|NIF_TIP; 
      strcpy(tnd.szTip, br_appname);
      strcpy(tnd.szTip, " (Suspended)");
      tnd.hIcon = LoadIcon(brInstance, MAKEINTRESOURCE(IDI_ICON6) );
      err = Shell_NotifyIcon(NIM_MODIFY, &tnd);
   }
   else
   {
      tnd.uFlags = NIF_ICON|NIF_TIP; 
      strcpy(tnd.szTip, br_appname);
      tnd.hIcon = LoadIcon(brInstance, MAKEINTRESOURCE(IDI_ICON3) );
      err = Shell_NotifyIcon(NIM_MODIFY, &tnd);
   }
   if(err == 0)
      dtrap();

   return;
}


/* perr() - pop-up a windows message box with information about a windows
 * system error; then terminate program. Sort of a windows-error "panic"
 */

void
perr(int code, char * message)
{
   int err;
   char titlebuf[50];

   if(code != 0)
      return;

   err = GetLastError();
   if(err < _sys_nerr)
      sprintf(errbuf, "%s, GetLastError: %s\n", message, _sys_errlist[err] );
   else
      sprintf(errbuf, "%s, GetLastError: %d\n", message, err );

   sprintf(titlebuf, "%s Startup Problem", br_appname);
   MessageBox(NULL, errbuf, titlebuf,  MB_OK );
   br_winexit(code, 0);
}


#define PERR(code, msg) perr((int)code, (char*)msg)

/* brWinHook() - Handler for messages for the console window */

LRESULT CALLBACK 
brWinHook(int code, unsigned int wparam, long lparam)
{
   int         err;
   char *      cp;
   CWPSTRUCT * cs;
   HANDLE      hData;
//   HWND        dlg;     /* Bug report dialog window */

   cs = (CWPSTRUCT*)lparam;

   if(cs->message == WM_POWERBROADCAST)
   {
      br_chgpower(cs->lParam, cs->wParam);
   }
   if(cs->hwnd == consoleWnd)
   {
      switch(cs->message)
      {
      case WM_CLOSE:         /* 0x010 */
         consoleWnd = NULL;      /* use null handle as flag */
         break;
      case WM_COMMAND:
      case WM_KEYUP:
      case WM_SETCURSOR:
      case WM_ERASEBKGND:
         return 0;
      case WM_PASTE:          /* 0x0302 */
         if(OpenClipboard(NULL))
         {
            hData = GetClipboardData(CF_TEXT);
            if(hData)
            {
               char far * lptr;
               lptr = GlobalLock(hData);
               while(*(char*)lptr)
               {
                  br_putkey((int)(*((char*)lptr)++));
               }
               GlobalUnlock(hData);
               CloseClipboard();
               return -1;
            }
            CloseClipboard();
         }
         break;
      default: 
         break;
      }
   }
   else     /* Must be for the message window */
   {
      if(cs->message == ICON_MESSAGE)   /* systray Icon event? */
      {
         if(cs->wParam == 1)
         {
            if(cs->lParam == 0x201)    /* Left click on icon */
            {
               br_LaunchBrowser();  /* launch a browser, pointed at our GUI */
            }
            else if(cs->lParam == 0x204)    /* Right click on icon */
            {
               int      menucode;
               POINT    pt;

               /* display pop-up menu over the system tray */
               GetCursorPos(&pt);
               SetForegroundWindow(cs->hwnd);
               menucode = TrackPopupMenuEx(hIconMenu, 
                  TPM_RIGHTALIGN | TPM_BOTTOMALIGN | TPM_HORIZONTAL |
                  TPM_RETURNCMD | TPM_LEFTBUTTON | TPM_NONOTIFY ,
                  pt.x, pt.y, cs->hwnd, NULL );

               switch(menucode)
               {
               case ID_LAUNCHWEBGUI:
                  br_LaunchBrowser();  /* launch a browser, pointed at our GUI */
                  break;
               case ID_EXPERTCONSOLE:
                  br_consoleWnd();     /* launch the debug console */
                  break;
               case ID_BUGREPORT:
                  err = DialogBox(brInstance, 
                     MAKEINTRESOURCE(IDD_BUGREPORT), 
                     messageWnd, br_brcallback );
                  if(err == -1)
                  {
                     err = GetLastError();
                     cp = strerror(err);
                  }
                  break;
               case ID_HALTBUSTER:
                  PostMessage(messageWnd, WM_QUIT, 0, 0);   /* kill ourself */
                  break;
               case ID_SUSPEND10MIN:
                  if(br_suspended)
                     br_suspendresume(0);  /* clear suspend state */
                  else
                     br_suspendresume(cticks + (10 * MINUTE));  /* set suspend state */
                  break;
               default:
                  err = GetLastError();
                  break;
               }
            }
         }
         else
         {
   if((cs->message < 0x400) || (cs->message > 0x500))
   {
      err = cs->wParam;
   }
         }
         return 0;
      }
   }
   err = CallNextHookEx(hook, code, wparam, lparam);
   return err;
}

/* winchar() - putchar for the windows console */

void
winchar(int c)
{
   if(consoleWnd)
      PostMessage(consoleWnd, WM_CHAR, c, 0);

   return;
}

void
winstr(char * s)
{
   if(consoleWnd)
   {
      while(*s)
         winchar(*s++);
   }
   return;
}


/* br_consoleWnd() - create a console window. Does nothing if
 * the window already exists (only one at a time is supported).
 */

void
br_consoleWnd()
{
   HWND  desktop;
   HICON hicon;
   RECT  rect;
   int   width, height;
   char  titlebuf[50];

   if(consoleWnd)  /* console already open? */
      return;

   /* figure out the screen dimensions */
   desktop = GetDesktopWindow();
   GetWindowRect(desktop, &rect);
   width = ((rect.right - rect.left) * 2)/3;
   height = ((rect.bottom - rect.top) * 2)/3;
   
   /* Make a transient console window. */
   sprintf(titlebuf, "%s Console", br_appname);
   consoleWnd = CreateWindow("EDIT", titlebuf, 
      WS_OVERLAPPEDWINDOW | WS_VSCROLL | ES_MULTILINE | ES_AUTOVSCROLL,
      200, 200, width, height, 
      NULL, NULL, brInstance, NULL);
   PERR( consoleWnd, titlebuf);

   /* Select an Icon and pretty it up */
   if(osVer.dwMajorVersion >= 5 && osVer.dwMinorVersion >= 1)  /* XP and up? */
      hicon = LoadIcon(brInstance, MAKEINTRESOURCE(IDI_ICON8) );
   else  /* Win2000 and down */
      hicon = LoadIcon(brInstance, MAKEINTRESOURCE(IDI_ICON9) );

   PostMessage(consoleWnd, WM_SETICON, ICON_BIG, (long)hicon);
   PostMessage(consoleWnd, EM_SETSEL, 0, 30);
   PostMessage(consoleWnd, WM_CLEAR, 0, 0);

   sprintf(promptbuf, "%s> ", br_appname);
   winstr(prompt);

   /* display the console */
   ShowWindow(consoleWnd, SW_SHOWNORMAL);
}

/* Code parameter to br_setTrayIcon(code) */
#define  TI_INITIAL  1
#define  TI_NORMAL   2
#define  TI_FLASH    3

int
br_SetTrayIcon(int code)
{
   int   err;
   int   ficon;    /* Resource ID for flash image icon. */
//   int   iwdth = GetSystemMetrics(SM_CXICON);

   switch(code)
   {
   case TI_INITIAL:
      tnd.cbSize = sizeof(NOTIFYICONDATA);
      tnd.hWnd = messageWnd;
      tnd.uCallbackMessage = ICON_MESSAGE;
      tnd.uID = ICON_ID_SYSTRAY;
      tnd.uFlags = NIF_MESSAGE|NIF_ICON|NIF_TIP; 
      strcpy(tnd.szTip, br_appname);
      tnd.hIcon = LoadIcon(brInstance, MAKEINTRESOURCE(IDI_ICON3) );
      PERR(tnd.hIcon, "LoadIcon: icon");
      err = Shell_NotifyIcon(NIM_ADD, &tnd);
      break;
   case TI_NORMAL:      /* used to return from "flash" state */
      if(br_suspended)
         return 0;
      tnd.uFlags = NIF_ICON;
      tnd.hIcon = LoadIcon(brInstance, MAKEINTRESOURCE(IDI_ICON3) );
      err = Shell_NotifyIcon(NIM_MODIFY, &tnd);
      break;
   case TI_FLASH:       /* flash icon (display inverted colors image) */
      if(br_suspended)
         return 0;
      if(flash_level > 0)
         ficon = IDI_ICON4;
      else
         ficon = IDI_ICON5;
      tnd.uFlags = NIF_ICON;
      tnd.hIcon = LoadIcon(brInstance, MAKEINTRESOURCE(ficon) );
      err = Shell_NotifyIcon(NIM_MODIFY, &tnd);
      break;
   default:
      panic("br_setTrayIcon");
      err = 0;
   }


   return err;
}



/**************** MAIN ********************/

char     homedir[MAXPATHNAME];

extern u_long  br_instance;

int APIENTRY 
WinMain(HINSTANCE hInstance,
      HINSTANCE hPrevInstance,
      LPSTR     lpCmdLine,
      int       nCmdShow)
{
   int      err;
   MSG      msg;
   char *   cp;
   char *   regPath;
   HKEY     hoKey;         /* Hokey Microsoft design */
   u_long   datatype;      /* for registry to return data type */
   u_long   buflen;        /* for registry to return data type */
   char msgbuf[300];


   /* Open a (barf) registry entry to find out where we were installed */
#ifdef NNSQUAD
   regPath = "SOFTWARE\\NNSquad\\Network Measurement Agent";
#else
   regPath = "SOFTWARE\\Praemio Inc.\\Buster";
#endif
   
   err = RegOpenKey(HKEY_LOCAL_MACHINE, regPath, &hoKey);
   if(err)
   {
      dtrap();    /* right thing to do? */
      _chdir("\\Program Files\\Praemio");
      strncpy(homedir, "\\Program Files\\Praemio", sizeof(homedir));
   }
   else
   {
      buflen = sizeof(homedir);
      err = RegQueryValueEx(hoKey, "Directory", NULL, &datatype,
         (u_char*)homedir, &buflen);
      if(err)
      {
         _chdir("\\Program Files\\Praemio");
         strncpy(homedir, "\\Program Files\\Praemio", sizeof(homedir));
      }
      else
      {
         _chdir(homedir);
      }
      RegCloseKey(hoKey);
   }

#ifdef BR_EXPIRATION_DATE
   /* See if the current date is past the expiration date */
   if(br_gettime() > BR_EXPIRATION_DATE)
   {
      int pos;    /* position of last char in errbuf */

      sprintf(errbuf, "Trial version '%s' of %s expired on ", 
         br_version, br_appname );
      pos = strlen(errbuf);
      br_writedatetime(BR_EXPIRATION_DATE, errbuf + pos, sizeof(errbuf) - pos);

      strcat(errbuf, "\nPlease download the latest from Praemio's web site\n"
         "       www.praemio.com        ");

      MessageBox(NULL, errbuf, "Evaluation has Expired", MB_OK);
      exit(-1);
   }
#endif /* BR_EXPIRATION_DATE */


   /* save permanent copy of instance and command line */
   cmdline = lpCmdLine;
   br_instance = (u_long)time(NULL);

   /* check if Win32s, if so, display notice and terminate */
   osVer.dwOSVersionInfoSize = sizeof(osVer);
   err = GetVersionEx(&osVer);
   PERR(err, "GetVersionEx");
   if (osVer.dwPlatformId == VER_PLATFORM_WIN32s)
   {
      sprintf(msgbuf, "%s cannot run on older Windows systems .\n"
         "This application will now terminate.",
         br_appname );

      MessageBox(NULL,msgbuf, 
         "Error: Windows 2000 or XP required",  MB_OK );

      USE_ARG(nCmdShow);
      USE_ARG(hPrevInstance);
      goto WinMainExit;
   } 
   /* Set windows version in global dwVersion */
   dwVersion = GetVersion();
   dwWindowsMajorVersion = (DWORD)(LOBYTE(LOWORD(dwVersion)));

   /* see if command line is setting web server port */
   cp = strstr(lpCmdLine, "-p");
   if(cp)
   {
      int   port;
      
      cp += 2;       /* prot number text */
      port = atoi(cp);
      if((port < 1) || (port > 0x0000FFFF))
      {
         MessageBox(NULL, "Web server port must be between 1 and 64K",
            "Command line error", MB_OK);
         goto WinMainExit;
      }
      http_port = port;
   }

   brInstance = hInstance;    /* save for brconsoleWnd() */

#ifdef MAKE_UP_YOUR_MIND
#ifdef NNSQUAD
      sprintf(errbuf, 
         "NNSquad Network Measurement Agent Beta.(%u.%u.%u.%u NNS-B1) \n\n"
         "Note: This will use your network for reverse DNS lookups "
         "and its internal web server on TCP port %d. See \"readme\" file",
         PUSH_VERSION(br_versnum),  http_port);
      err = MessageBox(NULL, errbuf, "NNsquad", MB_OKCANCEL);
      if(err == IDCANCEL)
         return -1;
#endif /* NNSQUAD */
#endif   /* MAKE_UP_YOUR_MIND */


   /* select tray icon based on OS (XP versus older) */
   if(osVer.dwPlatformId >= 2)
      trayicon = "buster.ico";   /* full version for XP tray */
   else
      trayicon = "buster6.ico";  /* 16x16x16 version for older trays */

   /* Make a permanent message handler window. */
   messageWnd = CreateWindow("EDIT", "Buster messages", WS_OVERLAPPEDWINDOW,
      200, 200, 400, 200, 
      NULL, NULL, hInstance, NULL);
   PERR( messageWnd, "messageWindow");

   /* See if we should make an initial expert console window. */
   if(strstr(cmdline, "-e"))
      br_consoleWnd();

   /* build the popup menu for the systray icon */
   hIconMenu = LoadMenu(brInstance, MAKEINTRESOURCE(IDR_MENU1));
   PERR(hIconMenu, "systray menu");
   hIconMenu = GetSubMenu(hIconMenu, 0);

   br_SetTrayIcon(TI_INITIAL);

   hook = SetWindowsHookEx(WH_CALLWNDPROC, brWinHook, 
      NULL, GetCurrentThreadId() );

   err = br_winmain();     /* Set up Windows port (threads & mutex) */
   perr(!err, "br_main failed\n");

   br_startdb();  /* set up databases & config */

   /* Windows prot should only affect local machine, not others */
   br_cfg.cflags |= BC_HOSTONLY;

   br_log(0, "********* WinMain: startup; cmdline=\"%s\"**********\n", lpCmdLine);

   /* Build a winthreads entry for the WinMain() thread */
   TH_NEWTASK(&mainWinThread);
   mainWinThread.state = TS_WINMSG;
   mainWinThread.ThreadId = GetCurrentThreadId();

   /* Now that the netmain task has had a chance to read the NV parms, get
    * The windows machine name and paste it over the systemname. 
    */
   regPath = "SYSTEM\\ControlSet001\\Control\\ComputerName\\ComputerName";
   err = RegOpenKey(HKEY_LOCAL_MACHINE, regPath, &hoKey);
   if(err)
   {
      dtrap();
   }
   else
   {
      buflen = sizeof(systemname);
      err = RegQueryValueEx(hoKey, "ComputerName", NULL, &datatype,
         (u_char*)systemname, &buflen);
      if(err)
      {
         dtrap();    /* What - cry in our beer? */
      }
      RegCloseKey(hoKey);
   }

   err = nd_prep();
   if(err)
   {
       char * cferr = "Can't attach to Windows network drivers.\n";
       br_log(0, cferr);
       perr(!err, cferr);
   }

   br_startnet();  /* set up network activity */

   /* now loop here for the duration; handling Windowsey stuff */
   while( (err = GetMessage( &msg, NULL, 0, ICON_MESSAGE )) > 0 )
   {
      mainWinThread.state = TS_RUNNING;

      switch(msg.message)
      {
      case  WM_CLOSE:         /* 0x010 */
         err = 5;
         if(msg.hwnd == consoleWnd)    /* how did this get here? Squirreley...*/
            consoleWnd = NULL;
         break;
      case  WM_QUIT:
         err = 6;
         break;
      case  WM_SYSCOMMAND:    /* 0x112 */
         err = 7;
         break;
      case  WM_POWERBROADCAST:
         br_chgpower(msg.lParam, msg.wParam);
         break;

#ifdef NOTDEF
      case WM_DEVICECHANGE:
         dprintf("Windows WM_DEVICECHANGE: wParam: %u, lParam: %lx\n",
            msg.wParam, msg.lParam);
         switch (msg.wParam)
         {
         case 0x8000:   /* DBT_DEVICEARRIVAL: */
         case 0x8004:   /* DBT_DEVICEREMOVECOMPLETE: */
            break;
         }
         break;
#endif /* NOTDEF */

      case  ICON_MESSAGE:
         err = 6;
         break;
      case  WM_CHAR:
         if(msg.lParam == 0)  /* It's an echo of our putchar */
            break;
         else
         {
            switch(msg.wParam)
            {
            /* Intercept Cut & Paste chars */
            case 0x03:     /* (ctl-C) */
            case 0x16:     /* (ctl-V) */
            case 0x18:     /* (ctl-X) */
               break;      /* let system handle these */
            default:
               __try
               {
                     br_putkey(msg.wParam);
               }
               __except( RecordExceptionInfo(GetExceptionInformation(), "br_putkey") )
               {
                  exit(-1);
               }

               continue;   /* Don't process message or char will echo to cons */
            }
         }
         break;
      default:
         cp = cp;
         break;
      }
      TranslateMessage( &msg );
      DispatchMessage( &msg );
      mainWinThread.state = TS_WINMSG;
   }


WinMainExit:

   exit(0);
   return 0;
}


/* some of the porting routines for windows follow */

void
dputchar(int c)
{
   winchar(c);
}


int dp_recursion = 0;   /* recursions into dprintf() */

void 
dprintf(char * fmt, ...)
{
   va_list a;
#define  DPFB_SIZE   1512
   char dpfbuf[DPFB_SIZE];

   if(dp_recursion++)
   {
      dp_recursion--;
      return;
   }

   va_start(a, fmt);
   vsprintf(dpfbuf, fmt, a);
   va_end(a);

   if(strlen(dpfbuf) >= DPFB_SIZE)
      panic("dpfbuf");

   winstr(dpfbuf);

   /* If we have free event structures AND 
    * this is not from the menu system AND
    * this is not a prompt, 
    * then send to log file. 
    */
   if((alarms_free.q_len) && (fmt != prompt))
   {
      br_log(0, dpfbuf);
   }
   dp_recursion--;
   return;
}


int bm_recursion = 0;   /* recursions into dprintf() */

int 
bm_printf(void* io, char * fmt, ...)
{
   va_list a;
#define  BMFB_SIZE   512
   char dpfbuf[BMFB_SIZE];

   if(bm_recursion++)
   {
      bm_recursion--;
      return -1;
   }

   va_start(a, fmt);
   vsprintf(dpfbuf, fmt, a);
   va_end(a);

   if(strlen(dpfbuf) >= DPFB_SIZE)
      panic("dpfbuf");

   if(io == NULL)
       winstr(dpfbuf);
   else
   {
       dtrap();     /* Send to file or socket later? */
   }

   bm_recursion--;
   return 0;
}





static int panicing = FALSE;

char * panicmsg = NULL;

void
panic(char * msg)
{
   if(panicing)
      goto do_fault;

   panicing = TRUE;
   dtrap();
   panicmsg = msg;
   br_log(0, "Panic: message: %s\n", msg);

   if(strncmp(msg, "Error reading NV parameters", 27) == 0)
   {
      MessageBox(NULL, 
         "Cannot use the saved options on this machine\n"
         "This may be because of incompatabillity between a new version and an older one.\n"
         "Try uninstalling, erasing the old programm directory, and re-install\n",
         br_appname,
         MB_OK);

      exit(-1);
   }

   /* At this point our best bet is to force an exception and let the
    * exection handling code report the problem.
    */
do_fault:
   msg = 0;             /* Use msg car to make a zero */
   msg = (char*)(7/(int)msg);    /* divide by zero */
   return;
}


/* br_winexit(int code, int restart) - Windows implementation 
 * of PORT_EXIT_FUNC(err); should not return.
 *
 * Will try to restart if restart arg is nonzero.
 */

static int exited = FALSE;

void
br_winexit(int code, int restart)
{

   if(exited)     /* see if this was already entered */
      return;

   exited = TRUE;

   /* Save local db info */
   br_dbwrite(&local_comm);

   /* We may be here as a result of a WM_QUIT to the mesage window,
    * but send another just in case we're not... 
    */
   PostMessage(messageWnd, WM_QUIT, 0, 0);

   /* Get our Icon off the system tray bar */
   Shell_NotifyIcon(NIM_DELETE, &tnd);
   DestroyMenu(hIconMenu);    /* release systray menu's memory */

   /* If restart is set, restart */
   if(restart)
      WinExec(br_appname, SW_SHOWNA);

   exit (code);      /* kill the current thread */
}

static int xpos = 300;
static int ypos = 200;

/* br_brcallback() - callback for the alert dialog */

BOOL CALLBACK
br_dlgcallback(  HWND hwndDlg,  /* handle to dialog box */
      UINT uMsg,     /* message */
      WPARAM wParam, /* first message parameter */
      LPARAM lParam  /* second message parameter */
   )
{
   int   wNotifyCode;
   int   wID;
   struct alarm_type * lastalarm;

   switch(uMsg)
   {
   case WM_INITDIALOG:
      SetWindowPos(hwndDlg, HWND_TOP, xpos += 20, ypos +=20, 0, 0, 
         SWP_SHOWWINDOW|SWP_NOSIZE);

      lastalarm = (struct alarm_type *)lParam;
      SetDlgItemText(hwndDlg, IDC_EDITTYPE, lastalarm->longname );
      SetDlgItemText(hwndDlg, IDC_EDIT1, (char*)(lastalarm->parm));
      return 0;;
   case WM_COMMAND:        /* 0x0111 - got a click on a dialog box control */
      wNotifyCode = HIWORD(wParam); // notification code 
      if(wNotifyCode == 0)
      {
         wID = LOWORD(wParam);         // item, control, or accelerator identifier 
         EndDialog(hwndDlg, wID);
         return wID;
      }
      return 0;
   default:
      return 0;
   }
}

/* alertcrlf()
 *
 * Format passed text to fit in alert dialog box.
 *
 */

#define MAXLINELEN 48

static void
alertcrlf(char * msgbuf, int buflen)
{
   char * cp;
   char * end;
   int   linelen = 0;

   
   end = msgbuf + (buflen - 4);
   for(cp = msgbuf; cp < end; cp++)
   {
      if(linelen++ > MAXLINELEN)
      {
         /* Back up to the last space */
         while((*cp != ' ') && (linelen > 0))
         {
            cp--;
            linelen--;
         }
         if(linelen == 0)
         {
            return;
         }

         memmove(cp + 2, cp, strlen(cp));
         cp++;          /* point past space */
         *cp++ = '\r';  /* Insert CR-LF */
         *cp++ = '\n';
         linelen = 0;
         continue;
      }
      if(*cp >= ' ')
         continue;
      if(*cp == 0)
         break;
      /* convert any CR or LF or pair to "\r\n" */
      if((*cp == '\n') || (*cp == '\r'))
      {
         *cp++ = '\r';
         if(*cp != '\n')
         {
            memmove(cp + 1, cp, strlen(cp));
            *cp++ = '\n';
         }
         linelen = 0;
      }
   }
   *cp = 0;
   return;
}



/* Thread routine for the br_popup call. This does the actual work of 
 * displaying the popup MessageBox and handling the result. 
 */

TH_ENTRY(alarm_popup)
{
   int      dlgcode;    /* return code from Dialog */
   int      msglen;     /* total length of message */
   char *   longmsg;
   char *   errtext;
   struct alarm_type * alarm;

   if(alarm_windows >= max_alarm_windows)
      return 0;

   if(++alarm_windows == max_alarm_windows)
   {
      MessageBox(NULL, 
         "Cannot open any more of these 'alarm' Windows until \n"
         "you close a few old ones by clicking their buttons.\n"
         "App is still recording the alarms in the log.",
         br_appname, 
         MB_OK|MB_ICONWARNING|MB_SETFOREGROUND | MB_APPLMODAL );
      alarm_windows--;
      return 0;
   }

   alarm = (struct alarm_type *)parm;
   errtext = (char*)alarm->parm;
   msglen = strlen(errtext) + 32;  /* few bytes extra for CRLFs */
   longmsg = npalloc(msglen);
   if(!longmsg)
      panic("popup text alloc failed");
   strcpy(longmsg, errtext);

   alertcrlf(longmsg, msglen);
   alarm->parm = longmsg;

   dlgcode = DialogBoxParam(brInstance, 
      MAKEINTRESOURCE(IDD_ALERTDIALOG), 
      messageWnd, br_dlgcallback, (long)alarm );

   npfree(longmsg);
   alarm_windows--;

   if(dlgcode == -1)
   {
      dtrap();
      dlgcode = GetLastError();
      errtext = strerror(dlgcode);
   }

   switch(dlgcode)
   {
   case IDINFO:
      ShellExecute(messageWnd, "open", alarm->al_url, NULL, NULL, 0 );
      break;
   case IDC_DISABLE:
      alarm->actions &= ~AL_POPUP;
      break;
   case IDOK:
   case IDCANCEL: /* clicked on the window frames "x" button */
      break;
   default:
      dtrap();
   }

   return 0;
}
TH_EXIT("alarm_popup")


/* br_popup(char * errtext) - display a "popup" windows  
 * with the passed text in it. Part of the 
 * alarm options system.
 */

static struct alarm_type tmptype;

int
br_popup(struct alarm_type * type, char * errtext)
{
   u_long   Id;

   if(type == NULL)
      type = &tmptype;

   /* Uglyhack - to get text to windows thread, overwrite the
    * alertsparm information. We happen to know the caller is 
    * done with it, sooo...
    */
   type->parm = errtext;

   /* Fire off a seperate windows thread to handle the modal dialog box */
   CreateThread(NULL, 8196, alarm_popup, (void *)type, 0, &Id);
   return 0;
}

int
br_beep()
{
   int err;
   err = MessageBeep(MB_ICONEXCLAMATION);
   return(!err);
}


/* br_flash() - give  a visual warning, eg. blink an LED or (in this
 * case) flash a systray Icon.
 *
 * This is called as follows:
 * br_flash(1, 0); - write to log
 * br_flash(10, 1); - alert
 * br_flash(10, 2); - block (requires FAST return)
 * 
 * Just to docuemnt this somewhere, icons are used as follows:
 * 
 * 
 * IDI_ICON2 - buster4.ico - 32x32 plain desktop
 * IDI_ICON3 - buster6.ico - 16x16, plain systray
 * IDI_ICON4 - icon4.ico - 16x16, yellow (alert)
 * IDI_ICON5 - icon5.ico - 16x16 black background (log)
 */

void
br_flash(int iterate, int level)
{
   /* Raise static level to passed level */
   if(flash_level < level)
      flash_level = level;

   /* If not a block, start the flashing icon */
   if(level != 2)
      br_SetTrayIcon(TI_FLASH);

   /* move ittereation count to max of current or passed */
   if(iterate > (int)TrayIconBlinks)
      TrayIconBlinks = iterate;

   /* Record time we started flashing activity */
   TrayIconLast = cticks;
}

/* br_flashTick() - called from the windows timer in osport.c to drive 
 * the "flashing" of the Sys Tray icon after Busster posts an alert.
 *
 */

void
br_flashTick()
{
   if((TrayIconBlinks > 0) &&
      ((TrayIconLast + (TPS/2)) < cticks))
   {
      if(--TrayIconBlinks & 1)    /* Odd counts show flashed image */
         br_SetTrayIcon(TI_FLASH);
      else
         br_SetTrayIcon(TI_NORMAL);

      TrayIconLast = cticks;

      /* Clear flash level for next round of blinking */
      if(TrayIconBlinks == 0)
         flash_level = 0;
   }
}



/* per-port thread status menu routine */

void
br_portthreads(void * pio)
{
   int      i;
   char *   state;
   int      priority;

   bm_printf(pio, 
"Status of threads\n Name   \t\t state \t\tblocks \tsleeps \twakes \tyields\tpriority \tcrit\n"
   );

   for(i = 0; i < total_threads; i++ )
   {
      switch(winthreads[i]->state)
      {
      case TS_PREINIT:     /* 0 */
         state = "PREINIT\t";
         break;
      case TS_SLEEPING:    /* 1 */
         state = "SLEEPING";
         break;
      case TS_IDLE:        /* 2 */
         state = "IDLE \t";
         break;
      case TS_BLOCKED:     /* 3 */
         state = "BLOCKED ";
         break;
      case TS_RUNNING:     /* 4 */
         state = "RUNNING ";
         break;
      case TS_READY:       /* 5 */
         state = "READY \t";
         break;
      case TS_SOCKSLEEP:   /* 6 */
         state = "SOCK-SLEEP";
         break;
      case TS_WINMSG:      /* 7 */
         state = "GET-MSG ";
         break;
      case TS_NDEVENT1:    /* 8 */
         state = "NDIS-EV1 ";
         break;
      case TS_NDEVENT2:    /* 9 */
         state = "NDIS-EV2 ";
         break;
      case TS_NDRX:        /* 10 */
         state = "NDIS-RX ";
         break;
      case TS_NDSEND:      /* 11 */
         state = "NDIS-TX ";
         break;
      case TS_MUTEXWAIT:   /* 12 */
         state = "MUTEXWAIT ";
         break;
      default:
         state = "BAD-STATE";
         break;
      }

      priority = GetThreadPriority(winthreads[i]->hThread);

      bm_printf(pio, "%8s  \t%s \t%u  \t%u \t%u \t%u \t0x%x \t%d\n", 
         winthreads[i]->name, state, 
         winthreads[i]->blocks,
         winthreads[i]->sleeps,
         winthreads[i]->wakes,
         winthreads[i]->yields,
         winthreads[i]->critsec,
         priority );
   }
}

/* br_getlogtime() - get systems time of day string for.log */

static char dt_buf[DATEBUFLEN]; /* date/time string buffer */

char *
br_getlogtime()
{
   SYSTEMTIME  logtime;

   GetLocalTime(&logtime);
   sprintf(dt_buf, "%d:%d:%d: ", logtime.wHour, logtime.wMinute, logtime.wSecond);
   return(dt_buf);
}


/* br_getlogdate() -  get string with current date for.log */

char *
br_getlogdate()
{
   char *      timezone;
   TIME_ZONE_INFORMATION tz;
   SYSTEMTIME  logtime;


   GetLocalTime(&logtime);
   GetTimeZoneInformation(&tz);

   if (!(dwVersion >= 0x80000000 && dwWindowsMajorVersion >= 4))
   {
      WORD *   unichar;

      /* Windows NT/2000/XP Version */
      /* Convert unicode in local buffer to ascii */
      unichar = (WORD *)(&tz.StandardName[0]);
      timezone = (char*)(&tz.StandardName[0]);
      while(*unichar)
         *timezone++ = (char)(*unichar++ & 0x00FF);

      timezone = (char*)tz.StandardName;      /* reset pointer to buffer */
   }
   else     /* Win 98, ME */
   {
      timezone = (char*)tz.StandardName;      /* just set pointer to ASCII */
   }

   sprintf(dt_buf, "Date: %d-%d-%d, %s", 
      logtime.wMonth, logtime.wDay, logtime.wYear, timezone );

   return(dt_buf);
}


void
br_portrunning()     /* do windoze port initialization */
{
   char * ug_args;      /* upgrade args, if any */
   char * filename;     /* name of this exe */
   char   oldappname[50];
   char   newappname[50];

   ug_args = strstr(cmdline, "-u");
   if(ug_args)
   {
      filename = cmdline;
      *(ug_args - 1) = 0;     /* terminate filename */

      dprintf("Upgrade %s, re-naming exe files\n", br_appname );
      sprintf( oldappname, "%s.old", br_appname);
      sprintf( newappname, "%s.exe", br_appname);

      unlink(oldappname);              /* clean-up any ancient copy */
      rename(newappname, oldappname);  /* rename old.exe */
      rename(filename, newappname);    /* swap us in as new.exe */
   }
   return;
}


/* br_newimage() - portability routine. This receives a new 
 * program image file over the passed socket, conforming to the 
 * passed autoupdate command structure. This routien should 
 * read the file form the socket, save it on the system, 
 * execute it, and ultimatly transfer control to it and 
 * terminate this process.
 *
 * Since this routine is invoked via a network callback and 
 * should return in a timely manner, we don't do all that herein.
 */

//#include "tcpapp.h"

SOCKTYPE  ug_sock = INVALID_SOCKET;

#define  IMBUFSIZE   2000
char *   imgbuf;
FILE *   imgfile;
char *   imgfilename = "tmpster.exe";  /* NOT a long name! */
char     serverfilepath[BR_UDPLEN];
unshort  ug_port;
ip_addr  ug_ipaddr;
u_long   ug_httplen;       /* HTTP header length for upgrade file */
u_long   ug_rxlen;         /* actual length of received upgrade file */

void
br_newimage(struct br_upgrade * autoup)
{

   /* save autoupdate info */
   ug_ipaddr = autoup->ipaddr;
   ug_port = autoup->port;
   strcpy(serverfilepath, autoup->pathname);

   /* allocate the receive file buffer */
   imgbuf = npalloc(IMBUFSIZE);
   if(imgbuf == NULL)
   {
      dprintf("upgrade: alloc error, aborting\n");
      return;
   }

   imgfile = fopen(imgfilename, "wb");
   if(!imgfile)
   {
      dprintf("upgrade: alloc error, aborting\n");
      return;
   }

   return;
}


void
ug_done()
{
   if(ug_sock != INVALID_SOCKET)
      socketclose(ug_sock);
   ug_sock = INVALID_SOCKET;
   if(imgfile)
      fclose(imgfile);
   imgfile = NULL;
   if(imgbuf)
      npfree(imgbuf);
   imgbuf = NULL;
   return;
}



/* br_user_alert() 
 * 
 * Portability routine to send a local operational alert.
 * This should queue the events to alertthe user that the 
 * system has internal problems (e.g. can't send packets).
 * This should return fast and NOT BLOCK!
 */

void
br_user_alert(char * msg)
{
   br_popup(NULL, msg);
   br_beep();
}

/* getdir() - return a list of all files in passed directory. Directory
 * is relative to install directory (or maybe root on applicances)
 *
 * Returned value is NULL if no files found, else a string of filenames
 * seperated by newlines.
 */

char * br_imagedir = "custom";
static char dirlist[2000]; /* buffer for return */

#include <io.h>

char * 
br_getdir(char * dir)
{
   long     handle;
   long     err;
   char *   cp;
   int      len;
   struct _finddata_t   find;

   /* Build directory pattern in dirlist buffer */
   strcpy(dirlist, dir);
   strcat(dirlist, "/*");

   /* find first directory entry matching specification */
   handle = _findfirst(dirlist, &find);
   if (handle == -1)
      return NULL;

   MEMSET(dirlist, 0, sizeof(dirlist));
   err = 0;
   cp = dirlist;

   /* while there's directory entries to process */
   for(err = 0; err == 0; err = _findnext(handle, &find))
   {
      if(find.attrib & _A_SUBDIR)
         continue;

      /* If we're out of room, just return what we have */
      len = strlen(find.name);
      if((len + 1) > (&dirlist[sizeof(dirlist)] - cp))
         return dirlist;

      sprintf(cp, "%s\n", find.name);
      cp += len + 1;
   }
   return dirlist;
}

/* br_opencustom(name) - Called from VFS to try to open one of 
 * on-disk the a custom files. 
 * 
 * Returns FILE case as VFILE if open, else NULL.
 */

void *
br_opencustom(char * name, char * mode)
{
   FILE * fp;

   while(*name == '/')
      name++;

   /* Build full path/name in dirlist buffer */
   strcpy(dirlist, homedir);
   strcat(dirlist, "\\");
   strcat(dirlist, br_imagedir);
   strcat(dirlist, "\\");
   strcat(dirlist, name);
   fp = fopen(dirlist, mode);

   return fp;
}


/* cksum()
 * 
 * Classic "unrolled loop" IP checksum. This one modified from 
 * uncopyrighted sources found on www.codeproject.com.
 *
 */

unshort
cksum(u_char * buf, unsigned len)
{
    unshort sum;

   _asm { mov  esi, buf };      /* load passed parms in regs */
   _asm { mov	ecx, len };     /* halfword count */

   _asm { mov	eax, ecx	 }; /* keep copy of word count */
   _asm { and	ecx, 0FFFFFFE0h };   /* mask count for div/32 */
   _asm { and	eax, 01eh };    /*  get remainder of count %32, masking odd word */
   _asm { shl  eax, 1  };       /* make it a byte count */
   _asm { mov	ebx, 64 };      /* make inverse of (64-remainder) */
   _asm { sub	ebx, eax };     /* 64 - count%64 in BX */
   _asm { sub  esi, ebx };      /* adjust pointer for table offsets */

   /* This next one deviates from the RFC. Since the unrolled loop
    * below starts by pointing 4 byes into the buffer (instead of zero)
    * we adjust pointer back 4 bytes to accomodate this.
    */
   _asm { sub  esi, 4 };
   _asm { mov	eax, ebx };     /* copy of offset into ax */
   _asm { add  eax, ebx };      /* times 2... */
   _asm { add  eax, ebx };      /* times 3 bytes per table opcode */
   _asm { shr  eax, 2 };        /* divide for dwords => sum64 table index */
   _asm { add  eax, offset sum64 } ; /* add to start */

   _asm { mov  edx, len };      /* get word count into DX */
   _asm { and  edx, 1 };        /* make odd-word flag in DX */

   _asm { push eax };           /* put jump address on stack */
   _asm { mov  ebx, esp };      /* ss:ebx now points to jump offset */

   _asm { xor	eax, eax	};  /*  initial value for xsum, clear carry */
   _asm { jmp	dword ptr ss:[ebx] };


sum64:
   /* First line has offset of 4 (not 0) to keep sizes uniform */
   _asm { adc	eax, dword ptr [ESI + 4] };
   _asm { adc	eax, dword ptr [ESI + 8] };
   _asm { adc	eax, dword ptr [ESI + 12] };
   _asm { adc	eax, dword ptr [ESI + 16] };
   _asm { adc	eax, dword ptr [ESI + 20] };
   _asm { adc	eax, dword ptr [ESI + 24] };
   _asm { adc	eax, dword ptr [ESI + 28] };
   _asm { adc	eax, dword ptr [ESI + 32] };
   _asm { adc	eax, dword ptr [ESI + 36] };
   _asm { adc	eax, dword ptr [ESI + 40] };
   _asm { adc	eax, dword ptr [ESI + 44] };
   _asm { adc	eax, dword ptr [ESI + 48] };
   _asm { adc	eax, dword ptr [ESI + 52] };
   _asm { adc	eax, dword ptr [ESI + 56] };
   _asm { adc	eax, dword ptr [ESI + 60] };
   _asm { adc	eax, dword ptr [ESI + 64] };

   _asm { adc  eax, 0 }         /* roll in last carry */

   _asm { jcxz fold_halfword }; /* done with all halfwords? */

   _asm { pushf };              /* save carry flag */
   _asm { sub	ecx, 32 };      /* decrement word count */
   _asm { add  esi, 64 };       /* byte pointer */
   _asm { popf };               /* restore carry flag */
   _asm { jmp	sum64 };        /* goto next loop */

add_halfword:
   _asm { mov  ecx, [ESI + 68] };/* get next word */
   _asm { and  ecx, 0000FFFFh }; /* mask down to first halfword */
   _asm { adc	eax, ecx };       /* add last word */
   _asm { xor  dx,dx };          /* clear odd (half)word flag */

fold_halfword:
   _asm { cmp  edx, 1 };         /* one odd halfword all thats left? */
   _asm { je   add_halfword };   /* go handle odd halfword */
   _asm { mov  ecx, eax };       /* duplicate 32 bit sum */
   _asm { and  ecx, 0000FFFFh }; /* mask to lower 16 bits */
   _asm { rol  eax, 16 };        /* get upper 16 bits in low reg. */
   _asm { and  eax, 0000FFFFh }; /* mask to lower 16 bits */
   _asm { add  ax, cx };         /* add two 16 bit values */
   _asm { adc  ax, 0 };          /* add carry bit */
   _asm { adc  ax, 0 };          /* add carry bit */

   _asm { pop  ebx };       /* even out stack frame */
   _asm { mov  sum, ax };

   return sum;
}


/* br_gettime();
 *
 * Portability routine to get the current time in UTF seconds.
 *
 * On most ports this will be "time(NULL)"
 */

u_long
br_gettime()    
{
    return (u_long)time(NULL);
}


/* br_writedatetime(() 
 *
 * Convert passed 32 bit app time/date stamp 
 * to text, writen at passed pointer.
 *
 * Passed buffer must be at least 16 bytes of free space.
 *
 * returns 0 if OK, else -1;
 */

int
br_writedatetime(u_long time, char * outbuf, int outbuflen)
{
    char * cp;

    if(time == 0)    /* time not set */
       return -1;

    cp = ctime( (long*)&time);
    if(!cp)
        return -1;
    strncpy(outbuf, cp, outbuflen);
    return 0;
}


/* Some stubs for later */

int
smtpalert(char * buf, unsigned len, int (*callback)(char*, int) )
{
    /* Someday, this will email the text out to ta predefined 
     * list of admins. 
     */

    USE_ARG(buf);
    USE_ARG(len);
    USE_ARG(callback);
#ifdef SMTP_ALERTS
    dtrap();
#endif
    return 0;
}

int      smtpalert_check()
{
#ifdef SMTP_ALERTS
    dtrap();
#endif
    return 0;
}


int  br_pop3(struct conn_entry * conn, u_char * data, int dlen,
                ip_addr srcip, ip_addr dstip)
{
    USE_ARG(conn);
    USE_ARG(data);
    USE_ARG(dlen);
    USE_ARG(srcip);
    USE_ARG(dstip);

//    dtrap();
    return 0;
}


static char timestring[48];

char * 
print_seconds(unsigned long total_seconds)
{
   unsigned seconds, minutes, hours, days;

   seconds = total_seconds % 60;
   minutes = (total_seconds/60)%60;
   hours =   (total_seconds/3600)%60;
   days =    (total_seconds/86400)%24;

   if (days)  /* Whole number of days? */
      sprintf(timestring, "%u days, %uh:%um:%us", days, hours, minutes, seconds);
   else if (hours)
      sprintf(timestring, "%u hours, %um:%us", hours, minutes, seconds);
   else
      sprintf(timestring, "%u minutes, %u sec.", minutes, seconds);

   return timestring;
}

static   u_long      start_time;
static   u_long      last_tick = 0xFFFFFFFF;
static   u_long      minslice = 1000;

int   gettimeofday( struct timeval * tvp)
{
   u_long winticks;

   winticks = timeGetTime();  /* milliseconds since windows started */

   /* If ticks wrapped, restart UTF second */
   if(winticks < last_tick)
      start_time = time( NULL );

   if(winticks > last_tick )
   {
      u_long slice = winticks - last_tick;
      if(slice < minslice)
         minslice = slice;
   }
   last_tick = winticks;

   tvp->tv_sec = start_time + (winticks/1000);
   tvp->tv_usec = (winticks%1000) * 1000;

   return 0;
}

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


