/*
 * brpcap.c
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 *
 * Code to do promisc mode and raw sockets-like stuff on Winpcap PACKET.LIB
 *
 */


#define _WINSOCK2API_   1
#define BUSTER 1

#include "busterport.h"
#include "brnet.h"
#include "ip.h"
#include "brdb.h"

#include "resource.h"
#include <shellapi.h> 
#include <direct.h> 
#include <time.h>

#define __MINGW32__ 1   /* required to omit IP v6 in pcap.h */
#include "pcap.h"

#include "brwinnet.h"

#define  RXPKTSIZE   1536


#define  NdisMedium802_3    0
#define  NdisMedium802_11   11

struct alarm_inst no_if_inst;

u_long   brtx_delays;      /* TX delays in Buster context */
u_long   stacktx_delays;   /* TX delays in stack context */
u_long   maxdelay;         /* longest TX delay, in cticks */

/* Array of structures for NDIS ifaces */
struct ndis_iface  ndisifs[MAXNETS];

int nd_init(int iface);     /* attach nets[] entry to ndis */
int nd_send(PACKET);        /* send raw packet */
int nd_close(int iface);    /* detach nets from ndis */

void  nd_stats(void * pio, int iface);   


/* Windows application stuff from brmain.c */
extern   HINSTANCE   brInstance;
extern   HWND        messageWnd;
extern   void        perr(int code, char * message);
extern   HANDLE      netThreadHandle;  /* set in brosport.c */


DWORD WINAPI PacketThreadProc(LPVOID lpParameter);   /* Ndis thread entry point */
int        br_rxpkt(int iface, PACKET rxpkt);        /* Buster rx entry point */

DWORD    NdisThreadId;

/* Flags set to 1 when any thread is in a driver */
int      thread_sending = 0;


/* Callback function for NDIS device selection Dialog box. In 
 * general it should return NONZERO if it processes the message, 
 * and return ZERO (0)  if it does not. 
 */

BOOL CALLBACK
netSelection(  HWND hwndDlg,  /* handle to dialog box */
      UINT uMsg,     /* message */
      WPARAM wParam, /* first message parameter */
      LPARAM lParam  /* second message parameter */
   )
{
#define NONZERO 1
   int   err;
   int   wNotifyCode;
   int   wID;
   int   i;

   switch(uMsg)
   {
   case WM_INITDIALOG:
      SendDlgItemMessage(hwndDlg, IDC_LANLIST, 
            LB_ADDSTRING, 0, (int)"Auto-select the busiest network");
      for (i = 0; i < (int)ifNumber; i++)
      {
         if( !(ndisifs[i].flags & NF_WORKING))
            continue;

         err = SendDlgItemMessage(hwndDlg, IDC_LANLIST, 
            LB_ADDSTRING, 0, (int)ndisifs[i].description);
         if((err == LB_ERRSPACE) || (err == LB_ERR))
            panic("NDIS Dialog listbox");
         ndisifs[i].lbindex = err;
      }
      return 0;
   case WM_COMMAND:        /* 0x0111 - got a click on a dialog box control */
      wNotifyCode = HIWORD(wParam); // notification code 
      wID = LOWORD(wParam);         // item, control, or accelerator identifier 
      if(wID == 1)               /* OK button */
      {
         int   i, current;

         /* Find the selected description */
         current = SendDlgItemMessage(hwndDlg, IDC_LANLIST, LB_GETCURSEL, 0, 0);
         for (i = 0; i < (int)ifNumber; i++)
         {
            if(ndisifs[i].lbindex == current)      /* is this Neo? (the one)? */
            {
               EndDialog(hwndDlg, i);
               return NONZERO;
            }
         }
         EndDialog(hwndDlg, -2);
         return 0;
      }
      else if(wID == 2)         /* CANCEL button */
      {
         EndDialog(hwndDlg, -1);
         return 0;
      }
      return 0;
   default:
      USE_ARG(lParam);
      return 0;
   }
}

static char one_net[MAXDNSNAME + 47] = 
{
   "This computer only has one network adapter:    "
};


/* Callback function for "Too many Net drivers" Dialog box. In 
 * general it should return NONZERO if it fully processes the message, 
 * and return ZERO (0)  if it does not. 
 */

BOOL CALLBACK
maxnetList(  HWND hwndDlg,  /* handle to dialog box */
      UINT uMsg,     /* message */
      WPARAM wParam, /* first message parameter */
      LPARAM lParam  /* second message parameter */
   )
{
   int   err;
   int   wNotifyCode;
   int   wID;
   int   i;

   switch(uMsg)
   {
   case WM_INITDIALOG:
      for (i = 0; i < MAXNETS; i++)
      {
         if(ndisifs[i].description[0] != 0)
         {
            err = SendDlgItemMessage(hwndDlg, IDC_MAXNET_LIST, 
               LB_ADDSTRING, i, (int)ndisifs[i].description);
         }
         else
         {
            err = SendDlgItemMessage(hwndDlg, IDC_MAXNET_LIST, 
               LB_ADDSTRING, i, (int)("No Description"));
         }

         if((err == LB_ERRSPACE) || (err == LB_ERR))
            panic("MACNETS Dialog listbox");
      }
      return 0;
   case WM_COMMAND:        /* 0x0111 - got a click on a dialog box control */
      wNotifyCode = HIWORD(wParam); // notification code 
      wID = LOWORD(wParam);         // item, control, or accelerator identifier 
      switch (wID)
      {
      case  1:               /* Quit button */
         br_log(0, "User quit from MAXNETS dialog");
         EndDialog(hwndDlg, -1); /* return -1 == kill Buster */
         return 0;
      case  2:               /* More info button */
         ShellExecute(messageWnd, "open", 
            "http://www.praemio.info/alerts/maxnets.html", 
            NULL, NULL, 0 );
         EndDialog(hwndDlg, -1);
         break;
      default:
         dtrap();
         break;
      }
      return 0;
   default:
      return 0;
   }
   USE_ARG(lParam);
   return 1;
}


int
br_getNdisMac(struct ndis_iface * nif, char * macbuf)
{
    int                 err;
    PACKET_OID_DATA *   poid;
    char                oidbuf[sizeof(PACKET_OID_DATA) + 6];

    nif->loAdapter = (LPADAPTER)PacketOpenAdapter(nif->name);
    if(!nif->loAdapter)
        return ENP_PARAM;

    poid = (PPACKET_OID_DATA)(&oidbuf[0]);
    poid->Oid = OID_802_3_CURRENT_ADDRESS;
    poid->Length = 6;
    err = PacketRequest(nif->loAdapter, FALSE, poid);

    PacketCloseAdapter(nif->loAdapter);
    nif->loAdapter = 0;

    if(err == FALSE)
    {
        br_log(0, "Packet API MAC address error on %s\n", 
            nif->description);
        return ENP_RESOURCE;
    }
    MEMCPY(macbuf, poid->Data, 6);

    return 0;
}


static	char   errbuf[PCAP_ERRBUF_SIZE];

char * 
br_ReadPcapCards()
{
	pcap_if_t * alldevs;
	pcap_if_t * dev;
    pcap_t *    adcap;
    int         linktype;
    int         keepers = 0;
	
	/* Retrieve the device list */
	if(pcap_findalldevs(&alldevs, errbuf) == -1)
        return errbuf;

	/* scan the list */
	for(dev = alldevs; dev; dev = dev->next)
	{
        strncpy(ndisifs[keepers].name, dev->name, sizeof(ndisifs[keepers].name));
        strncpy(ndisifs[keepers].description, dev->description, 
            sizeof(ndisifs[keepers].description));

        if(strstr(ndisifs[keepers].name, "GenericDialupAdapter"))
            continue;
        if(br_getNdisMac(&ndisifs[keepers], (char*)&ndisifs[keepers].macaddr[0]) < 0)
            continue;

        adcap = pcap_open_live(dev->name,	// name of the device
							 65536,			// portion of the packet to capture. 
							 1,				// promiscuous mode (nonzero)
							 1000,			// read timeout
							 errbuf			// error buffer
							 );
        if(adcap == NULL)
            continue;

        linktype = pcap_datalink(adcap);
        pcap_close(adcap);

        if(linktype != DLT_EN10MB)
            continue;

        keepers++;
    }

    pcap_freealldevs(alldevs);
    ifNumber = keepers;
    return NULL;    /* OK return */
}


/* prep_ndis()
 * 
 *
 * Make a list of NDIS drivers and populate nets[] table and the 
 * ndmacs[] table with the data. Also sets the global ifNumber.
 *
 * Returns number of working interfaces found (0 - n), or negative error.
 */

int
prep_ndis()
{
    int     index;
    char *  cp;
    struct ndis_iface * nif;

    /* Populate ndis table and set ifNumber from pcap */ 
    cp = br_ReadPcapCards();
    if(cp)
    {
        MessageBox(NULL, cp, "Buster: Pcap error", MB_OK);
        return 0;
    }
    if(ifNumber < 1)
    {
        dtrap();    /* Happen anymore? */
    }

    for(index = 0; index < (int)ifNumber; index++)
    {
        nif = &ndisifs[index];

        /* Remove any Microsoft advertising */
        cp = strstr(nif->description, "(Microsoft's Packet Sch");
        if(cp && (cp != nif->description))
            *cp = 0;

        nif->flags |= NF_WORKING;
    }
    return (ifNumber);
}


#define NDIS_PACKET_TYPE_ALL_LOCAL              0x0080
#define RX_MASK   0x00A0      // works for toshiba notebook wireless

void   br_setup_nif(struct ndis_iface * nif)
{
   int   err;
   int filtermask = RX_MASK;

   err = PacketSetHwFilter(nif->loAdapter, filtermask);
   if(err == 0)
   {
       err = GetLastError();
       dprintf("Unable to set promiscuoues NDIS RX filter on %s, Error Code : %1x\n",
          nif->description, err);

       /* Some broken WiFi drivers don't support promiscuous mode. Try to 
        * work with this anyway. Maybe later we should set some kind of 
        * "broken driver" flag.
        */
       filtermask = NDIS_PACKET_TYPE_ALL_LOCAL;
       err = PacketSetHwFilter(nif->loAdapter, filtermask);
       if(err == 0)
       {
           err = GetLastError();
           dprintf("Unable to set non-promisc NDIS RX filter on %s, Error Code : %1x\n",
              nif->description, err);
           return;
       }
    }

    PacketSetMinToCopy(nif->loAdapter, 1);
    PacketSetReadTimeout(nif->loAdapter, 1000);
    PacketSetBuff(nif->loAdapter, 1000);      /* Set buffer, in KBytes */
    return;
}


/* PacketThreadProc() - Entry point for threads which do the NDIS driver
 * receiving. The Buster model creates an additional windows thread 
 * (which runs at a very high priority) for each adapter and does 
 * a blocking receive on the pcap driver. The high priority allows 
 * Buster to respond more quickly to network events. 
 */

TH_ENTRY(PacketThreadProc)
{
    int     iface;
    int     err;
    NET     net;
    struct ndis_iface * nif;
    PACKET      newpkt;
    struct bpf_hdr *  pcaphdr;
    u_long  idle;       /* number of cticks without recieving */

    iface = (int)parm;
    net = nets[iface];
    nif = (struct ndis_iface *)net->n_local;

    errbuf[0] = 0;  /* clear error buffer to detect warning text */
	/* Open the adapter */
    nif->lpAdapter = pcap_open_live(nif->name,	// name of the device
							 65536,	        // portion of the packet to capture. 
							 1,	            // promiscuous mode (nonzero)
							 1,	            // read timeout
							 errbuf	        // error buffer
							 );

    if(nif->lpAdapter == NULL)
	{
		br_log(0, "Unable to open pcap adapter \"%s\": %s\n", 
            nif->description, errbuf);
		dtrap();
        nd_close(iface);
        nif->buster_state = THREAD_FINISHED;
        ExitThread(0);
	}
    if(errbuf[0])
    {
        dtrap();
		br_log(0, "pcap open warning (%s) on adapter \"%s\"\n", 
            errbuf, nif->description);
    }


    /* now raise the "Ndis" receive thread priority. All Buster time-critical 
     * work happens on these high priority threads.
     */
    err = SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS);
    if(err == 0)
    {
        dtrap();
        dprintf("NDIS: can't set priority class, \n");
        ExitThread(0);
    }

    /* Don't let this thread run off until the system has fired up 
     * the NDIS interface fully.
     */
    while(net->ifAdminStatus != 1)
    { 
       Sleep(10);    /* windoze thread sleep call, in milliseconds */
       if(nif->buster_state >= CLOSE_CALLED)
          ExitThread(0);
    }

    /* Don't need high level handle anymore */
    pcap_close(nif->lpAdapter);
    nif->lpAdapter = NULL;
    
    /* Now re-open as low level adapter */
    nif->loAdapter = (LPADAPTER)PacketOpenAdapter(nif->name);
    if(nif->loAdapter == 0)
    {
        dtrap();
        dprintf("NDIS: can't open Packet adapter\n");
        ExitThread(0);
    }

    nif->rxPacket = PacketAllocatePacket();
    if(nif->rxPacket == NULL)
    {
        dprintf("NDIS: can't get pcap Packet\n");
        ExitThread(0);
    }
    nif->txPacket = PacketAllocatePacket();
    if(nif->txPacket == NULL)
    {
        dprintf("NDIS: can't get pcap Packet\n");
        ExitThread(0);
    }

    br_setup_nif(nif);

    idle = cticks;
    newpkt = NULL;

    /* start the capture, hang here for the duration... */
    while(1)
    {
       if(newpkt == NULL)
       {
           newpkt = pk_alloc(0);  /* try to get empty buf */
           if (!newpkt)
           {
               Sleep(100);    /* windoze thread sleep call, in milliseconds */
               nif->no_rxbuf++;
               continue;
           }
           newpkt->pk_buff = nif->rxbuf;
           newpkt->pk_blen = sizeof(nif->rxbuf);
           PacketInitPacket(nif->rxPacket, newpkt->pk_buff, newpkt->pk_blen);
       }

        err = PacketReceivePacket(nif->loAdapter, nif->rxPacket, 0);
        if(err == 0)    /* error - timeout */
           continue;

        if(nif->rxPacket->ulBytesReceived == 0)
        {
            /* If it's been idle a while, kick it */
            if((nif->receives) && (cticks > (idle + (30*TPS))))
            {
                PacketCloseAdapter(nif->loAdapter);
                nif->loAdapter = (LPADAPTER)PacketOpenAdapter(nif->name);
                if(nif->loAdapter == 0)
                {
                   dtrap(); /* lost adapter - now what? */
                }
                br_setup_nif(nif);
            }
            continue;
        }

        nif->receives++;
        idle = cticks;
        newpkt->pk_tstamp = cticks;

        pcaphdr = (struct bpf_hdr *)newpkt->pk_buff;
        newpkt->pk_buff += pcaphdr->bh_hdrlen;
        newpkt->pk_prot = newpkt->pk_buff + MAC_HDRLEN;
        newpkt->pk_plen = pcaphdr->bh_datalen - MAC_HDRLEN;
        newpkt->pk_tstamp = cticks;
        newpkt->pk_net = net;

        net->rxbytes += newpkt->pk_plen;
        if(nif->maxRxBytes < newpkt->pk_plen)
            nif->maxRxBytes = newpkt->pk_plen;

        br_rxpkt(nif->iface, newpkt);  /* frees newpkt */
        newpkt = NULL;                 /* clear local pointer */
    }
	
    PacketFreePacket(nif->rxPacket);
    PacketFreePacket(nif->txPacket);

    nif->buster_state = THREAD_FINISHED;
    ExitThread(0);
    return 0;
}
TH_EXIT("PacketThreadProc")



int
nd_ifsetup(int iniface, int ndiface)
{
   NET      ifp;
   struct ndis_iface * nif;
   int      err;

   ifp = nets[iniface];
   nif = &ndisifs[ndiface];
   nif->iface = iniface;

   /* Save description in generic MIB */
   strcpy(ifp->description, nif->description);

   /* Each NDIS adapter get's it's own Windows thread. Set this up now */
   nif->net_task.tk_ptr = &nif->to_ndis;
   nif->net_task.name = ifp->name;    /* Use net name for task name */
   nif->net_task.entry = PacketThreadProc;
   nif->net_task.priority = NET_PRIORITY;
   nif->net_task.stacksize = APP_STACK_SIZE;

   /* Allocate the Windows thread via TK_ macros */
   nif->net_task.parameter = iniface;
   err = TH_NEWTASK(&nif->net_task);
   if(err)
   {
      dprintf("can't Create thread for iface %d\n", iniface);
      return ENP_RESOURCE;
   }
   nif->net_task.state = TS_READY;
   nif->NdisThread = nif->net_task.tk_ptr;

   return 0;      /* OK return */
}


int
br_ndis2net(int ndis_iface, int iface)
{
   NET ifp;
   int err = 0;

   /* set up a buster net structure for an interface */
   ifp = nets[iface];
   ifp->n_macaddr = (char*)&ndisifs[ndis_iface].macaddr[0];
   ifp->n_flags = 0;

   ifp->ifAdminStatus = 2;   /* status = down */
   ifp->ifOperStatus = 2;    /* will be set to UP in init()  */
   ifp->ifLastChange = cticks * (100/TPS);
 
   ifp->n_init = nd_init;
   ifp->n_send = nd_send;
   ifp->n_close = nd_close;
   ifp->n_stats = nd_stats;

   sprintf(ifp->name, "ndis_%d", ndis_iface);
   ifp->n_local = (void*)&ndisifs[ndis_iface];

   /* Skip actual packet.dll level setup for placeholders */
   if( ! (ndisifs[ndis_iface].flags & NF_PLACEHOLDER))
   {
      dprintf("Checking NDIS device %s\n", ndisifs[ndis_iface].description);
      err = nd_ifsetup(iface, ndis_iface);
   }

   return err;
}

/* nd_prep()
 *
 * Prepare the ndis/pcap system for Buster. This includes the initial call to 
 * ifsetup(), which deternines how many usable interfaces there are in the system
 * and populates the tables.
 *
 * Returns 0 or negative error code. A return of 0 does not mean there are network 
 * interfaces ready, just that no errors were encountered. The number of interfaces 
 * set up in left in the global unsigned "ifNumber".
 */ 

int
nd_prep()
{
   int      err;
   int      iface;
   int      working_ifs = 0;

   /* Call ndis code to list the working ifaces. */
   err = prep_ndis();
   if(err == -1)
   {
      dtrap();       /* We may have gotten rid of this */
      return  -1;    /* Too soon after restart? */
   }

   /* loop through NDIS drivers, setting up a Buster 
    * iface for each one. 
    */ 
   for(iface = 0; iface < (int)ifNumber; iface++)
   {
      if(ndisifs[iface].flags & NF_WORKING)
         working_ifs++;
      if(ndisifs[iface].flags & NF_INITIALIZED)
         continue;
      err = br_setipinfo(iface);
      if(err)
      {
         dprintf("br_setipinfo error %d\n", err);
         continue;
      }

      /* Set up the Buster iface structure */
      err = br_ndis2net(iface, iface);
      if(err)
      {
         dprintf("ndis2net error %d\n", err);
         continue;
      }
      ndisifs[iface].flags |= NF_INITIALIZED;
   }

   if(working_ifs < 1)
   {
      long  uptime;
      uptime = br_gettime() - br_lastresume;
      if(uptime > (1 * MINUTE))
         alarm_general(&no_iface, &no_if_inst, NULL );
   }

    if(ifNumber == 0)   /* If no windows ifaces, return */
    {
        MessageBox(0, "No working networks on this computer yet. If you leave Bbuster "
           "running it will recheck every few seconds.", 
           "Buster", MB_OK);

       return 0;
    }

    if(ifNumber > MAXNETS)
    {
        dtrap();    /* can this still happen? */
        dprintf("ifNumber exceeded:MAXNETS, using first %d\n", MAXNETS);
        ifNumber = MAXNETS;
        return 0;
    }

    /* Lastly. mark the interfacxes as "up" */
    for(iface = 0; iface < (int)ifNumber; iface++)
        nd_init(iface);

    return 0;     /* OK return */
}


int
nd_init(int iface)
{
    if(iface >= (int)ifNumber)
       return ENP_PARAM;

    /* The interface(s) or adapter(s) were opened and initialized in nd_prep logic */
    nets[iface]->ifAdminStatus = 1;   /* Admin status UP */
    if(nets[iface]->n_ipaddr && nets[iface]->n_router)
        nets[iface]->ifOperStatus = 1;    /* operationally UP */
    nets[iface]->ifLastChange = cticks * (100/TPS);

    return 0;
}



int
nd_send(PACKET pkt)
{
    struct ndis_iface * nif;
    int  err;

    /* Check to make sure the iface is installed and ready */
    if(pkt->pk_net->ifOperStatus != 1)
    {
        /* Report error. Note: If it's not ready, there may be no description. */
        if(pkt->pk_net->description[0])
           dprintf("nd_send: iface %s not up\n", pkt->pk_net->description);
        else
           dprintf("nd_send: iface %d not up\n", net_num(pkt->pk_net));
        return -1;
    }

    nif = (struct ndis_iface *)pkt->pk_net->n_local;  /* get struct */

    if(nif->loAdapter == 0)   /* maybe ndis thread is starting */
    {
       Sleep(100);
       if(nif->loAdapter == 0)   /* still? */
       {
//          dtrap(); // It happens, but seems to recover
          return -1;
       }
    }

    thread_sending = 1;
    nif->sends_started++;

    PacketInitPacket(nif->txPacket, pkt->pk_prot, pkt->pk_plen);
    err = PacketSendPacket(nif->loAdapter, nif->txPacket, 0);

    nif->sends_done++;
    thread_sending = 0;
 
    if(err != 1)
    {
        /* complain on the first few errors */
        if(nif->ndis_badsends++ < 10)
            dprintf("NDIS: Send error %d\n", nif->ndis_badsends);
        pk_free(pkt);

        /* This happens a lot coming back from hibernation on windows. In 
         * this don't mark interface down just yet.
         */
        if(br_sysstate != WAKEUP)
           nif_down(nif->iface);

        return -1;
    }

    if(nif->ndis_badsends)
    {
        dprintf("NDIS: send OK\n");
        nif->ndis_badsends = 0;
        nif_up(nif->iface);
    }

    if(*pkt->pk_prot & 0x80)
       pkt->pk_net->txpkts++;
    else
       pkt->pk_net->txpkts++;
    pkt->pk_net->txbytes += pkt->pk_plen;

    pk_free(pkt);
    return 0;
}



int
nd_close(int iface)
{
   u_long   timeout;
   struct ndis_iface * nif;

   if(iface >= (int)ifNumber)
      return ENP_PARAM;

   nif = (struct ndis_iface *)nets[iface]->n_local;

   if(nif->lpAdapter == NULL)
      return ENP_RESOURCE;

   if (nets[iface]->ifAdminStatus == 2)
      return 0;
   nets[iface]->ifAdminStatus = 2;   /* Admin status DOWN */

   nif->buster_state = CLOSE_CALLED;

   pcap_close(nif->lpAdapter);
   nif->lpAdapter = NULL;

   /* Wait for NDIS stuff to terminate */
   if((ndisifs[iface].flags & NF_WORKING) &&
      (nets[iface]->ifOperStatus == 1) &&
      (nets[iface]->ifAdminStatus == 1))
   {
      timeout = cticks + (3 * TPS);
   }
   else     /* short timeout on non-working drivers */
   {
      timeout = cticks + 2;
   }

   while(nif->buster_state == CLOSE_CALLED)
   {
      Sleep(10);
      if(cticks > timeout)
      {
         /* should we kill the thread? */
         dprintf("Close timeout on NDIS device %d - %s\n", 
            iface, nif->description );
         dtrap();
         break;
      }
   }

   nets[iface]->ifOperStatus = 2;

   if(nif->loAdapter)
   {
      PacketCloseAdapter(nif->loAdapter);
      nif->loAdapter = 0;
   }

   if(nif->buster_state == CLOSE_CALLED)
      return ENP_TIMEOUT;
   else
      return 0;
}


void
nd_stats(void * iodev, int iface)
{
   NET      ifp;
   struct ndis_iface * nif;


   ifp = nets[iface];
   nif = (struct ndis_iface *)nets[iface]->n_local;

   bm_printf(iodev, "Device: %s\n", nif->description );

   bm_printf(iodev, " sends; started:%d, done:%d\n",
    nif->sends_started, nif->sends_done );

   bm_printf(iodev, " receive: total:%d\n",
    nif->receives );
   
   bm_printf(iodev, " receive: bad_ip:%d, last_bad:%u.%u.%u.%u\n", 
    nif->bad_ips, PASS_IPADDR(nif->last_bad));
   
   bm_printf(iodev, " resets:%d\n", nif->resets);
   
   bm_printf(iodev, " no_rxbuf:%d Max RXbytes:%d \n",
    nif->no_rxbuf, nif->maxRxBytes );

}

/*
Copyright (C) 2003-2008 by Praemio Inc. All rights reserved.

Author: John Bartas - jbartas@praemio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/




