/*
 * brthreads.c
 *
 * Copyright 2003-2008 by Praemio Inc. All rights reserved.
 * Author: John Bartas - jbartas@praemio.com
 * This software is released under the terms of the LGPL.
 * See LGPL Notice at end of this file; Full license in ../license.txt
 *
 *
 * June 2003-2004 - Our version seperated from Nichetask osport.c
 * 2007 - Modifed to use FreeRTOS definitions instead of Iniche.
 *
 */


#include "busterport.h"   /* Praemio */
#include "brnet.h"
#include "ip.h"
#include "brdb.h"
#include "brmenus.h"


/* a few windows globals */
HANDLE   timerThread;
int      total_threads = 0;
int      killtimer = FALSE;

extern  void        br_flashTick();
extern  u_long      TrayIconBlinks;
extern  brqueue     largebufq;
extern  brqueue     smallbufq;
extern  brqueue     emptybufq;

extern  void        br_check(void);


/* Windows Mutex for this app. This is a Windows CRITICAL_SECTION object
 * wrapped in a local structure.
 */
struct br_mutex_win
{
    CRITICAL_SECTION csection;
    int             locked;     /* 1 when locked, else 0 */
    u_long          locks;      /* count of lock calls */
    u_long          unlocks;    /* count of unlock calls */
    void *          user;       /* back pointer to self */
};

CRITICAL_SECTION THREAD_SECTION;    /* Mutex for thread code */
CRITICAL_SECTION heap_section;

/* Table of all the mutexes (windows crit sections) we use. */

void ** mutexlist[] = 
{
    &largebufq.mutex,
    &smallbufq.mutex,
    &emptybufq.mutex,
    &alarms_free.mutex,
    &alarms_queued.mutex,
    &control_ip_mutex,
    &br_dns_mutex,
    &ce_table_mutex,
    &hosttab_mutex,
    &ip_hosttab_mutex,
    &br_tree_mutex,
    &br_comm_mutex,
};

#define NUM_MUTEXES sizeof(mutexlist)/sizeof(void**)

struct br_mutex_win mutex_structs[NUM_MUTEXES];

char *   MultiappMutexName = "BusterMultiappMutex";
HANDLE   MultiappMutex;    /* Multiple BUster Apps running? */


/* tick counter */
u_long   cticks   =  0; /* need to increment this later - 18 TPS? */

/* array of pointers to task/thread structure, indexed by ThreadId */
struct br_thread * winthreads[MAXTHREADS];


extern   char *   name;

extern   void  br_running(void);
extern   int   prep_win32(int ifaces_found);
extern   int   wd_prep(int);        /* port iface prep routine to install */


/* FreeRTOS portable definitions for thread to handle alerts */
extern   TH_PROTO( alert_handler );

extern struct br_thread mainWinThread;

struct br_thread * netrx_thread;    /* thread for pktDemux */

HANDLE   to_wintimer;
DWORD WINAPI wintime_proc(LPVOID unused);

struct br_thread win32TimerThread = 
{
   &to_wintimer,
   "W32 timer",
   wintime_proc,
   APP_PRIORITY,
   APP_STACK_SIZE,
};


/* start our windows stuff - called from main() in from winapp.c */

int
br_winmain()
{
   int   err;
   int   i;
   struct WSAData version;    /* For pass to WSAStartup */

   /* Check if there are other instances of Buster in the system */
   MultiappMutex = CreateMutex(NULL, FALSE, MultiappMutexName);
   if(MultiappMutex == NULL)
   {
      err = GetLastError();
      br_log(0, "Can't create MultiappMutex, error %d", err);
      return ENP_RESOURCE;
   }
   else  /* got mutex, make sure it's ours */
   {
      if(GetLastError() ==  ERROR_ALREADY_EXISTS)
      {
         MessageBox(NULL, 
            "Application is already running in this Computer",
            br_appname,  MB_OK );
         br_winexit(-1, 0);
         return ENP_RESOURCE;
      }
   }

   for(i = 0; i < NUM_MUTEXES; i++)
   {
      /* Init windows mutex */
      InitializeCriticalSection(&mutex_structs[i].csection);
      /* Set buster portable mutex "void *" pointer to local struct */
      *mutexlist[i] = (void*)(&mutex_structs[i]);
      /* Set backpointer in  local struct */
      mutex_structs[i].user = mutexlist[i];
   }

   /* And a couple of windows port mutii... */
   InitializeCriticalSection(&THREAD_SECTION);
   InitializeCriticalSection(&heap_section);

   SetLastError(0);
   err = WSAStartup( MAKEWORD(2,2), &version);
   if(err)
   {
      dtrap();
      return err;
   }

   return (err);
}

void
br_startcticks()
{
   /* start a windows thread which will pump the timer */
   if(TH_NEWTASK(&win32TimerThread))
      panic("clock_init");

   timerThread = win32TimerThread.hThread;
   return;
}

struct br_thread * winthreads[MAXTHREADS];


/* TH_NEWTASK() - Portability "new task" routine "TH_" macro. 
 * This creates a windows thread for the passed task structure
 * and maps it.
 * 
 * Returns 0 if OK, else error code.
 */

int
TH_NEWTASK(struct br_thread * nettask)
{
   DWORD    Id;
   int      err;

   /* make sure we're not out of entrys in our thread pointer array */
   if(total_threads >= MAXTHREADS)
   {
      // panic("newWinThread: MAXTHREADS");   /* programming error */

      /* panic() is not robust here because the system is not 
       * fully up yet, and other threads run when the message 
       * box displays. Use dtrap/exit instead.
       */
      dtrap();
      exit(-1);
   }

   /* If this is the WinMain() task then we already have a thread; else
    * get one from Windows 
    */
   EnterCriticalSection(&THREAD_SECTION);
   if(nettask != &mainWinThread)
   {
      nettask->hThread = CreateThread( NULL, nettask->stacksize, 
         nettask->entry, (void*)nettask->parameter, 0, &Id);

      if(nettask->hThread == 0)
      {
         LeaveCriticalSection(&THREAD_SECTION);
         return -1;
      }
      /* Save the thread ID for stats, etc.  */
      nettask->ThreadId = Id;
   }
      
   /* Save pointer to this task in master task pointer array */
   winthreads[total_threads] = nettask;

   /* Get a windoze block/wake event for this thread */
   winthreads[total_threads]->winEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
   if(winthreads[total_threads]->winEvent == NULL)
   {
      err = GetLastError();
      LeaveCriticalSection(&THREAD_SECTION);
      dtrap();
      panic("newtask: event");
   }

   total_threads++;
   LeaveCriticalSection(&THREAD_SECTION);
   return 0;
}


/* FUNCTION: lswap()
 * 
 * PARAM1: u_long lval
 *
 * RETURNS: 
 */

u_long
lswap(u_long lval)
{
   _asm { mov  eax, lval }    /* load value to swap */
   _asm { mov  edx, eax }     /* copy for high halfword */
   _asm { xchg al, ah }       /* swap low word */
   _asm { and  eax, 0000FFFFh }  /* clear high bits of low word */
   _asm { shl  eax, 16 }      /* move low halfword to high half of eax */
   _asm { shr  edx, 16 }      /* move high word to low half of edx */
   _asm { xchg dl, dh }       /* swap high word */
   _asm { or   eax, edx }     /* put both halfwords back together */
   _asm { mov  lval, eax }    /* load swapped value for return */

   return lval;
}

int wintime_delay = 0;

TH_ENTRY(wintime_proc)
{
   USE_ARG(parm);

   /* Timing condition: Windows can exec this thread even 
    * before the creating call returns and sets up 
    * "timerThread". If this happens, wait for the 
    * creating thread to catch up 
    */
   if(timerThread == 0)
   {
      wintime_delay++;     /* diagnostic */
      Sleep(100);
   }

   for (;;)
   {
      TH_SLEEP(1);
      cticks++;
      if(cticks%TPS == 1)
      {
         br_check();
      }

      if(killtimer)
      {
         timerThread = 0;
         ExitThread(0);
      }
      if(TrayIconBlinks)
      {
         br_flashTick();
      }
   }
}
TH_EXIT("wintime_proc")

u_long   memtrapsize = 0;

char *   wrap_alloc(unsigned size, void *(*alloc_rtn)(unsigned));
void     wrap_free(char * ptr, void(*free_rtn)(char *));

static int	in_heap = 0;

void *
npalloc(unsigned size)
{
   char *   ptr;  /* pointer to return to caller */

   EnterCriticalSection(&heap_section);
   if(in_heap++)
      panic("heap");

#ifdef MEM_WRAPPERS
   ptr = wrap_alloc(size, malloc);
#else
   ptr = calloc(1, size);
#endif

   if (ptr)       /* zero memory before returning */
      MEMSET(ptr, 0, size);

   in_heap--;
   LeaveCriticalSection(&heap_section);

   return ptr;      
}

void
npfree(char * ptr)
{
   EnterCriticalSection(&heap_section);
   if(in_heap++)
      panic("heap");

#ifdef MEM_WRAPPERS
   wrap_free((char*)ptr, free);
#else
   free(ptr);
#endif

   in_heap--;
   LeaveCriticalSection(&heap_section);
   return;
}



void 
TH_LOCK_MUTEX(void *p)
{
   struct br_mutex_win * mutex = (struct br_mutex_win *)p;

   if((*(int*)mutex->user) != (int)p)
       panic("lock");

   EnterCriticalSection(&mutex->csection);
   mutex->locks++;
   if(++mutex->locked != 1)
       dtrap();
   return;
}

void 
TH_UNLOCK_MUTEX(void *p)
{
   struct br_mutex_win * mutex = (struct br_mutex_win *)p;

   if((*(int*)mutex->user) != (int)p)
       panic("unlock");

   mutex->unlocks++;
   if(mutex->locked-- != 1)
       dtrap();
   LeaveCriticalSection(&mutex->csection);
   return;
}

/* diagnostic helper */
int   win_mutex_locked(void *p)
{
   struct br_mutex_win * mutex = (struct br_mutex_win *)p;
   return(mutex->locked);
}

/* Windows requires a horrible for threads. Their broken 
 * thread API does not implement a real "GetCurrentThread()" 
 * routine. Instead, "GetCurrentThread()" always returns 0xfffffffe, 
 * which can be passed to other API routines to indicate "the 
 * current thread". This is usless for the pupose of calling 
 * ThreadResume(), since the thread we want to resume is not 
 * the current one. Hard to imagine that it ever is. Instead, tcp_sleep()
 * must save the ThreadId, which is unique to the calling thread (as 
 * the GetCurrentThread() handle should have been). In tcp_wakeup()
 * we have to use the ID to search the inet_taskinfo[] table for the
 * handle of the thread we want to awaken. Uhg.
 */


struct br_thread *
br_currentTask()    /* get current TK task entry */
{
   int      i;
   DWORD    thisThreadId;

   /* find the calling thread's table entry: */
   thisThreadId = GetCurrentThreadId();
   for(i = 0; i < total_threads; i++)
   {
      if(winthreads[i]->ThreadId == thisThreadId)
      {
         return(winthreads[i]);
      }
   }
   return NULL;
}


/* In any sane system TH_BLOCK() & TH_WAKE() would be wrappers for
 * SuspendThread() and Resume Thread(), but this is Windows. ResumeThread()
 * fails for arbitrary reasons, and is pretty much useless. The workaround 
 * is to block threads on an event using  WaitForSingleObject(), and let 
 * the thread resume by calling PulseEvent(). 
 */

void
TH_BLOCK()
{
   int   err;
   struct br_thread * tab;

   /* Get our Thread Table entry, which has our block event */
   tab = br_currentTask();
   if(tab == NULL)
   {
      dtrap();    /* serious error */
   }
   tab->state = TS_BLOCKED;
   tab->blocks++;
   err = WaitForSingleObject(tab->winEvent, INFINITE);
   if(err == WAIT_FAILED)
   {
      dtrap();
   }
   return;
}


int
TH_WAKE(long handle)
{
   int   err;
   int   i;

   /* find the wakeup thread's table entry: */
   for(i = 0; i < total_threads; i++)
   {
      if(handle == (long)winthreads[i]->tk_ptr)
      {
         break;
      }
   }
   if(i >= total_threads)
   {
      dtrap();    /* handle not found in task list */
      return -1;
   }
   err = PulseEvent(winthreads[i]->winEvent);
   if(err == 0)
   {
      err = GetLastError();
      dtrap();
      return -1;
   }

   winthreads[i]->state = TS_READY;
   winthreads[i]->wakes++;
   return 0;
}


void
TH_SLEEP(long ticks)
{
   struct br_thread * current;

   current = br_currentTask();   /* get calling thread's info */

   /* Sometimes, during system init, we get here before the calling
    * thread has set up it's entry in winthreads[]. I've trapped this 
    * in the early call to TH_SLEEP() from tk_nettick(). If this 
    * happens early, let it slide; otherwise panic(). -JB-
    */
   if(current == NULL)
   {
      if(cticks > 10)   /* Are we past early startup? */
      {
         panic("TH_SLEEP 2"); /* Yes, panic */
      }
      else
      {
         Sleep(ticks * (1000/TPS));    /* Do sleep w/o stats. */
         return;
      }
   }
   current->state = TS_SLEEPING;
   current->sleeps++;
   Sleep(ticks * (1000/TPS));
   current->state = TS_RUNNING;
}

void
th_yield()
{
   struct br_thread * current;

   current = br_currentTask();
   current->yields++;
   TH_SLEEP(1);
}


void 
wintrap(char * file, int line)
{
    dprintf("warning: trap at file %s, line %d.\n", file, line);
}


int
br_timerset( void (*func)(long parm), long msecs, long parm )
{
    dtrap();
    return -1;
}


/* Portability routine - Wrapper for gethostbyname() */

u_long   br_dnslookup(char * textname)
{
    struct hostent * hosts;

    hosts = gethostbyname(textname);
    if(hosts)
        return *(u_long*)(hosts->h_addr_list[0]);
    else
        return 0L;
}


