/* brwinnet.h
 *
 * Buster Windows networking iface support, including registry and ndis 
 * nonsense.
 *
 */

#ifndef _BRWINNET_H_
#define _BRWINNET_H_ 1

#define  NDIS_RXBUFMAX   10240
#define  NDISNAMELEN     200



#include "packet32.h"   /* For direct acces to "Packet" layer of pcap */

struct bpf_hdr {
	struct timeval	bh_tstamp;	///< The timestamp associated with the captured packet. 
								///< It is stored in a TimeVal structure.
	UINT	bh_caplen;			///< Length of captured portion. The captured portion <b>can be different</b>
								///< from the original packet, because it is possible (with a proper filter)
								///< to instruct the driver to capture only a portion of the packets.
	UINT	bh_datalen;			///< Original length of packet
	USHORT		bh_hdrlen;		///< Length of bpf header (this struct plus alignment padding). In some cases,
								///< a padding could be added between the end of this structure and the packet
								///< data for performance reasons. This filed can be used to retrieve the actual data 
								///< of the packet.
};


/* ndis_state is used to signal buster shutdown */
enum       ndis_state { STARTING, OPEN, CLOSE_CALLED, THREAD_FINISHED };

/* we use an array of these to build a list of NDIS devices in the 
 * system. These map to nets[] indexes, e.g. nets[2] corresponds 
 * to ndismacs[2]
 */

struct ndis_iface
{
   int   selection;           /* 0 for first, etc... */
   int   lbindex;             /* listbox index */
   int   iface;               /* nets[] index */
   char  name[NDISNAMELEN];           /* "name" from registry */
   char  description[NDISNAMELEN];    /* description from registry */


   /* Winpcap has two levels - the "pcap" lever which is similat to Linunx,
    * and the lower "Packet" later to which pcap is mapped on Iwndows. The 
    * upper layer is easier to program, but the lower is fast and has a 
    * few indispensable features, like ability the get MAC address. We
    * need  both, which means keeping two different interface objects.
    */
   pcap_t *    lpAdapter;    /* WinPCAP upper Adapter object */
   LPADAPTER   loAdapter;    /* lower level adapter */

   LPPACKET    rxPacket;      /* a Packet layer Packet for receives */
   LPPACKET    txPacket;      /* another for sending */

   char     rxbuf[1550];      /* receives go in here */

   u_char   macaddr[6];

   /* Support for NDIS iface thread */
   HANDLE   to_ndis;       /* Iniche TK_ object for thread */
   HANDLE   NdisThread;    /* Windows handle for thread */
   struct   br_thread net_task;

   /* Statistics */
   u_long   sends_done;
   u_long   sends_started;
   u_long   receives;   /* total windrv layer receives */
   u_long   bad_ips;    /* receive with dest IP != us or bcast */
   u_long   last_bad;   /* address in last bad_ips */
   u_long   resets;
   u_long   no_rxbuf;      /* failed to get a free buffer for RX */
   u_long   ndis_badsends; /* CONSECUTIVE send errors */
   u_long   maxRxBytes;    /* Most bytes in a single receive operation */

   /* weighted count of recent receive packets */
   u_long   recent_rx;

   struct alarm_inst ifalarm;

   int      flags;         /* NF_ flags bitmask */
   enum     ndis_state buster_state;
};

extern struct ndis_iface  ndisifs[MAXNETS];


struct ndis_iface *  br_ndisByName(char * name);
int   br_ndis2net(int ndis_iface, int iface);

extern   int   ndis_ix;       /* default iface in NV params driver */

#define  IF_AUTOSELECT  -1    /* ndis_ix value for Autoselect */

/* Values for ndis_iface[]->flags */
#define  NF_PLACEHOLDER    0x01     /* this entry is currently a placeholder */
#define  NF_UNUSABLE       0x02     /* device is unusable */
#define  NF_WORKING        0x04     /* Device is usable and working */
#define  NF_INITIALIZED    0x08     /* ndisifs[] is initialized */

extern   struct alarm_type    no_iface;
extern   struct alarm_inst    no_if_inst;

/* Buster windows routines */
extern int     br_ifsetup(void);       /* Initialize faces based on Windows info */
extern int     list_ndis(void);        /* Initial sweep to find all ifaces */
extern int     br_winip(int iface);    /* Extract Windows IP info */
extern char *  select_ndis(int *);     /* Select NDIS to monitor */

extern void    nif_down(int iface);    /* NDIS iface is down */
extern void    nif_up(int iface);      /* NDIS iface is up */


#endif /* _BRWINNET_H_ */
